module.exports = function (grunt) {
    grunt.initConfig({
        pkg: grunt.file.readJSON('package.json'),
        ngAnnotate: {
            options: {
                singleQuotes: true
            },
            dist: {
                files: [{
                    expand: true,
                    src: ['scripts/*.js','scripts/**/*.js', 'views/**/*.js'],
                    ext: '.annotated.js',
                    extDot: 'last'
                }]
            }
        },
        concat: {
            js: {
                src: ['scripts/*.annotated.js','scripts/**/*.annotated.js', 'views/**/*.annotated.js'],
                dest: 'dist/orangeGestor.annotated.js'
            }
        },
        uglify: {
            js: {
                src: ['dist/orangeGestor.annotated.js'],
                dest: 'dist/orangeGestor.min.js'
            }
        },
        copy: {
            main: {
                files: [
                    {
                        expand: true,
                        cwd: 'i18n',
                        src: '**',
                        dest: 'dist/i18n/'
                    },
                    {
                        expand: true,
                        src: 'views/**/*.html',
                        dest: 'dist/'
                    },
                    {
                        expand: true,
                        src: 'index_grunt.html',
                        flatten: true,
                        dest: 'dist/'
                    },
                    {
                        expand: true,
                        src: [
                            'node_modules/bootstrap/dist/css/bootstrap.min.css',
                            'node_modules/font-awesome/css/font-awesome.min.css',
                            'node_modules/angular-toastr/dist/angular-toastr.min.css',
                            'node_modules/metismenu/dist/metisMenu.min.css',
                            'node_modules/chosen-js/chosen.min.css',
                            'node_modules/ladda/dist/ladda.min.css',
                            'node_modules/angular-ui-switch/angular-ui-switch.min.css',
                            'node_modules/fullcalendar/dist/fullcalendar.min.css',
                            'node_modules/angularjs-datepicker/dist/angular-datepicker.min.css',
                            'node_modules/ui-select/dist/select.min.css',
                            'node_modules/components-jqueryui/themes/base/jquery-ui.min.css',
                            'styles/style.css',
                            'styles/custom.css',
                            'styles/noqAltabox.css',
                            'styles/bewin.css',
                            'styles/orange.css',
                            'styles/mobile.css',
                            'styles/tablet.css'

                        ],
                        flatten: true,
                        dest: 'dist/styles/'
                    },
                    {
                        expand: true,
                        src: ['images/*', 'images/**/*'],
                        dest: 'dist/'
                    },
                    {
                        expand: true,
                        flatten: true,
                        src: ['fonts/*','node_modules/bootstrap/fonts/*','node_modules/font-awesome/fonts/*'],
                        dest: 'dist/fonts/',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        flatten: true,
                        src: [
                            'node_modules/components-jqueryui/jquery-ui.min.js',
                            'node_modules/components-jqueryui/ui/i18n/datepicker-es.js',
                            'node_modules/angular-ui-sortable/src/sortable.js',
                            'node_modules/timepicker/jquery.timepicker.min.js',
                            'node_modules/chosen-js/chosen.jquery.min.js',
                            'node_modules/spin.js/spin.min.js',
                            'node_modules/metismenu/dist/metisMenu.min.js',
                            'node_modules/ng-file-upload/dist/ng-file-upload-shim.min.js',
                            'node_modules/angular-i18n/angular-locale_es-es.js',
                            'node_modules/ui-select/dist/select.min.js',
                            'node_modules/angular-smart-table/dist/smart-table.min.js',
                            'node_modules/angular-toastr/dist/angular-toastr.tpls.min.js',
                            'node_modules/angular-ui-bootstrap/dist/ui-bootstrap-tpls.js',
                            'node_modules/angular-chosen-localytics/dist/angular-chosen.min.js',
                            'node_modules/angularjs-datepicker/dist/angular-datepicker.min.js',
                            'node_modules/angular-ui-calendar/src/calendar.js',
                            'node_modules/angular-jquery-timepicker/src/timepickerdirective.min.js',
                            'node_modules/fullcalendar/dist/locale/es.js',
                            'lib/naturalsort/naturalSortVersionDates.js'
                        ],
                        dest: 'dist/libs/',
                        filter: 'isFile'
                    },
                    {
                        expand: true,
                        src: ['*.js','*.css','lang/*','plugins/**',
                            'skins/bootstrapck/*.css','skins/bootstrapck/*.js','skins/bootstrapck/*.png','skins/bootstrapck/images/**'],
                        cwd: 'lib/ckeditor/',
                        dest: 'dist/libs/ckeditor',
                        filter: 'isFile'
                    }
                ]
            }
        },
        clean: {
            annotated: ['./**/*.annotated.js']
        },
        rename: {
            main: {
                files: [
                    {
                        src: ['dist/index_grunt.html'],
                        dest: 'dist/index.html'
                    }
                ]
            }
        },
        copydeps: {
            target: {
                pkg: 'package.json',
                dest: 'dist/libs/',
            }
        }

    });

    grunt.loadNpmTasks('grunt-contrib-concat');
    grunt.loadNpmTasks('grunt-contrib-uglify-es');
    grunt.loadNpmTasks('grunt-ng-annotate');
    grunt.loadNpmTasks('grunt-contrib-copy');
    grunt.loadNpmTasks('grunt-contrib-clean');
    grunt.loadNpmTasks('grunt-copy-deps');
    grunt.loadNpmTasks('grunt-contrib-rename');

    grunt.registerTask('default', ['ngAnnotate', 'concat', 'uglify', 'copy', 'copydeps', 'rename', 'clean']);
};