'use strict';

apiServices.factory('APIAuth', function ($log, $http, $rootScope, RestBeWin, configurationService, sessionService, UserAccessService,
                                         APIUser, APITicket, APIQueue) {

    var api = {};

    api.logout = function () {

        if (sessionService.user || sessionService.user.roles) {

            if (sessionService.getQueue() !== null) {

                var k = _.find(sessionService.user.roles, function (r) {

                    if (r.authority == 'STATION_CAJA_ROLE' || r.authority == 'STATION_ROLE') {
                        return true;
                    } else {
                        return false;
                    }
                });

                if (typeof k != 'undefined' && typeof k.authority != 'undefined'
                    && (k.authority == 'STATION_CAJA_ROLE' || k.authority == 'STATION_ROLE')) {

                    if (typeof sessionService.user.currentWorkProfile != 'undefined') {
                        APITicket.cleanWaitLineForUser(sessionService.getUserId()).then(function (data) {
                            outInService(sessionService.getUserId());
                        });
                    } else {
                        outInService(sessionService.getUserId());
                    }

                } else {
                    outInService(sessionService.getUserId());
                }
            } else {
                UserAccessService.logOut(sessionService.getUserId());
                sessionService.securityToken = null;
                sessionService.logout();
            }
        }
    };

    api.computeAuthUsernameToken = function (username, password) {
        return btoa(username + password);
    };

    api.login = function (username, password, cb, errorcb) {

        $http({
            method: 'POST',
            url: configurationService.apiURL + '/authenticate',
            headers: {
                'X-Auth-Username': api.computeAuthUsernameToken(username, password)
            }
        }).then(function (response) {
            $log.log('auth response>', response);
            sessionService.securityToken = response.data.token;
            RestBeWin.setDefaultHeaders({'X-Auth-Token': sessionService.securityToken});
            response.data.user.roles = response.data.roles;
            var user = response.data.user;
            // comprobamos si el usuario tiene tiendas asignadas y en su caso obtenemos y asociamos esa informacion
            APIQueue.forStationWorker(user.id).then(function (q) {
                user.queues = q;
                sessionService.login(user, user.roles);
                UserAccessService.logIn(user.id, user.currentQueue !== undefined ? user.currentQueue.id : null);
                if (cb) {
                    cb(user);
                }
            });
        }, function (response) {
            $rootScope.laddaLoading = false;

            $log.log('Error in login..');
            $log.log("response>", response);
            if (errorcb) {
                errorcb(response);
            }
        });

    };

    api.asDevice = function (uuid, cb, errCb) {
        $log.log(uuid)
        RestBeWin.setDefaultHeaders({'X-Auth-Username': uuid});

        $http({
            method: 'POST',
            url: configurationService.apiURL + '/authenticate',
            headers: {
                'X-Auth-Username': uuid || ""
            }
        }).then(function (response) {
            $log.log('authenticated response>', response);
            sessionService.securityToken = response.data.token;
            RestBeWin.setDefaultHeaders({'X-Auth-Token': sessionService.securityToken});
            if (cb)
                cb(response);
        }, function (response) {
            $log.log("Error autenticando", response)
            // RestBeWin.setDefaultHeaders(  {} );
            if (errCb) errCb(response);
        });
    }

    api.supportsRole = function (user, role) {
        var retValue = false;

        if (!role) return true;
        if (!user) return false;

        if (!user.roles) return false;
        _.each(user.roles, function (r) {
            if (retValue == true) return;
            if (!r || !r.authority) return;
            retValue = (r.authority == role);
        });

        return retValue;
    };

    var outInService = function (userId) {
        APIUser.outInservice(userId).then(function (data) {
            UserAccessService.logOut(userId);
            sessionService.securityToken = null;
            sessionService.logout();
        });
    };

    return api;
});
