'use strict';

apiServices.factory('APIGeneric', function ($log, RestBeWin, APIBase, $filter) {
    var api = new APIBase('generic');

    var COUNTRY = 1;
    var PROVINCE = 2;
    var CHANNEL = 3;
    var CITY = 4;
    var QUEUE_TAG = 5;
    var USER_ROLE = 6;
    var TICKET_SMS = 10;

    api.byType = function (id) {
        return RestBeWin.all(this.entityClass).one('type', id).getList();
    };

    api.byTypes = function (idArr) {
        return RestBeWin.all(this.entityClass).several('types', idArr).getList();
    };

    api.byParent = function (id) {
        return RestBeWin.all(this.entityClass).one('parent', id).getList();
    };

    api.countries = function () {
        return api.byType(COUNTRY);
    };

    api.provinces = function (countryId) {
        return api.byParent(countryId);
    };

    api.cities = function (provinceId) {
        return api.byParent(provinceId);
    };

    api.channels = function () {
        return api.byType(CHANNEL);
    };

    // por ahora la configuracion de la tienda es una combinacion entre
    // generic tipo 5 (configuracion del kiosko) y generic tipo 10 (configuracion tienda y servicio)
    api.queueConfParams = function () {

        return api.byTypes([QUEUE_TAG, TICKET_SMS]);
        //return api.byType( 5 );
    };

    api.serviceConfParams = function () {
        return api.byType(10);
    };

    api.roles = function () {
        return api.byType(USER_ROLE);
    };

    api.priorityPolicies = function () {
        return api.byType(7);
    };

    api.ticketExtendedData = function () {
        return api.byType(11);
    };

    api.smsMesageConf = function () {
        return api.byType(13);
    };

    return api;

});
