'use strict';

apiServices.factory('APITicket', function ($log, RestBeWin, APIBase, configurationService, utilService, $http, $window) {
    var api = new APIBase('ticket');

    api.markTicketAsServiced = function (ticket) {
        if (ticket.servicedBy) {
            delete ticket.servicedBy.authorities;
        }
        return RestBeWin.one(this.entityClass).one('serviced').customPUT(ticket);
    };

    api.markTicketAsNotPresented = function (ticket) {
        if (ticket.servicedBy) {
            delete ticket.servicedBy.authorities;
        }
        return RestBeWin.one(this.entityClass).one('notPresented').customPUT(ticket);
    };

    api.getNext = function (userId) {
        return RestBeWin.one(this.entityClass).one('serve', userId).post();
    };

    api.findNext = function (userId) {
        return RestBeWin.one(this.entityClass).one('find', userId).post();
    };

    api.callTicket = function (userId, ticketId) {
        return RestBeWin.one(this.entityClass).one('serve', userId).one('ticket', ticketId).post();
    };

    api.recallTicket = function (ticketId) {
        return RestBeWin.one(this.entityClass).one('recall', ticketId).put();
    };

    api.waitingInQueue = function (queueId) {
        $log.log("Searching tickets waiting", queueId);
        return RestBeWin.one(this.entityClass).one('waiting', queueId).get();
    };

    api.waitingInQueueByServicePoint = function (queueId, servicePointId) {
        $log.log("Searching tickets waiting for", queueId);
        return RestBeWin.one(this.entityClass).one('waiting', queueId).one('servicePoint', servicePointId).get();
    };

    api.inServiceInQueue = function (queueId) {
        $log.log("Searching tickets in service for", queueId);
        return RestBeWin.one(this.entityClass).one('serve', queueId).get();
    };

    api.inServiceByUser = function (userId) {
        $log.log("Searching tickets in service for", userId);
        return RestBeWin.one(this.entityClass).one('serve').one('user', userId).get();
    };

    api.waitingAppointmentsForQueue = function (queueId) {
        return RestBeWin.all(this.entityClass).one('waitappointments', queueId).getList();
    };

    api.waitLineForUser = function (userId) {
        $log.log("Searching tickets in service for", userId);
        return RestBeWin.one(this.entityClass).one('waitline', userId).get();
    };

    api.waitRedirectLineForUser = function (userId) {
        $log.log("Searching tickets in redirect for", userId);
        var params = {
            idUser: userId,
            forProfile: 0 //este parametro indica si se buscan tickets redirigidos filtrando por workprofile o no
        };
        return RestBeWin.one(this.entityClass).one('waitredirectline').customPOST(params);
    };

    api.redirect = function (ticket) {
        var parameter = {
            'ticket': ticket,
            'aditionalInformation': null
        };
        return RestBeWin.one(this.entityClass).one('redirect').customPOST(parameter);
    };

    api.issueTicket = function (ticket, firstName, secondName, phoneNumber) {
        $log.log("Issuing ticket ");

        return RestBeWin.one(this.entityClass).one('issue').customPOST(ticket);
    };

    api.callHistoryForQueue = function (idQueue) {
        return RestBeWin.one(this.entityClass).one('callhistory', idQueue).get();
    };

    api.cleanWaitLineForUser = function (userId) {
        return RestBeWin.one(this.entityClass).one('cleanWaitLine', userId).post();
    };

    api.printTicket = function (ticket) {

        $log.log("printTicket with configuration>", ticket.queue.configuration);

        // printerless kiosk
        var c = utilService.byID(ticket.queue.configuration, 6002);
        if (c) {
            $log.log('printerless kiosk')
            return;
        }

        // SO printing
        c = utilService.byID(ticket.queue.configuration, 6004);
        if (c) {
            $log.log('SO printing')
            $window.print();
            return;
        }

        // impresion a traves de la app printer server en tableta ..
        c = utilService.byID(ticket.queue.configuration, 6005);
        if (c) {
            $log.log('app printer')
            $log.log("printing ticket in printer server .. ");
            ticket = JSON.parse(JSON.stringify(ticket));
            ticket.configuration = null;

            var url = configurationService.printerServiceURL;
            if (url) {
                ticket.url = configurationService.ticketURL + ticket.queue.id + '/' + ticket.id;

                $http({
                    method: 'POST',
                    url: url,
                    data: ticket
                }).then(function successCallback(response) {
                }, function errorCallback(response) {
                    alert('Error durante la impresion.');
                    $log.log("error printing ticket>", response);
                });

                return;
            }

        }

        $log.log("No printer configuration set.. no printing done")
    }

    return api;

});
