'use strict';

angular.module('bewin_orange')
    .factory('HttpInterceptor', function ($rootScope) {
        return {
            request: function (config) {
                $rootScope.laddaLoading = true;
                return config;
            },
            response: function (res) {
                $rootScope.laddaLoading = false;
                return res;
            }
        }
    })
    .config(function ($stateProvider, $urlRouterProvider, $translateProvider, $httpProvider) {

        $urlRouterProvider.otherwise('/login');

        $translateProvider.useStaticFilesLoader({
            prefix: 'i18n/',
            suffix: '.json'
        })
            .registerAvailableLanguageKeys(['es', 'en', 'jp'], {
                'en_US': 'en',
                'en_UK': 'en',
                'es_ES': 'es',
                'ja_JP': 'jp'
            })
            .determinePreferredLanguage()
            //.preferredLanguage('jp')
            .fallbackLanguage('en')
            .useSanitizeValueStrategy('escape');

        $httpProvider.interceptors.push('HttpInterceptor');

        $stateProvider
            .state('root', {
                abstract: true,
                url: "/",
                views: {
                    'navigation@': {
                        templateUrl: 'views/common/navigation.html',
                        controller: 'MainCtrl'
                    },
                    'footer@': {
                        templateUrl: 'views/common/footer.html',
                        controller: 'FooterController'
                    }
                }
            })
            .state('admin', {
                abstract: true,
                parent: 'root',
                url: "admin",
                templateUrl: 'views/common/content.html'
            })
            .state('tenant', {
                abstract: true,
                parent: 'root',
                url: "tenant",
                templateUrl: 'views/common/content.html'
            })
            .state('station', {
                abstract: true,
                parent: 'root',
                url: "station",
                templateUrl: 'views/common/content.html'
            })
            .state('reception', {
                abstract: true,
                parent: 'root',
                url: "reception",
                templateUrl: 'views/common/content.html'
            })
            .state('supervision', {
                abstract: true,
                parent: 'root',
                url: "supervision",
                templateUrl: 'views/common/content.html'
            });
    })
    .controller('WarningCtrl', function ($scope, $state, $timeout, $uibModalInstance, sessionService, UserAccessService,
                                         msg, redirectState, entity, reloadTime) {
        $scope.msg = msg;
        $scope.reloadTime = null;

        $scope.accept = function () {
            $uibModalInstance.close();

            if (reloadTime != null) {
                $scope.reloadTime = null;
                UserAccessService.logOut(sessionService.getUserId());
                sessionService.logout();
            }

            if (redirectState && redirectState !== null) {
                $state.go(redirectState);
            }
        };

        var timer = function () {
            if ($scope.reloadTime !== null) {
                if ($scope.reloadTime > 0) {
                    $scope.reloadTime -= 1;
                    $timeout(function () {
                        timer();
                    }, 1000);
                } else {
                    $scope.accept();
                }
            }
        };

        var init = function () {
            if (reloadTime && reloadTime !== null) {
                $scope.reloadTime = reloadTime;
                timer();
            }
        };

        init();
    })
    .controller('DeleteWarningCtrl', function ($scope, $uibModalInstance, msg) {
        $scope.msg = msg;

        $scope.accept = function () {
            $uibModalInstance.close();
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        };
    })
    .factory('RestAppointment', function (configurationService, Restangular) {
        return Restangular.withConfig(function (RestangularConfigurer) {
            RestangularConfigurer.setBaseUrl(configurationService.apiAppointmentURL);
            RestangularConfigurer.setDefaultHeaders({
                'Content-Type': 'application/json'
            });
        });
    })
    .factory('RestPromotions', function (configurationService, Restangular) {
        return Restangular.withConfig(function (RestangularConfigurer) {
            RestangularConfigurer.setBaseUrl(configurationService.apiPromotionURL);
            RestangularConfigurer.setDefaultHeaders({
                'Content-Type': 'application/json'
            });
        });
    })
    .factory('RestBeWin', function (configurationService, Restangular) {
        return Restangular.withConfig(function (RestangularConfigurer) {
            RestangularConfigurer.setBaseUrl(configurationService.apiURL);
            RestangularConfigurer.setDefaultHeaders({
                'Content-Type': 'application/json'
            });
        });
    })
    .factory('RestCapacity', function (configurationService, Restangular) {
        return Restangular.withConfig(function (RestangularConfigurer) {
            RestangularConfigurer.setBaseUrl(configurationService.apiCapacityURL);
            RestangularConfigurer.setDefaultHeaders({
                'Content-Type': 'application/json'
            });
        })
    })
    .run(function ($log, $rootScope, $state, $window, $http, sessionService, modalService, toastr, RestPromotions,
                   RestAppointment, RestBeWin, RestCapacity) {

        var user = $window.sessionStorage.getItem(sessionService.SESSION_USER);
        var token = $window.sessionStorage.getItem(sessionService.SESSION_TOKEN);
        var selectedTenant = $window.sessionStorage.getItem(sessionService.SELECTED_TENANT);

        if (user) {
            sessionService.user = JSON.parse(user);
            sessionService.user.supportsRole = function (r) {
                var k = _.find(sessionService.user.roles, ['authority', r]);

                if (!k || k == undefined)
                    return false;

                return true;
            };
            sessionService.securityToken = JSON.parse(token);

            RestAppointment.defaultHeaders['X-Auth-Token'] = JSON.parse(token);
            RestBeWin.defaultHeaders['X-Auth-Token'] = JSON.parse(token);
        }
        if (selectedTenant) {
            sessionService.setSelectedTenant(JSON.parse(selectedTenant));
        }

        $rootScope.$state = $state;
        $rootScope.laddaStyle = 'expand-right';

        $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams) {
            return needLogin(toState, event);
        });

        $rootScope.$on('$stateChangeSuccess', function () {
            window.scrollTo(0, 0);
        });

        $rootScope.$on("$routeChangeSuccess", function (event, currentRoute, previousRoute) {
            window.scrollTo(0, 0);
        });

        var needLogin = function (toState, event) {

            if ("login" == toState.name)
                return;

            if (sessionService.user && sessionService.user.id) {
                return;
            }

            event.preventDefault();

            return $state.go("login");
        };

        var requestInterceptor = function (element, operation, what, url, headers, params, config) {
            $rootScope.laddaLoading = true;
            return element;
        };

        var responseInterceptor = function (data, operation, what, url, response, deferred) {
            $rootScope.laddaLoading = false;

            var extractedData;
            if (data.achieved === undefined) {
                extractedData = data;
            } else if (data.achieved === true) {
                extractedData = data.data[0];
            } else {
                extractedData = [];
            }
            if (isObject(extractedData) || Array.isArray(extractedData)) {
                extractedData.timeDiff = parseInt(new Date().getTime()) - parseInt(new Date(response.headers('currentdate')).getTime());
            }
            return extractedData;
        };

        var errorInterceptor = function (response, deferred, responseHandler) {
            $log.log("error>", response);

            $rootScope.laddaLoading = false;

            var msg;
            if (response.status === 401) {
                $state.go('login');
                return false;
            } else if (response.status === 428) {
                msg = 'El token de usuario ha caducado. Su sesión se cerrará y será necesario que vuelva a realizar el login. Gracias.';
                modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, 'login', null, 60);
                return false;
            } else if (response.status === -1) {
                msg = 'Error indeterminado. Por favor recargue su aplicación pulsando las teclas CTRL+F5.' +
                    'Si es usuario de una tablet, borre su historial y vuelva a acceder a la aplicación.' +
                    'Si el problema persiste, revise su conexión a la red.';
                modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
                return false;
            } else {
                if (response.status && response.statusText) {
                    $log.log(response.status + ":" + response.statusText);
                    toastr.error("Error:" + response.status + " (" + response.statusText + ")");
                } else if (response.data) {
                    if (response.data.message) {
                        $log.log(response.data.message);
                        toastr.error("Error:" + response.data.message);
                    } else if (response.data.achieved !== undefined && response.data.achieved === false) {
                        $log.log(response.data.details.errorMessage);
                        toastr.error("Error:" + response.data.details.errorMessage);
                    }
                } else {
                    $log.log("No ha sido posible conectar con el servidor.");
                    toastr.error("No ha sido posible conectar con el servidor.");
                }
                return false;
            }
        };

        function isObject(value) {
            return value && typeof value === 'object' && value.constructor === Object;
        }

        RestAppointment.addRequestInterceptor(requestInterceptor);
        RestPromotions.addRequestInterceptor(requestInterceptor);
        RestBeWin.addRequestInterceptor(requestInterceptor);

        RestAppointment.addResponseInterceptor(responseInterceptor);
        RestPromotions.addResponseInterceptor(responseInterceptor);
        RestBeWin.addResponseInterceptor(responseInterceptor);

        RestAppointment.setErrorInterceptor(errorInterceptor);
        RestPromotions.setErrorInterceptor(errorInterceptor);
        RestBeWin.setErrorInterceptor(errorInterceptor);

        RestCapacity.addRequestInterceptor(requestInterceptor);
        RestCapacity.addResponseInterceptor(responseInterceptor);
        RestCapacity.setErrorInterceptor(errorInterceptor);

    });

var apiCapacityServices = angular.module('noq.api.capacity', []);
var apiAppointmentServices = angular.module('noq.api.appointments', []);
var apiPromotionsServices = angular.module('noq.api.promotions', []);
var apiServices = angular.module('noq.api', []);
var services = angular.module('noq.services', []);