'use strict';

services.factory('NavigationServices', function ($q, $state, APITicket, APIUser, modalService, sessionService, toastr) {

    var navigationServices = {};

    navigationServices.modalInstance = {};

    navigationServices.selectCurrentQueue = function (user) {
        if (user.currentQueue !== undefined) {
            sessionService.setQueue(user.currentQueue).then(function (resp) {
                navigationServices.stateGo(user);
            });
            return;
        }

        if (user.queues !== undefined || user.queues != null) {
            if (user.queues.length == 1) {
                sessionService.setQueue(user.queues[0]).then(function (resp) {
                    navigationServices.stateGo(user);
                });
            }
            if (user.queues.length > 1) {
                navigationServices.openModal(user);
            }
        }
    };

    navigationServices.changeCurrentQueue = function () {
        navigationServices.canUserLogout().then(function (response) {
            if (response) {
                navigationServices.openModal(sessionService.getUser());
            }
        });
    };

    navigationServices.openModal = function (user) {
        var msg = 'Seleccione una tienda';
        modalService.open('views/login/queue_modal.html', 'QueueModalController', msg, null, user);
        this.modalInstance = modalService.modalInstance;

        this.modalInstance.result.then(function (result) {
            if (result !== undefined) {
                toastr.info("Tienda seleccionada correctamente", "Operación realizada");
                if (sessionService.getQueueId() !== null) {
                    APITicket.cleanWaitLineForUser(user.id);
                }
                sessionService.setQueue(result.queue).then(function (resp) {
                    navigationServices.stateGo(user);
                });
            }
        });
    };

    navigationServices.closeModal = function () {
        this.modalInstance.close();
    };

    navigationServices.stateGo = function (user) {
        if (user.supportsRole('ADMIN_ROLE')) {
            $state.go('admin.tenantadmin', {}, {reload: true});
            return;
        }

        if (user.supportsRole('TENANT_ADMIN_ROLE')) {
            $state.go('tenant.queue', {}, {reload: true});
            return;
        }

        if (user.supportsRole('RECEPTION_ROLE')) {
            if (user.supportsRole('STATION_ROLE')) {
                if (typeof user.currentQueue != 'undefined' && typeof user.currentStation != 'undefined' && typeof user.currentWorkProfile != 'undefined') {
                    $state.go('station.serve', {
                        queueId: user.currentQueue.id,
                        stationId: user.currentStation.id
                    }, {reload: true});
                } else {
                    $state.go('station.home', {}, {reload: true});
                }
                return;
            } else {
                $state.go('reception.home', {}, {reload: true});
                return;
            }
        }

        if (user.supportsRole('SUPERVISOR_ROLE')) {
            $state.go('supervision.dashboard', {id: sessionService.getQueueId()}, {reload: true});
            return;
        }

        if (user.supportsRole('STATION_CAJA_ROLE') || user.supportsRole('STATION_ROLE')) {
            if (typeof user.currentQueue != 'undefined' && typeof user.currentStation != 'undefined' && typeof user.currentWorkProfile != 'undefined') {
                $state.go('station.serve', {
                    queueId: user.currentQueue.id,
                    stationId: user.currentStation.id
                }, {reload: true});
            } else {
                $state.go('station.home', {}, {reload: true});
            }
        }
    };

    navigationServices.unselectCurrentQueue = function (link) {
        navigationServices.canUserLogout().then(function (response) {
            if (response) {
                if (sessionService.getQueueId() !== null) {
                    APITicket.cleanWaitLineForUser(sessionService.getUserId());
                }
                sessionService.setQueue(null).then(function (resp) {
                    $state.go(link, {}, {reload: true});
                });
            }
        });
    };

    navigationServices.canUserLogout = function () {
        var deferred = $q.defer();

        if (sessionService.isServing) {
            var msg = 'Por favor, cierre el ticket en servicio antes de cerrar la sesión.';
            modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
            deferred.resolve(false);

        } else if (sessionService.getQueueId() === null) {
            deferred.resolve(true);

        } else {
            APIUser.canUserLogout(sessionService.getQueueId(), sessionService.getUserId()).then(function (resp) {
                if (resp == null || resp.length == 0) {
                    deferred.resolve(true);

                } else {
                    var perfiles = '';
                    for (var i = 0; i < resp.length; i++) {
                        if (i === 0) {
                            perfiles = resp[i].name;
                        } else {
                            perfiles = perfiles + ', ' + resp[i].name;
                        }
                    }

                    var msg = 'Por favor, comprueba si existen tickets en espera o redirigidos a ti que no pueden ser atendidos por otro usuario antes de cerrar la sesión en estos perfiles: ' + perfiles;
                    modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
                    deferred.resolve(false);
                }
            });

        }
        return deferred.promise;
    };

    return navigationServices;
});