'use strict';

services.factory('sessionService', function ($q, $window, APIQueue, APIQueueLicenseType, APIUser) {

    var session = {
        user: null,
        securityToken: null,
        selectedTenant: null,
        isServing: false,
        SESSION_USER: 'adminUser',
        SESSION_TOKEN: 'adminToken',
        SELECTED_TENANT: 'selectedTenant'
    };

    session.getTenant = function () {
        if (!session || !session.user) {
            return null;
        }
        return session.user.tenant;
    };

    session.getQueue = function () {
        if (!session || !session.user) {
            return null;
        }
        return session.user.currentQueue;
    };

    session.getUser = function () {
        if (!session) {
            return null;
        }
        return session.user;
    };

    session.getTenantId = function () {
        if (!session || !session.user || !session.user.tenant) {
            return null;
        }
        return session.user.tenant.id;
    };

    session.getQueueId = function () {
        if (!session || !session.user || !session.user.currentQueue) {
            return null;
        }
        return session.user.currentQueue.id;
    };

    session.getUserId = function () {
        if (!session || !session.user) {
            return null;
        }
        return session.user.id;
    };

    session.getStationId = function () {
        if (!session || !session.user || !session.user.currentStation) {
            return null;
        }
        return session.user.currentStation.id;
    };

    session.getWorkProfileId = function () {
        if (!session || !session.user || !session.user.currentWorkProfile) {
            return null;
        }
        return session.user.currentWorkProfile.id;
    };

    session.setUser = function (user) {
        session.user = user;
        $window.sessionStorage.setItem(session.SESSION_USER, JSON.stringify(session.user));
    };

    session.setQueue = function (queue) {
        var deferred = $q.defer();
        if (queue !== undefined && queue !== null && queue.id !== undefined) {

            if (session.getQueueId() !== null && session.getQueueId() === queue.id) {

                APIQueue.loadOne(queue.id).then(function (resp) {
                    session.user.currentQueue = resp.plain();

                    session.setUser(session.user);
                    deferred.resolve(session.user);
                });
            } else {
                APIQueueLicenseType.findByQueue(queue.id).then(function (resp) {

                    APIUser.joinUserQueue(session.user.id, queue.id).then(function (resp1) {
                        var result = resp1.plain();
                        if (typeof result.currentQueue != 'undefined') {
                            session.user.currentQueue = result.currentQueue;
                            session.user.currentQueue.queueLicenseType = resp;
                        } else {
                            delete session.user.currentQueue;
                        }
                        delete session.user.currentStation;
                        delete session.user.currentWorkProfile;

                        session.setUser(session.user);
                        deferred.resolve(resp1);
                    });

                });
            }
        } else {
            session.isQueueLicenseTDB = true;

            APIUser.outInservice(session.user.id).then(function (resp1) {
                delete session.user.currentQueue;
                delete session.user.currentStation;
                delete session.user.currentWorkProfile;

                session.setUser(session.user);
                deferred.resolve(resp1);
            });
        }
        return deferred.promise;
    };

    session.setUserTenant = function (tenant) {
        session.user.tenant = tenant;
        $window.sessionStorage.setItem(session.SESSION_USER, JSON.stringify(session.user));
    };

    session.getUserTenant = function () {
        return session.user.tenant;
    };

    session.setSelectedTenant = function (tenant) {
        session.selectedTenant = tenant;
        $window.sessionStorage.setItem(session.SELECTED_TENANT, JSON.stringify(tenant));
    };

    session.getSelectedTenant = function () {
        return JSON.parse($window.sessionStorage.getItem(session.SELECTED_TENANT));
    };

    session.login = function (v, roles) {
        v.supportsRole = function (r) {
            var k = _.find(session.user.roles, ['authority', r]);

            if (!k || k == undefined)
                return false;

            return true;
        };

        v.roles = roles;
        session.setUser(v);
        $window.sessionStorage.setItem(session.SESSION_TOKEN, JSON.stringify(session.securityToken));
    };


    session.logout = function () {
        session.user = null;
        session.selectedTenant = null;
        $window.sessionStorage.removeItem(session.SESSION_USER);
        $window.sessionStorage.removeItem(session.SESSION_TOKEN);
        $window.sessionStorage.removeItem(session.SELECTED_TENANT);
    };

    return session;
});
