'use strict';

services.factory('TimetableServices', function ($log, modalService, toastr, TimetableAPI, TimetableGroupingAPI,
                                                TIMETABLE_TYPE) {

    var timetableServices = {};

    timetableServices.selectTimetableGrouping = function (queue, form, model) {
        TimetableGroupingAPI.findByQueue(queue.id, TIMETABLE_TYPE.OPEN_CLOSE).then(function (response) {
            timetableServices.parseTimetableGroupingToModel(response, form, model);
        });
    };

    timetableServices.selectTimetableGroupingAppoiment = function (queue, form, model) {
        TimetableGroupingAPI.findByQueue(queue.id, TIMETABLE_TYPE.APPOINTMENT).then(function (response) {
            timetableServices.parseTimetableAppoimentToModel(response, form, model);
        });
    };

    timetableServices.selectGenericTimeTable = function (form, model) {
        if (model.timetableGrouping.extend === undefined || model.timetableGrouping.extend == null) {
            toastr.warning("Recuerde que debe pulsar al guardar para mantener los cambios", "Atención");
        }
        if (model.genericTimetableGrouping !== undefined) {
            form.timetableGrouping.extend = timetableServices.getGenericTimetableGrouping(model);
        }
        model.showSaveTimetable = true;
    };

    timetableServices.selectGenericTimeTableAppoiment = function (form, model) {
        if (model.timetableGroupingAppoiment.extend === undefined || model.timetableGroupingAppoiment.extend == null) {
            toastr.warning("Recuerde que debe pulsar al guardar para mantener los cambios", "Atención");
        }
        if (model.genericTimetableAppoiment !== undefined) {
            form.timetableGroupingAppoiment.extend = timetableServices.getGenericTimetableAppoiment(model);
        }
        model.showSaveAppointments = true;
    };

    timetableServices.selectEspecificTimeTable = function (form, model) {
        form.timetableGrouping.extend = null;
        form.timetableGrouping.timetables = [];
        model.showSaveTimetable = false;
    };

    timetableServices.selectEspecificTimeTableAppoiment = function (form, model) {
        form.timetableGroupingAppoiment.extend = null;
        form.timetableGroupingAppoiment.timetables = [];
        model.showSaveAppointments = false;
    };

    timetableServices.saveTimeTableGrouping = function (form, model) {
        if (form.timetableGrouping.extend != undefined && form.timetableGrouping.extend !== null) {
            form.timetableGrouping.timetables = [];
            form.timeTable = {};
        }
        form.timetableGrouping.queue = form.queue;

        form.timetableGrouping.timeTableType = TIMETABLE_TYPE.OPEN_CLOSE;
        TimetableGroupingAPI.saveWithTimetable(form.timetableGrouping).then(function (resp) {
            toastr.info("Horario guardado correctamente", "Operación realizada");
            timetableServices.selectTimetableGrouping(form.queue, form, model);
        });
    };

    timetableServices.saveTimeTableGroupingAppoiment = function (form, model) {
        if (form.timetableGroupingAppoiment.extend != undefined && form.timetableGroupingAppoiment.extend !== null) {
            form.timetableGroupingAppoiment.timetables = [];
            form.timeTableAppoiment = {};
        }
        form.timetableGroupingAppoiment.queue = form.queue;

        form.timetableGroupingAppoiment.timeTableType = TIMETABLE_TYPE.APPOINTMENT;
        TimetableGroupingAPI.saveWithTimetable(form.timetableGroupingAppoiment).then(function (resp) {
            toastr.info("Horario guardado correctamente", "Operación realizada");
            timetableServices.selectTimetableGroupingAppoiment(form.queue, form, model);
        });
    };

    timetableServices.saveTimetable = function (form, model) {
        form.timetableGrouping.queue = form.queue;
        form.timetableGrouping.timeTableType = TIMETABLE_TYPE.OPEN_CLOSE;

        form.timeTable = timetableServices.parseFormToTimetable(form.timeTable, form.time);
        form.timeTable.timetableGrouping = {id: form.timetableGrouping.id};
        form.timetableGrouping.editTimetable = form.timeTable;

        TimetableGroupingAPI.saveWithTimetable(form.timetableGrouping).then(function (resp) {
            toastr.info("Horario guardado con éxito", "Operación realizada");
            timetableServices.selectTimetableGrouping(form.queue, form, model);
        });
    };

    timetableServices.saveTimetableAppoiment = function (form, model) {
        form.timetableGroupingAppoiment.queue = form.queue;
        form.timetableGroupingAppoiment.timeTableType = TIMETABLE_TYPE.APPOINTMENT;

        form.timeTableAppoiment = timetableServices.parseFormToTimetable(form.timeTableAppoiment, form.timeAppoiment);
        form.timeTableAppoiment.timetableGrouping = {id: form.timetableGroupingAppoiment.id};
        form.timetableGroupingAppoiment.editTimetable = form.timeTableAppoiment;

        TimetableGroupingAPI.saveWithTimetable(form.timetableGroupingAppoiment).then(function (resp) {
            toastr.info("Horario guardado con éxito", "Operación realizada");
            timetableServices.selectTimetableGroupingAppoiment(form.queue, form, model);
        });
    };

    timetableServices.selectTimeTable = function (select, form) {
        form.timeTable = JSON.parse(JSON.stringify(select));
        form.time = timetableServices.parseTimetableToForm(form.time, form.timeTable);
    };

    timetableServices.selectTimeTableAppoiment = function (select, form) {
        form.timeTableAppoiment = JSON.parse(JSON.stringify(select));
        form.timeAppoiment = timetableServices.parseTimetableToForm(form.timeAppoiment, form.timeTableAppoiment);
    };

    timetableServices.removeTimetable = function (timetableId, form, model) {
        $log.log("Borrando > ", timetableId);

        var msg = 'Se va a deshabilitar este horario. ¿Desea continuar?';
        modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
        var modalInstance = modalService.modalInstance;

        modalInstance.result.then(function () {
            TimetableAPI.deleteOne(timetableId).then(function (r) {
                toastr.info("Horario eliminado con éxito", "Operación realizada");

                timetableServices.selectTimetableGrouping(form.queue, form, model);
            });
        });
    };

    timetableServices.removeTimetableAppoiment = function (timetableId, form, model) {
        $log.log("Borrando > ", timetableId);

        var msg = 'Se va a deshabilitar este horario. ¿Desea continuar?';
        modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
        var modalInstance = modalService.modalInstance;

        modalInstance.result.then(function () {
            TimetableAPI.deleteOne(timetableId).then(function (r) {
                toastr.info("Horario eliminado con éxito", "Operación realizada");

                timetableServices.selectTimetableGroupingAppoiment(form.queue, form, model);
            });
        });
    };

    timetableServices.parseTimetableGroupingToModel = function (response, form, model) {
        if (response.length > 0) {
            model.timetableGrouping = response[0];
            form.timetableGrouping = JSON.parse(JSON.stringify(response[0]));

            if (form.timetableGrouping.extend !== undefined) {
                model.showSaveTimetable = true;
                form.timetableGrouping.extend = _.find(model.genericTimetableGrouping, ['id', form.timetableGrouping.extend.id]);
            } else {
                model.showSaveTimetable = false;
            }

            form.timeTable = {};
            form.time = {};
        } else {
            form.timetableGrouping = {};
            form.timetableGrouping.extend = timetableServices.getGenericTimetableGrouping(model);
            form.timeTable = {};
            form.time = {};

            model.showSaveTimetable = true;
        }
    };

    timetableServices.parseTimetableAppoimentToModel = function (response, form, model) {
        if (response.length > 0) {
            model.timetableGroupingAppoiment = response[0];
            form.timetableGroupingAppoiment = JSON.parse(JSON.stringify(response[0]));

            if (form.timetableGroupingAppoiment.extend !== undefined) {
                model.showSaveAppointments = true;
                form.timetableGroupingAppoiment.extend = _.find(model.genericTimetableAppoiment, ['id', form.timetableGroupingAppoiment.extend.id]);
            } else {
                model.showSaveAppointments = false;
            }

            form.timeTableAppoiment = {};
            form.timeAppoiment = {};
        } else {
            form.timetableGroupingAppoiment = {};
            form.timetableGroupingAppoiment.extend = timetableServices.getGenericTimetableAppoiment(model);
            form.timeTableAppoiment = {};
            form.timeAppoiment = {};

            model.showSaveAppointments = true;
        }
    };

    timetableServices.getGenericTimetableGrouping = function (model) {
        var grouping = null;
        for (var i = 0; i < model.genericTimetableGrouping.length; i++) {
            if(model.genericTimetableGrouping[i].predetermined === true){
                grouping = model.genericTimetableGrouping[i];
            }
        }
        if(grouping == null){
            grouping = model.genericTimetableGrouping[0];
        }
        return grouping;
    };

    timetableServices.getGenericTimetableAppoiment = function (model) {
        var grouping = null;
        for (var i = 0; i < model.genericTimetableAppoiment.length; i++) {
            if(model.genericTimetableAppoiment[i].predetermined === true){
                grouping = model.genericTimetableAppoiment[i];
            }
        }
        if(grouping == null){
            grouping = model.genericTimetableAppoiment[0];
        }
        return grouping;
    };

    timetableServices.parseTimetableToForm = function (model, timeTable) {
        model = {};

        if (timeTable !== null) {
            model.openMonday = timetableServices.parseStringtimeToDate(timeTable.openMonday);
            model.openTuesday = timetableServices.parseStringtimeToDate(timeTable.openTuesday);
            model.openWednesday = timetableServices.parseStringtimeToDate(timeTable.openWednesday);
            model.openThursday = timetableServices.parseStringtimeToDate(timeTable.openThursday);
            model.openFriday = timetableServices.parseStringtimeToDate(timeTable.openFriday);
            model.openSaturday = timetableServices.parseStringtimeToDate(timeTable.openSaturday);
            model.openSunday = timetableServices.parseStringtimeToDate(timeTable.openSunday);

            model.closeMonday = timetableServices.parseStringtimeToDate(timeTable.closeMonday);
            model.closeTuesday = timetableServices.parseStringtimeToDate(timeTable.closeTuesday);
            model.closeWednesday = timetableServices.parseStringtimeToDate(timeTable.closeWednesday);
            model.closeThursday = timetableServices.parseStringtimeToDate(timeTable.closeThursday);
            model.closeFriday = timetableServices.parseStringtimeToDate(timeTable.closeFriday);
            model.closeSaturday = timetableServices.parseStringtimeToDate(timeTable.closeSaturday);
            model.closeSunday = timetableServices.parseStringtimeToDate(timeTable.closeSunday);
        }
        return model;
    };

    timetableServices.parseFormToTimetable = function (timeTable, model) {
        if (timeTable === undefined) {
            timeTable = {};
        }
        if (model !== undefined) {
            timeTable.openMonday = timetableServices.formatDateToStringtime(model.openMonday);
            timeTable.openTuesday = timetableServices.formatDateToStringtime(model.openTuesday);
            timeTable.openWednesday = timetableServices.formatDateToStringtime(model.openWednesday);
            timeTable.openThursday = timetableServices.formatDateToStringtime(model.openThursday);
            timeTable.openFriday = timetableServices.formatDateToStringtime(model.openFriday);
            timeTable.openSaturday = timetableServices.formatDateToStringtime(model.openSaturday);
            timeTable.openSunday = timetableServices.formatDateToStringtime(model.openSunday);

            timeTable.closeMonday = timetableServices.formatDateToStringtime(model.closeMonday);
            timeTable.closeTuesday = timetableServices.formatDateToStringtime(model.closeTuesday);
            timeTable.closeWednesday = timetableServices.formatDateToStringtime(model.closeWednesday);
            timeTable.closeThursday = timetableServices.formatDateToStringtime(model.closeThursday);
            timeTable.closeFriday = timetableServices.formatDateToStringtime(model.closeFriday);
            timeTable.closeSaturday = timetableServices.formatDateToStringtime(model.closeSaturday);
            timeTable.closeSunday = timetableServices.formatDateToStringtime(model.closeSunday);
        }
        return timeTable;
    };

    timetableServices.parseStringtimeToDate = function (stringtime) {
        var date = null;
        if (stringtime != undefined && stringtime !== null) {
            date = moment(stringtime, 'HH:mm:ss').toDate();
        }
        return date;
    };

    timetableServices.formatDateToStringtime = function (date) {
        var stringtime = null;
        if (date != undefined && date !== null) {
            stringtime = moment(date).format('HH:mm:ss');
        }
        return stringtime;
    };

    timetableServices.assureTimetable = function (timeTable, type) {
        if (timeTable === undefined) {
            return false;
        } else {
            if (type === TIMETABLE_TYPE.OPEN_CLOSE) {
                if (timeTable.time.openMonday === undefined &&
                    timeTable.time.openTuesday === undefined &&
                    timeTable.time.openWednesday === undefined &&
                    timeTable.time.openThursday === undefined &&
                    timeTable.time.openFriday === undefined &&
                    timeTable.time.closeMonday === undefined &&
                    timeTable.time.closeTuesday === undefined &&
                    timeTable.time.closeWednesday === undefined &&
                    timeTable.time.closeThursday === undefined &&
                    timeTable.time.closeFriday === undefined) {
                    return false;
                } else {
                    return true;
                }
            } else if (type == TIMETABLE_TYPE.APPOINTMENT) {
                if (timeTable.timeAppoiment.openMonday === undefined &&
                    timeTable.timeAppoiment.openTuesday === undefined &&
                    timeTable.timeAppoiment.openWednesday === undefined &&
                    timeTable.timeAppoiment.openThursday === undefined &&
                    timeTable.timeAppoiment.openFriday === undefined &&
                    timeTable.timeAppoiment.closeMonday === undefined &&
                    timeTable.timeAppoiment.closeTuesday === undefined &&
                    timeTable.timeAppoiment.closeWednesday === undefined &&
                    timeTable.timeAppoiment.closeThursday === undefined &&
                    timeTable.timeAppoiment.closeFriday === undefined) {
                    return false;
                } else {
                    return true;
                }
            }
        }
    };

    return timetableServices;
});

function fillTimetable(obj) {
    var objName = obj.name;
    _.each($(":input[name^='" + objName + "_']"), function (inp) {
        $(inp).val($(obj).val());
        $(inp).trigger('input');
    });
}