'use strict';

angular.module('bewin_orange')
    .controller('ApplicationsAdminController', function ($scope, $log, toastr, modalService, APIApplication, APIGeneric, APITenant) {

        $log.log("ApplicationsAdminController started ..");

        $scope.model = {
            applications: [],
            application: {},
            channels: APIGeneric.byType(3).$object,
            tenants: APITenant.list().$object
        };

        $scope.form = {
            application: {}
        };

        $scope.select = function (app) {

            $scope.form.errorMessage = null;

            if (app && app.id) {

                APIApplication.loadOne(app.id).then(function (response) {
                    $scope.model.application = response;
                    $scope.form.application = JSON.parse(JSON.stringify(response));
                    $scope.form.application.channel = _.find($scope.model.channels, ['id', response.channel.id]);

                    $scope.form.application.tenants = [];
                    response.tenants.map(function (tenant) {
                        var found = _.find($scope.model.tenants, ['id', tenant.id]);
                        if (found !== undefined) {
                            $scope.form.application.tenants.push(found);
                        }

                    });
                });
            } else {
                $scope.model.application = {};
                $scope.form.application = {};
            }
        };

        $scope.save = function () {

            var PRECONDITION_FAILED = 412;
            var DUPLICATED = 409;

            $scope.form.application.name = $scope.form.application.name.toUpperCase();

            APIApplication.update($scope.form.application).then(
                function (application) {
                    $scope.select(application);
                    $scope.form.applications = loadAll();
                    toastr.info("Applicación guardad correctamente", "Operación realizada");
                    $scope.form.errorMessage = null;
                },
                function (res) {
                    var msg = res.data.details.errorMessage;
                    if (res.status === PRECONDITION_FAILED) {

                        msg += '. Por tanto , debe desvincular primero los dispositivos configurados con esta aplicación.';
                        modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);

                        var modalInstance = modalService.modalInstance;

                        modalInstance.result.then(function () {
                            $scope.select($scope.model.application);
                        });
                    }
                    if (res.status === DUPLICATED) {
                        modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
                    }
                }
            );
        };

        $scope.delete = function () {
            var msg = 'Si elimina la aplicación se eliminarán todos los dispositivos configurados para la misma. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;
            modalInstance.result.then(function () {
                APIApplication.deleteOne($scope.form.application.id).then(function () {
                    $scope.form.applications = loadAll();
                    $scope.select({});
                    toastr.info("Applicación eliminada correctamente", "Operación realizada");
                });

            });
        };

        $scope.cancel = function (app) {
            $scope.select({});
        };

        var loadAll = function () {
            $scope.model.applications = APIApplication.list().$object;
        };

        loadAll();

    });
