'use strict';

angular.module('bewin_orange')
    .controller('EmailAdminController', function ($scope, $http, $log, sessionService, configurationService, APIQueue, APIEmail, toastr) {

        $scope.model = {
            queues: [],
            file: {}
        };

        $scope.form = {
            selectedQueues: [],
            text: '',
            subject: '',
            recipient: ''
        };

        $scope.cancel = function () {
            $scope.form = {
                selectedQueues: [],
                text: '',
                subject: '',
                recipient: ''
            };
            $scope.model.file = {};
        };

        $scope.loadFile = function (element) {
            $scope.model.file = element.files[0];
        };

        $scope.sendMassiveEmail = function () {
            var body = {
                subject: $scope.form.subject,
                text: $scope.form.text
            };
            $http({
                method: 'POST',
                url: configurationService.apiURL + '/email/sendCommunication',
                headers: {
                    'X-Auth-Token': sessionService.securityToken,
                    'Content-Type': undefined
                },
                //This method will allow us to change how the data is sent up to the server
                // for which we'll need to encapsulate the model data in 'FormData'
                transformRequest: function (data) {
                    var formData = new FormData();
                    //need to convert our json object to a string version of json otherwise
                    // the browser will do a 'toString()' on the object which will result
                    // in the value '[Object object]' on the server.
                    formData.append("params", new Blob([JSON.stringify(data.model)]));
                    //now add all of the assigned files

                    //add each file to the form data and iteratively name them
                    formData.append("file", data.files);

                    return formData;
                },
                data: {model: body, files: $scope.model.file}
            }).then(function successCallback(response) {
                // this callback will be called asynchronously
                // when the response is available
                $scope.cancel();
                $("#attachment").val('');
                toastr.info("Mensaje enviado correctamente");
            }, function errorCallback(response) {
                // called asynchronously if an error occurs
                // or server returns response with an error status.
                if (response.status === 428) {
                    var msg = 'El token de usuario ha caducado. Su sesión se cerrará y será necesario que vuelva a realizar el login. Gracias.';
                    modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, 'login');
                    return false;
                } else {
                    if (response.status && response.statusText) {
                        toastr.error("Error:" + response.status + " (" + response.statusText + ")");
                    } else if (response.data && response.data.message) {
                        toastr.error("Error:" + response.data.message);
                    } else if (response.data.achieved !== undefined && response.data.achieved === false) {
                        toastr.error("Error:" + response.data.details.errorMessage);
                    }
                }
            });
        };

        $scope.sendUsersEmail = function () {

            var body = {
                selectedQueues: $scope.form.selectedQueues,
                text: $scope.form.text,
                subject: $scope.form.subject
            };
            APIEmail.sendUsersEmail(body).then(function (resp) {
                if (resp === true) {
                    toastr.success("Mensaje enviado con éxito");
                }
            });
        };

        $scope.sendEmail = function () {
            var body = {
                subject: $scope.form.subject,
                text: $scope.form.text,
                recipient: $scope.form.recipient
            };
            $http({
                method: 'POST',
                url: configurationService.apiURL + '/email/sendMail',
                headers: {
                    'X-Auth-Token': sessionService.securityToken,
                    'Content-Type': undefined
                },
                //This method will allow us to change how the data is sent up to the server
                // for which we'll need to encapsulate the model data in 'FormData'
                transformRequest: function (data) {
                    var formData = new FormData();
                    //need to convert our json object to a string version of json otherwise
                    // the browser will do a 'toString()' on the object which will result
                    // in the value '[Object object]' on the server.
                    formData.append("params", new Blob([JSON.stringify(data.model)]));
                    //now add all of the assigned files
                    //add each file to the form data and iteratively name them
                    formData.append("file", data.files);

                    return formData;
                },
                data: {model: body, files: $scope.model.file}
            }).then(function successCallback(response) {
                // this callback will be called asynchronously
                // when the response is available
                $scope.cancel();
                $("#attachment").val('');
                toastr.info("Mensaje enviado correctamente");
            }, function errorCallback(response) {
                // called asynchronously if an error occurs
                // or server returns response with an error status.
                if (response.status === 428) {
                    var msg = 'El token de usuario ha caducado. Su sesión se cerrará y será necesario que vuelva a realizar el login. Gracias.';
                    modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, 'login');
                    return false;
                } else {
                    if (response.status && response.statusText) {
                        toastr.error("Error:" + response.status + " (" + response.statusText + ")");
                    } else if (response.data && response.data.message) {
                        toastr.error("Error:" + response.data.message);
                    } else if (response.data.achieved !== undefined && response.data.achieved === false) {
                        toastr.error("Error:" + response.data.details.errorMessage);
                    }
                }
            });
        };

        var init = function () {

            $scope.cancel();

            APIQueue.byTenant(sessionService.selectedTenant.id).then(function (resp) {
                $scope.model.queues = resp;
            });
        };

        init();
    });