'use strict';

angular.module('bewin_orange')
    .controller('FilesAdminController', function ($scope, sessionService, configurationService, Upload, toastr,
                                                  APIQueue, APIFiles, APIOperationsProfile, APIQueueLicenseType,
                                                  APITicketDesign, APIGeneric) {

        var uploadUrl = configurationService.apiURL + "/files/";
        var uploadAppointmentUrl = configurationService.apiAppointmentURL + '/files/';

        // en el form definiremos los objetos que serviran para almacenar las variables del HTML,
        // pero en vez de poner 1 por 1, generamos un objeto con todas sus propiedades
        // si fuera necesario aniadir la carga de otro objeto, se pondria aqui (p.ej. servicePoint)
        $scope.form = {
            user: {
                isUser: true
            },
            queue: {
                isQueue: true
            }
        };

        // en el model los maestros
        $scope.model = {
            // queues
            operationsProfiles: [],
            queueLicenseTypes: [],
            ticketDesigns: [],
            queueConfigurations: {},
            // user
            roles: [],
            workProfiles: []
        };

        // reseteo de los valores
        $scope.clean = function () {
            $scope.form = {
                user: {
                    isUser: true
                },
                queue: {
                    isQueue: true
                }
            };

            $scope.model = {
                operationsProfiles: [],
                queueLicenseTypes: [],
                ticketDesigns: [],
                queueConfigurations: {},
                // user
                roles: [],
                workProfiles: []
            };
        };

        $scope.upload = function (selectedFile, selectedItem, method) {

            if (selectedFile) {
                if (selectedItem.isUser) {
                    selectedItem = {
                        user: {
                            tenant: {id: sessionService.selectedTenant.id},
                            roles: [],
                            workProfiles: []
                        }
                    };
                    $scope.form.user.roles.forEach(function (obj) {
                        selectedItem.user.roles.push({id: obj.id})
                    });
                    $scope.form.user.workProfiles.forEach(function (obj) {
                        selectedItem.user.workProfiles.push({id: obj.id})
                    });
                } else if (selectedItem.isQueue) {
                    selectedItem = {
                        queue: {
                            tenant: {id: sessionService.selectedTenant.id},
                            operationsProfile: {id: ($scope.form.queue.operationsProfile !== undefined ? $scope.form.queue.operationsProfile.id : null)},
                            queueLicenseType: {id: ($scope.form.queue.queueLicenseType !== undefined ? $scope.form.queue.queueLicenseType.id : null)},
                            ticketDesign: {id: ($scope.form.queue.ticketDesign !== undefined ? $scope.form.queue.ticketDesign.id : null)},
                            configuration: []
                        }
                    };
                    if ($scope.form.queue.configuration) {
                        $scope.form.queue.configuration.forEach(function (obj) {
                            selectedItem.queue.configuration.push({id: obj.id});
                        });
                    }
                }
                Upload.upload({
                    url: uploadUrl + method,
                    headers: {
                        'X-Auth-Token': sessionService.securityToken,
                        'Content-Type': 'multipart/form-data'
                    },
                    data: {"item": JSON.stringify(selectedItem)},
                    file: selectedFile
                }).then(function (response) {
                    $scope.clean();
                    if (response) {
                        toastr.success("Archivo subido correctamente", "Operación realizada");
                    }
                }, function (error) {
                    var mensaje = "";
                    if (typeof error.data.details.errorMessage != 'undefined') {
                        mensaje = error.data.details.errorMessage;
                    } else {
                        mensaje = error.status + " (" + error.statusText + ")";
                    }
                    toastr.error("Error: " + mensaje);
                });
            }
        };

        $scope.uploadAppointment = function (selectedFile) {
            if (selectedFile) {
                Upload.upload({
                    url: uploadAppointmentUrl + 'update',
                    headers: {
                        'Content-Type': 'multipart/form-data'
                    },
                    data: {},
                    file: selectedFile
                }).then(function (response) {
                    $scope.clean();
                    if (response) {
                        toastr.success("Archivo subido correctamente", "Operación realizada");
                    }
                }, function (error) {
                    var mensaje = "";
                    if (typeof error.data.details.errorMessage != 'undefined') {
                        mensaje = error.data.details.errorMessage;
                    } else {
                        mensaje = error.status + " (" + error.statusText + ")";
                    }
                    toastr.error("Error: " + mensaje);
                });
            }
        };

        var loadQueueConfiguration = function () {

            // mapeamos las opciones disponibles para configuración teniendo en cuenta las opciones del Tenant
            // por ejemplo, estara disponible la opción "Enviar ticket por SMS" solo si el tenant tiene configurado el envio por SMS

            APIGeneric.queueConfParams().then(function (resp) {

                var tQueueConfParams = resp.plain();

                if (!sessionService.selectedTenant.smsEnabled) {
                    $scope.model.queueConfigurations = _.filter(tQueueConfParams, function (el) {
                        return el.id !== 10001;
                    });
                } else {
                    $scope.model.queueConfigurations = tQueueConfParams;
                }
            });
        };

        var loadWorkProfiles = function () {

            $scope.model.workProfiles = [];

            APIQueue.byTenant(sessionService.getTenantId()).then(function (queues) {
                _.each(queues, function (q) {
                    _.each(q.operationsProfile.workProfiles, function (w) {
                        $scope.model.workProfiles.push(w);
                    });
                });

                $scope.model.workProfiles = _.uniqBy($scope.model.workProfiles, "id");
            });
        };

        var loadData = function () {
            // queue
            APIOperationsProfile.byTenant(sessionService.getTenantId()).then(function (resp) {
                $scope.model.operationsProfiles = resp;
            });

            APIQueueLicenseType.loadAll().then(function (resp) {
                $scope.model.queueLicenseTypes = resp;
            });

            APITicketDesign.byTenant(sessionService.getTenantId()).then(function (resp) {
                $scope.model.ticketDesigns = resp;
            });

            loadQueueConfiguration();

            // user
            APIGeneric.roles().then(function (resp) {
                $scope.model.roles = resp;
            });
            loadWorkProfiles();

        };

        loadData();

    });