'use strict';

angular.module('bewin_orange')
    .controller('TenantAdminController', function ($scope, $log, $stateParams, modalService, sessionService, NavigationServices,
                                                   toastr, APITenant, APIVariable, APISMSProvider, APIModule, APITicket, APITenantMarcas) {

        $log.log("TenantAdminController started ..");

        $scope.model = {
            tenants: [],
            variables: [],
            smsproviders: APISMSProvider.loadAll().$object,
            modules: APIModule.loadAll().$object,
            marcasPropias: [],
            marcasAjenas: []
        };

        $scope.form = {
            tenant: {},
            variable: {},
            newMarcaPropia: null,
            newMarcaAjena: null
        };

        $scope.orderMarcasPropias = {
            stop: function (e, ui) {
                if ($scope.model.marcasPropias != undefined && $scope.model.marcasPropias.length > 0) {
                    var i = 0;
                    while (i < $scope.model.marcasPropias.length) {
                        $scope.model.marcasPropias[i].tenant = {
                            id: sessionService.getUserTenant().id
                        };
                        $scope.model.marcasPropias[i].orderBy = i;
                        i++;
                    }
                    APITenantMarcas.updateOrderList($scope.model.marcasPropias).then(function (res) {
                        toastr.info("Orden actualizado", "Operación realizada");
                    });
                }
            }
        };

        $scope.orderMarcasAjenas = {
            stop: function (e, ui) {
                if ($scope.model.marcasAjenas != undefined && $scope.model.marcasAjenas.length > 0) {
                    var i = 0;
                    while (i < $scope.model.marcasAjenas.length) {
                        $scope.model.marcasAjenas[i].tenant = {
                            id: sessionService.getUserTenant().id
                        };
                        $scope.model.marcasAjenas[i].orderBy = i;
                        i++;
                    }
                    APITenantMarcas.updateOrderList($scope.model.marcasAjenas).then(function (res) {
                        toastr.info("Orden actualizado", "Operación realizada");
                    });
                }
            }
        };

        $scope.select = function (t) {

            $(".onoffswitch-checkbox").attr('checked', false);

            if (t && t.id) {
                APITenant.loadOne(t.id).then(function (response) {
                    $scope.form.tenant = response;
                    $scope.model.variables = [];
                    $scope.model.variables = APIVariable.byTenant(t.id).$object;
                    $scope.form.tenant.variables = $scope.model.variables;
                    $scope.model.modules.map(function (element) {
                        element.checked = _.find(response.modules, ['id', element.id]) !== undefined;
                    });

                    $scope.model.marcasPropias = response.tenantMarca.filter((subject) => subject.type === 'PROPIA');
                    $scope.model.marcasAjenas = response.tenantMarca.filter((subject) => subject.type === 'AJENA');

                    if (response.tenantAppointmentConfig !== undefined && response.tenantAppointmentConfig.smsReminderAppointment === 1) {
                        $scope.form.tenant.tenantAppointmentConfig.smsReminderAppointment = true;
                    } else {
                        $scope.form.tenant.tenantAppointmentConfig.smsReminderAppointment = false;
                    }
                    $scope.manageTenant();
                });
            } else {
                $scope.form.tenant = {};
                $scope.model.variables = [];
                $scope.model.modules.map(function (element) {
                    element.checked = false;
                });

                $scope.manageTenant();
            }
        };

        $scope.cancel = function () {
            $scope.select({});
        };

        $scope.save = function () {
            if ($scope.form.tenant.tenantAppointmentConfig.smsReminderAppointment === true) {
                $scope.form.tenant.tenantAppointmentConfig.smsReminderAppointment = 1;
            } else {
                $scope.form.tenant.tenantAppointmentConfig.smsReminderAppointment = null;
            }
            APITenant.update($scope.form.tenant).then(function (res) {
                $scope.select(res);
                toastr.info("Cliente actualizado correctamente", "Operación realizada");
                loadAll();
            });
        };

        $scope.manageTenant = function () {
            if ($scope.form.tenant.id === sessionService.getUserTenant().id) {
                sessionService.setUserTenant($scope.form.tenant);
                sessionService.setSelectedTenant($scope.form.tenant);
            } else {
                NavigationServices.canUserLogout().then(function (response) {
                    if (response === true) {
                        sessionService.setUserTenant($scope.form.tenant);
                        sessionService.setSelectedTenant($scope.form.tenant);

                        if (sessionService.getQueueId() !== null) {
                            APITicket.cleanWaitLineForUser(sessionService.user.id);
                        }
                        sessionService.setQueue(null);
                    }
                });
            }
        };

        $scope.selectVar = function (v) {
            $scope.form.variable = angular.copy(v);
        };

        $scope.cancelVar = function () {
            $scope.form.variable = {};
        };

        $scope.deleteVar = function () {
            if (!$scope.form.variable || !$scope.form.variable.id) return;
            APIVariable.deleteOne($scope.form.variable.id).then(function () {
                $scope.model.variables = APIVariable.byTenant($scope.form.tenant.id).$object;
                toastr.info("Variable eliminada correctamente", "Operacion realizada");
                $scope.cancelVar();
            });
        };

        $scope.saveVar = function () {
            delete $scope.form.variable.tenant;
            $scope.form.variable.tenant = {
                id: $scope.form.tenant.id
            };

            APIVariable.update($scope.form.variable).then(function (v) {
                $scope.model.variables = APIVariable.byTenant($scope.form.tenant.id).$object;
                toastr.info("Variable actualizada correctamente", "Operacion realizada");
                $scope.cancelVar();
            });
        };

        $scope.smsEnableModule = function () {
            if (!$scope.form.tenant.smsEnabled) {
                $scope.form.tenant.smsProvider = null;
            }
        };

        $scope.remove = function () {
            var msg = 'Se va a deshabilitar el cliente y ya no se podrá trabajar sobre el mismo. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;
            modalInstance.result.then(function () {
                APITenant.deleteOne($scope.form.tenant.id).then(function (r) {
                    toastr.info("Cliente eliminado correctamente", "Operación realizada");
                    loadAll();
                });
                $scope.cancel();
            });
        };

        // guardamos o eliminamos un modulo de un tenant
        $scope.saveModuleState = function (event) {
            var module = {
                id: event.currentTarget.value,
                name: event.currentTarget.name
            };
            if (sessionService.selectedTenant !== null) {
                // si el checkbox esta seleccionado, añadimos el modulo al tenant
                if (event.currentTarget.checked) {
                    sessionService.selectedTenant.modules.push(module);
                } else {
                    // si el checkbox NO esta seleccionado, quitamos el modulo del tenant
                    sessionService.selectedTenant.modules = sessionService.selectedTenant.modules.filter(function (elem) {
                        return elem.id !== parseInt(module.id);
                    });
                }
                APITenant.update(sessionService.selectedTenant);
            } else {
                event.currentTarget.checked = false;
                alert('No hay tenant seleccionada');
            }
        };

        $scope.saveMarca = function (marcaName, type) {
            var orderBy = -1;
            if (type === 'PROPIA') {
                orderBy = $scope.model.marcasPropias.length;
            } else if (type === 'AJENA') {
                orderBy = $scope.model.marcasAjenas.length;
            }

            var newMarca = {
                name: marcaName,
                type: type,
                orderBy: orderBy,
                tenant: {
                    id: $scope.form.tenant.id
                }
            };

            APITenantMarcas.update(newMarca).then(function (response) {
                console.log(response);

                if (response.id !== undefined && response.id != null) {

                    toastr.info("Marca creada", "Operación realizada");

                    $scope.form.newMarcaAjena = null;
                    $scope.form.newMarcaPropia = null;

                    APITenantMarcas.byTenant($scope.form.tenant.id).then(function (data) {
                        $scope.model.marcasPropias = data.plain().filter((subject) => subject.type === 'PROPIA');
                        $scope.model.marcasAjenas = data.plain().filter((subject) => subject.type === 'AJENA');
                    });
                }
            });
        };

        $scope.removeMarca = function (marcaId) {
            APITenantMarcas.deleteOne(marcaId).then(function (response) {
                toastr.info("Marca eliminada correctamente", "Operación realizada");

                APITenantMarcas.byTenant($scope.form.tenant.id).then(function (data) {
                    $scope.model.marcasPropias = data.plain().filter((subject) => subject.type === 'PROPIA');
                    $scope.model.marcasAjenas = data.plain().filter((subject) => subject.type === 'AJENA');
                });
            });
        };

        var loadAll = function () {
            $scope.model.tenants = APITenant.list().$object;
        };

        loadAll();

    });
