'use strict';

angular.module('bewin_orange')
    .controller('ReceptionEsperaController', function ($scope, $state, $stateParams, $timeout, $uibModal, APIQueue,
                                                       APIUser, toastr, APIServicePoint, APITicket, APICapacity, APIGeneric,
                                                       NavigationServices) {

        $scope.model = {
            queue: {},
            waitingTickets: [],
            zone: null,
            capacity: 0
        };

        $scope.reverseSort = false;
        $scope.reverseSortTEspera = false;
        $scope.reverseSortName = false;

        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.showTab = function (tabId) {
            switch (tabId) {
                case 1:
                    $state.go('reception.ticket', {id: $stateParams.id});
                    break;
                case 2:
                    $state.go('reception.espera', {id: $stateParams.id});
                    break;
            }
        };

        $scope.toUTCDate = function (date) {
            return new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(),
                date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
        };

        $scope.toDate = function (t) {
            var diff;
            var waitingTime;

            var clock = Date.now(); // get the current time
            var offset = moment().utcOffset();
            var serverOffset = moment().tz("Europe/Madrid").utcOffset();
            var difference = parseInt(offset) - parseInt(serverOffset);

            if (t.scheduledServiceTime === undefined) {
                clock = moment(clock).subtract(difference, 'm').toDate();
            }

            var date = new Date(t.issueTime);
            if (clock.valueOf() < date.valueOf()) {
                diff = 0;
            } else {
                diff = clock.valueOf() - date.valueOf();
            }
            waitingTime = $scope.toUTCDate(new Date(diff));
            return waitingTime;
        };

        $scope.orderByTEspera = function () {
            $scope.orderByField = 'maxWaitTime';
            $scope.reverseSortTEspera = !$scope.reverseSortTEspera;
            $scope.reverseSort = $scope.reverseSortTEspera;
        };

        $scope.orderByServiceName = function () {
            $scope.orderByField = 'name';
            $scope.reverseSortName = !$scope.reverseSortName;
            $scope.reverseSort = $scope.reverseSortName;
        };

        $scope.reenviarTicket = function (t) {
            var modalInstanceMaitreOrangeReenvia = $uibModal.open({
                templateUrl: 'views/reception/espera/modal_reenvio_servicio_puesto.html',
                controller: 'ReenvioCtrl',
                backdrop: 'static',
                keyboard: false,
                resolve: {
                    ticket: function () {
                        return t;
                    },
                    queue: function () {
                        return $scope.model.queue;
                    }
                }
            });

            modalInstanceMaitreOrangeReenvia.result.then(
                function (t) {
                    APITicket.waitingInQueue($scope.model.queue.id).then(function (ts) {
                        $scope.model.waitingTickets = ts;
                    });
                }
            );
        };

        $scope.editarTicket = function (t) {
            var modalInstanceMaitreOrangeEdita = $uibModal.open({
                templateUrl: 'views/reception/espera/modal_editar_datos_ticket.html',
                controller: 'EditarCtrl',
                backdrop: 'static',
                keyboard: false,
                scope: $scope,
                resolve: {
                    ticket: function () {
                        return t;
                    },
                    queue: function () {
                        return $scope.model.queue;
                    }
                }
            });
        };


        var timer;

        var init = function () {

            // obtenemos información de la tienda, el ID está en la URL
            if (typeof $stateParams.id !== 'undefined' && $stateParams.id !== null) {

                APIQueue.loadOne($stateParams.id).then(function (data) {

                    $scope.model.queue = data;

                    if ($scope.model.queue !== {}) {
                        // buscamos tickets en espera
                        APITicket.waitingInQueue($scope.model.queue.id).then(function (ts) {
                            $scope.model.waitingTickets = ts;
                            _.each($scope.model.waitingTickets, function (sp) {
                                var generics = sp.ticketGenerics;
                                var k = _.find(generics, ['generic.id', 11003]);
                                if (k !== undefined) {
                                    sp.motive = k.valor;
                                } else {
                                    sp.motive = '';
                                }
                            });
                        });

                        // si aun no hemos buscado la zone de conteo, la buscamos
                        if ($scope.model.zone === null) {
                            // obtenemos un valor para saber si la tienda tiene conteo o no
                            APICapacity.findZone($scope.model.queue.internalCode).then(function (resp) {
                                if (resp && resp.internal_code && resp.max_capacity) {
                                    $scope.queueHasCounter = true;
                                    $scope.model.zone = resp;
                                }
                            });
                        } else {
                            if ($scope.queueHasCounter) {
                                APICapacity.findCapacity($scope.model.queue.internalCode).then(function (capacity) {
                                    if (capacity !== undefined) {
                                        $scope.model.capacity = Math.round(capacity * 100 / $scope.model.zone.max_capacity);
                                    } else {
                                        $scope.model.capacity = '-';
                                    }
                                });
                            }
                        }
                    }
                });
            }

            timer = $timeout(init, 5000);
        };

        $scope.$on("$destroy", function () {
            if (timer) {
                $timeout.cancel(timer);
            }
        });

        $scope.actualizarEditado = function () {
            init();
        };

        init();

    })

    //CONTROLLER PARA EL MODAL DE EDITAR LOS DATOS DE UN TICKET
    .controller('EditarCtrl', function ($scope, $uibModalInstance, $stateParams, $q, APIServicePoint, APIUser,
                                        APITicket, APIVariable, APIGeneric, APIWorkProfile, sessionService, ticket, queue) {
        $scope.model = {
            ticketGenerics: APIGeneric.ticketExtendedData(11).$object,
            t: ticket
        };

        if (verifyVariable($scope.model.t.visitorName)){
            $scope.nameComplete = $scope.model.t.visitorName;
            if (verifyVariable($scope.model.t.visitorFamilyName))
            {
                $scope.nameComplete += ' ' + $scope.model.t.visitorFamilyName;
            }
        }

        $scope.motivoVisita = _.find(ticket.ticketGenerics, ['generic.id', 11003]);

        //Verifica las variables
        function verifyVariable(data){
            return data !== undefined && data !== null && data !== '';
        }

        $scope.salvarTicketEditado = function (waitStatus) {
            if (verifyVariable($scope.motivoVisita.valor)) {
                var motivoGeneric = _.find($scope.model.t.ticketGenerics, ['generic.id', 11003]);
                // Si existe el ticketgeneric lo actualizo
                if (verifyVariable(motivoGeneric)) {
                    motivoGeneric.valor = $scope.motivoVisita.valor;

                    // Si no existe el ticketgeneric lo creo
                } else if (verifyVariable(_.find($scope.model.ticketGenerics, ['id', 11003]))) {
                    var extendedData = {
                        generic: _.find($scope.model.ticketGenerics, ['id', 11003]), // motivo Orange
                        valor: $scope.motivoVisita.valor
                    };
                    $scope.model.t.ticketGenerics.push(extendedData);

                }
            }
            if ($scope.nameComplete !== $scope.model.t.visitorName + ' ' + $scope.model.t.visitorFamilyName) {
                $scope.model.t.visitorFamilyName = '';
                $scope.model.t.visitorName = $scope.nameComplete;
            }

            APITicket.save($scope.model.t).then(function (result) {
                console.log(result);
                $uibModalInstance.dismiss('cancel');
                $scope.actualizarEditado();
            });
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        };
    })

    //CONTROLLER PARA EL MODAL DE REENVIAR EL TICKET
    .controller('ReenvioCtrl', function ($scope, $uibModalInstance, $stateParams, $q, APIServicePoint, APIUser,
                                         APITicket, APIVariable, APIWorkProfile, APIGeneric, sessionService, ticket, queue) {

        $scope.reenvioServicio = false;
        $scope.reenvioUsuario = false;

        $scope.model = {
            t: ticket,
            ticketGenerics: APIGeneric.ticketExtendedData(11).$object,
            servicios: [],
            usuarios: []
        };

        $scope.form = {
            servicio: {},
            usuario: {}
        };

        $scope.noSelection = function () {
            if (typeof $scope.form.servicio.id == 'undefined' && typeof $scope.form.usuario.id == 'undefined') {
                return true;
            } else {
                return false;
            }
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        };

        $scope.verServicios = function () {
            $scope.reenvioServicio = true;
            $scope.reenvioUsuario = false;
            $scope.form.usuario = {};
        };

        $scope.verUsuarios = function () {
            $scope.reenvioUsuario = true;
            $scope.reenvioServicio = false;
            $scope.form.servicio = {};
        };

        $scope.reenviaTicket = function (waitStatus) {
            // cuando se reenvia un ticket siempre se mantiene el tiempo de espera
            // en el caso de reenvio al servicio se llamara segun el tiempo de espera
            // en el caso de reenvio a un usuario, se mantiene el mismo servicio del ticket y sera atendido antes que los tickets que no esten asignado a el

            var t = $scope.model.t;
            t.servicePoint = (typeof $scope.form.servicio.id != 'undefined') ? $scope.form.servicio : $scope.model.t.servicePoint;
            t.assignedTo = (typeof $scope.form.usuario.id != 'undefined') ? $scope.form.usuario : null;
            t.status = waitStatus;

            APITicket.redirect(t).then(function (result) {
                $uibModalInstance.close(result);
                sessionService.isServing = false;
            });

        };

        $scope.saveEditedTicket = function (waitStatus) {
            console.log("Salvando lo editado");
        };

        var loadServicios = function () {
            // obtenemos los servicios que puede ver quien trabaje en la tienda
            if (typeof queue != 'undefined' && queue.id != null) {
                return deferredFn();
            }
        };

        var init = function () {
            loadServicios();
        };

        function deferredFn() {
            var defer = $q.defer();
            var promises = [];
            $scope.model.usuarios = [];

            function lastTask() {
                APIServicePoint.byQueue(queue.id).then(function (services) {
                    // obtenemos los servicios de la tienda que pueden ser atendidos por los usuarios logeados con sus perfiles actuales
                    $scope.model.servicios = services.filter(function (service) {
                        var serviceFound = _.find(userServices, ['id', service.id]);
                        return serviceFound !== undefined;
                    });

                    defer.resolve();
                });
            }

            var userServices = [];

            // obtenemos los usuarios trabajando en la tienda
            APIUser.workingInQueue(queue.id).then(function (users) {

                _.each(users, function (u) {

                    if (typeof u.currentStation !== 'undefined') {

                        $scope.model.usuarios.push(u);

                        promises.push(APIWorkProfile.loadOne(u.currentWorkProfile.id).then(function (workProfile) {
                            _.each(workProfile.servicePoints, function (sp) {
                                var found = _.find(userServices, ['id', sp.id]);
                                if (!found) {
                                    userServices.push(sp);
                                }
                            });
                        }));
                    }
                });

                $q.all(promises).then(lastTask);
            });

            return defer.promise;
        }

        init();
    });