'use strict';

angular.module('bewin_orange')
    .controller('ReceptionTicketController', function ($scope, $log, $stateParams, $state, $compile, $timeout, $window,
                                                       $uibModal, sessionService, modalService, toastr, ORANGE_QUEUES,
                                                       APIQueue, APIUser, APIServicePoint, APIGeneric, APITicket, APISms,
                                                       APIVariable, NavigationServices,APIOperationsProfile,$q) {
        $log.log("ReceptionTicketController started ..");

        $scope.model = {
            ticketGenerics: APIGeneric.ticketExtendedData(11).$object,
            tipoClientes: APIGeneric.byType(12).$object,
            ticket: {},
            customerDescriptionArray: [],
            politicaPrivacidad: null
        };

        $scope.form = {
            queue: {},
            services: [],
            ticket: {
                nombre: "",
                phone: "",
                motivo: "",
                serviceId: 0,
                esCliente: 0,           // es cliente (1), no es cliente (2)
                tipoCliente: 3,         // si es Particular (1) o Empresa (2), no se ha identificado (3)
                customerDescription: "" //caracterización del cliente
            }
        };

        $scope.model.customerDescriptionArray = [];

        $scope.smsTicket = false;
        $scope.printTicket = false;
        $scope.esClienteJazztel = false;
        $scope.classSwitch = false;

        $scope.orangeQueues = ORANGE_QUEUES;

        $scope.showServices = {
            Cliente_AtencionCliente_conCita: false,
            Cliente_AtencionCliente_sinCita: false,
            Cliente_InfoComercial_Empresas_conCita: false,
            Cliente_InfoComercial_Empresas_sinCita: false,
            Cliente_InfoComercial_Particulares_Cita: false,
            Cliente_InfoComercial_Particulares: false,
            Cliente_RecargasPrepago: false,
            Cliente_TuExperto: false,
            FT_Duplicados_SIM: false,
            FT_Info_Accesorios: false,
            FT_RecogidaPedidos: false,
            Reparaciones: false,
            NoCliente_InfoComercial_Empresas_conCita: false,
            NoCliente_InfoComercial_Empresas_sinCita: false,
            NoCliente_InfoComercial_Particulares_Cita: false,
            NoCliente_InfoComercial_Particulares: false,
            NoCliente_RecargasPrepago: false,
            NoCliente_TuExperto: false,
            NoCliente_FT_Info_Accesorios: false,
            Entrega_router: false,
            ClienteJazztel_Info: false,
            ClienteJazztel_SIM: false,
            ClienteJazztel_Consulta: false,
            Cliente_Ayuda_Sobre_TuServicio: false
        };

        $scope.visibility = {
            orange : false,
            jazztel : false,
            noCliente : false,
            fastTrack : false
        };

        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.replaceServicePointName = function(name) {
            debugger;
            if (name) {
                return name
                    .replace('Cliente Orange -', '')
                    .replace('Fast Track -', '')
                    .replace('Cliente Jazztel -', '')
                    .replace('Cliente -', '')
                    .replace('No cliente', '');
            }
            return name; // Retorna el nombre original si no existe
        };

        $scope.getServiceColor = function (servicePoint) {
            var style = '';
            if(servicePoint.color != null){
                style = {"background-color":  servicePoint.color };
            }
            //console.log(style);
            return style;
        }

        $scope.getQueueServices = function() {
            console.log("Iniciando getQueueServices");

            return $q(function(resolve, reject) {
                if (!$scope.form.queue.operationsProfile || !$scope.form.queue.operationsProfile.id) {
                    console.error("El perfil de operaciones no está definido");
                    reject("Perfil de operaciones no definido");
                    return;
                }

                $scope.workProfiles = [];

                APIOperationsProfile.workProfile($scope.form.queue.operationsProfile.id)
                    .then(function(data) {
                        if (data && data.plain) {
                            $scope.workProfiles = data.plain();
                             // Filtro para eliminar workProfiles con hide_work_profile a true
                            $scope.workProfiles = $scope.workProfiles.filter(function(workprofile) {
                                return !workprofile.hideWorkProfile;
                            });
                            console.log("WorkProfile obtenido con éxito", $scope.workProfiles);
                            resolve($scope.workProfiles);
                        } else {
                            console.warn("Los datos recibidos no tienen el formato esperado");
                            reject("Datos con formato inesperado");
                        }
                    })
                    .catch(function(error) {
                        console.error("Error al obtener el workProfile:", error);
                        reject(error);
                    })
                    .finally(function() {
                        console.log("getQueueServices completado");
                    });
            });
        };

        $scope.cancel = function () {
            $scope.form.ticket = {
                nombre: "",
                phone: "",
                motivo: "",
                serviceId: 0,
                esCliente: 0,
                tipoCliente: 3,
                customerDescription: ""
            };

            angular.element(document.getElementsByClassName('maitreOrangeTblRow')).removeClass('maitreOrangeServiceClicked');
        };


        $scope.serviceSelected = function (servicioId, event) {

            // marcamos el servicio seleccionado
            angular.element(document.getElementsByClassName('maitreOrangeTblRow')).removeClass('maitreOrangeServiceClicked');
            $(event.target).addClass('maitreOrangeServiceClicked');

            $scope.form.ticket.serviceId = servicioId;
        };

        $scope.showCustomerDescriptionModal = function () {

            var modalInstance = $uibModal.open({
                templateUrl: 'views/reception/ticket/customer_description_modal.html',
                controller: 'CustomerDescriptionModalCtrl',
                backdrop: 'static',
                keyboard: false,
                scope: $scope
            });

            modalInstance.result.then(
                function (customerDescription) {

                    $scope.form.ticket.customerDescription = null;

                    angular.forEach(customerDescription, function (desc) {
                        if (!$scope.form.ticket.customerDescription) {
                            $scope.form.ticket.customerDescription = desc;
                        } else {
                            $scope.form.ticket.customerDescription += ", " + desc;
                        }
                    });

                });
        };

        $scope.generarTicket = function () {

            if (!isServiceSelected()) {
                var msg = 'Debe seleccionar un servicio';
                modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
            } else {
                $scope.crearVisita();
                completeTicketClean();
            }


        };

        $scope.crearVisita = function () {

            // comprobamos la variable si es una tienda con impresora portatil
            APIVariable.byNameAndQueue($scope.form.queue.id, 'IMPRESORA_PORTATIL').then(function (resp) {

                var data = (typeof resp != 'undefined') ? resp.plain() : 'undefined';

                if (typeof data == 'undefined') {
                    // para la tienda no esta definida impresora portatil
                    $scope.printTicket = false;
                    $log.log("Imrpresora portatil UNDEFINED");

                    // si el cliente ha dejado un telefono movil se debe mandar el ticket por SMS si la tienda lo tiene configurado
                    // se comprueba si el Tenant, Queue y Service tienen configurado el envio de ticket SMS
                    isSMSConfigured();

                } else {

                    if (data.value == 'true') {
                        // la tienda tiene definida impresora portatil
                        $scope.printTicket = true;
                        $log.log("Impresora portatil DEFINED and TRUE");
                    } else if (data.value == 'false') {
                        // para la tienda no esta definida impresora portatil
                        $scope.printTicket = false;
                        $log.log("Impresora portatil DEFINED and FALSE");
                    }

                    isSMSConfigured();
                }
            });

        };

        var isSMSConfigured = function () {

            $log.log("isSMSConfigured");

            if (typeof $scope.form.ticket.phone !== undefined && $scope.form.ticket.phone !== '') {

                // compobamos si el tenant, queue y service tienen configurados el envio de ticket SMS
                if ($scope.form.queue.tenant.smsEnabled && typeof $scope.form.queue.tenant.smsProvider !== undefined) { // Tenant tiene envio de SMS configurado

                    $log.log("tenant SMS configured");

                    var l = _.find($scope.form.queue.configuration, ['id', 10001]);
                    if (typeof l !== undefined) { // la tienda tiene envio de tickets por SMS configurado

                        $log.log("tienda SMS configured");

                        // comprobamos si el mensaje está configurado a nivel de tienda
                        if (typeof $scope.form.queue.ticketSmsMsg !== undefined && $scope.form.queue.ticketSmsMsg != null && $scope.form.queue.ticketSmsMsg !== '') {
                            // preguntar el tipo de ticket que se desea
                            $log.log("tienda SMS message configured");
                            $scope.smsTicket = true;
                        } else {
                            $log.log("tienda NO SMS message configured");
                            // comprobamos si el mensaje está configurado a nivel de servicio
                            var k = _.find($scope.form.services, ['id', $scope.form.ticket.serviceId]);

                            APIServicePoint.loadOne(k.id).then(function (serv) {
                                if (serv !== undefined && serv.smsMessages !== undefined && serv.smsMessages.ticketSmsMsg !== undefined && serv.smsMessages.ticketSmsMsg !== null) {
                                    // preguntar el tipo de ticket que se desea
                                    $log.log("service SMS message configured");
                                    $scope.smsTicket = true;
                                } else {
                                    // imprimir directamente el ticket
                                    $log.log("service NO SMS message configured");
                                    $scope.smsTicket = false;
                                }
                            });
                        }
                    } else {    // la tienda NO tiene envio de tickets por SMS configurado
                        // imprimir directamente el ticket
                        $scope.smsTicket = false;
                        $log.log("tienda NO SMS configured");
                    }
                } else {    // Tenant NO tiene envio de SMS configurado
                    // imprimir directamente el ticket
                    $log.log("tenant NO SMS configured");
                    $scope.smsTicket = false;
                }

            } else {    // el cliente NO ha dejado telefono movil
                $scope.smsTicket = false;
            }

            if ($scope.smsTicket && $scope.printTicket) { // elegir si se quiere ticket por SMS o impreso
                $scope.navigateTo("ticketSmsOrPrintedForm");
            } else {
                generarTicketOpcion1();
            }
        };

        $scope.generarTicketSMS = function () {

            $scope.smsTicket = true;
            $scope.printTicket = false;

            generarTicketOpcion1();
        };

        $scope.generarTicketPrint = function () {

            $scope.printTicket = true;
            $scope.smsTicket = false;

            generarTicketOpcion1();
        };

        $scope.newTicket = function () {
            $scope.cancel();
            $scope.generatedTicket = {};
        };

        $scope.showTab = function (tabId) {

            switch (tabId) {
                case 1:
                    $state.go('reception.ticket', {id: $stateParams.id});
                    break;
                case 2:
                    $state.go('reception.espera', {id: $stateParams.id});
                    break;
            }
        };

        $scope.navigateTo = function (formName) {

            initializeNavigation();

            switch (formName) {
                case 'customerDataForm':
                    $scope.cancel();
                    $scope.form.serviceId = {};
                    $scope.customerDataForm = true;
                    $scope.model.customerDescriptionArray = [];
                    $('.maitreOrangeTblRow').removeClass('maitreOrangeServiceClicked');
                    break;
                case 'ticketInfoForm':
                    $scope.ticketInfoForm = true;
                    break;
                case 'ticketSmsOrPrintedForm':
                    $scope.ticketSmsOrPrintedForm = true;
                    break;
                default:
                    $scope.cancel();
                    $scope.customerDescriptionArray = [];
                    break;
            }
        };

        //Calcula la hora a partir de la zona horaria
        function parseTimeZone(hour){
            var offset = moment().utcOffset();
            var serverOffset = moment().tz("Europe/Madrid").utcOffset();
            var difference = parseInt(offset) - parseInt(serverOffset);
            hour = moment(hour).add(difference, 'm').toDate();
            return hour;
        }

        var generarTicketOpcion1 = function () {

            // mapeamos los datos del formulario
            var ticket = ticketData();

            // generamos el ticket
            APITicket.issueTicket(ticket).then(function (resp) {

                var data = resp.plain();

                $scope.ticketCreado = 0;
                $scope.generatedTicket = data;

                //Transformamos la hora segun la zona horaria
                $scope.generatedTicket.issueTime = parseTimeZone($scope.generatedTicket.issueTime);

                // volvemos a pedir los datos completos de la Queue y ServicePoint para tener informacion de ticket design
                // y del nombre de servicio
                if (typeof $scope.generatedTicket.queue.id != 'undefined') {

                    APIQueue.loadOne($scope.generatedTicket.queue.id).then(function (resp) {
                        $scope.generatedTicket.queue = resp.plain();

                        if (typeof $scope.generatedTicket.servicePoint.id != 'undefined') {
                            APIServicePoint.loadOne($scope.generatedTicket.servicePoint.id).then(function (resp) {

                                $scope.generatedTicket.servicePoint = resp.plain();

                                if ($scope.printTicket) {
                                    $scope.navigateTo("ticketInfoForm");
                                    printTicket($scope.generatedTicket);
                                } else {
                                    if ($scope.smsTicket) {
                                        $scope.navigateTo("ticketInfoForm");

                                        var smsData = {
                                            ticketData: $scope.generatedTicket,
                                            smsProviderData: $scope.form.queue.tenant.smsProvider
                                        };

                                        APISms.sendSmsData(smsData).then(function (result) {
                                            if (result == null || typeof result.error != 'undefined') {
                                                // ha habido error en mandar el SMS
                                                $log.log("error mandar SMS");
                                                $scope.ticketSmsSuccess = "KO";
                                            } else {
                                                $scope.ticketSmsSuccess = "OK";
                                            }
                                        });
                                    } else {
                                        // unicamente mostrar los datos del ticket
                                        $scope.navigateTo("ticketInfoForm");
                                    }
                                }
                            });
                        }
                    });
                }
            });
        };

        var ticketData = function () {

            var t = {
                queue: {
                    id: $scope.form.queue.id
                },
                servicePoint: {
                    id: $scope.form.ticket.serviceId
                },
                issueChannel: {
                    id: 900     // issued from web
                },
                visitorMobile: $scope.form.ticket.phone,
                visitorName: ($scope.form.ticket.nombre.indexOf(' ') != -1) ? $scope.form.ticket.nombre.substr(0, $scope.form.ticket.nombre.indexOf(' ')) : $scope.form.ticket.nombre,
                visitorFamilyName: ($scope.form.ticket.nombre.indexOf(' ') != -1) ? $scope.form.ticket.nombre.substr($scope.form.ticket.nombre.indexOf(' ') + 1) : "",
                visitorEmail: $scope.form.ticket.email,
                vipScore: $scope.form.ticket.esCliente,
                ticketGenerics: []
            };

            // mapeamos la informacion adicional con los valores de los Generics
            if ($scope.model.ticketGenerics.length > 0) {

                if (typeof $scope.form.ticket.customerDescription != 'undefined' && $scope.form.ticket.customerDescription != "") {
                    if (typeof _.find($scope.model.ticketGenerics, ['id', 12011]) != 'undefined') {
                        var extendedData = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 12011]), // descripción del cliente
                            valor: $scope.form.ticket.customerDescription
                        };
                        t.ticketGenerics.push(extendedData);
                    }
                }

                if ($scope.form.ticket.dni != "") {
                    if (typeof _.find($scope.model.ticketGenerics, ['id', 11001]) != 'undefined') {
                        var extendedData = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11001]), // documento de identificacion,
                            valor: $scope.form.ticket.dni
                        };
                        t.ticketGenerics.push(extendedData);
                    }
                }

                if ($scope.form.ticket.info != "") {
                    if (typeof _.find($scope.model.ticketGenerics, ['id', 11002]) != 'undefined') {
                        var extendedData = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11002]),    // informacion
                            valor: $scope.form.ticket.info
                        };
                        t.ticketGenerics.push(extendedData);
                    }
                }

                if (typeof $scope.form.ticket.motivo != 'undefined' && $scope.form.ticket.motivo != "") {
                    if (typeof _.find($scope.model.ticketGenerics, ['id', 11003]) != 'undefined') {
                        var extendedData = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11003]), // motivo Orange
                            valor: $scope.form.ticket.motivo
                        };
                        t.ticketGenerics.push(extendedData);
                    }
                }
            }

            // mapeamos la infromacion del tipo de cliente
            if (typeof $scope.form.ticket.tipoCliente != 'undefined' && $scope.form.ticket.tipoCliente != 0 && $scope.model.tipoClientes.length > 0) {

                switch ($scope.form.ticket.tipoCliente) {

                    case 1:
                        if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') {
                            var extendedData = {
                                generic: _.find($scope.model.ticketGenerics, ['id', 11004]), // Tipo Cliente
                                valor: _.find($scope.model.tipoClientes, ['id', 12001]).name  // Particular
                            };
                            t.ticketGenerics.push(extendedData);
                        }
                        break;

                    case 2:
                        if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') {
                            var extendedData = {
                                generic: _.find($scope.model.ticketGenerics, ['id', 11004]), // Tipo Cliente
                                valor: _.find($scope.model.tipoClientes, ['id', 12002]).name // Empresa
                            };
                            t.ticketGenerics.push(extendedData);
                        }
                        break;

                    case 3:
                        if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') {
                            var extendedData = {
                                generic: _.find($scope.model.ticketGenerics, ['id', 11004]), // Tipo Cliente
                                valor: _.find($scope.model.tipoClientes, ['id', 12003]).name // Otro
                            };
                            t.ticketGenerics.push(extendedData);
                        }
                        break;
                }
            }

            return t;
        };

        var printTicket = function (ticket) {

            var ticketDesign = {
                htmlDesign: ""
            };

            // comprobamos y configuramos el correcto diseño del ticket
            // el diseño de ticket puede ser configurado a nivel de servicio o a nivel de tienda
            // si está definido a nivel de tienda este diseño prevalece
            APIQueue.loadOne(ticket.queue.id).then(function (resp) {
                ticket.queue = resp.plain();
                if (typeof ticket.queue.ticketDesign == 'undefined') {
                    APIServicePoint.loadOne(ticket.servicePoint.id).then(function (resp) {
                        ticket.servicePoint = resp.plain();
                        if (typeof ticket.servicePoint.ticketDesign != 'undefined') {
                            ticketDesign = ticket.servicePoint.ticketDesign;
                        } else {
                            // no hay ticket design configurado, imprimimos solo el número
                            ticketDesign.htmlDesign = ticket.name;
                        }
                    });
                } else {
                    ticketDesign = ticket.queue.ticketDesign;
                }

                $scope.model.ticket = ticket;

                $compile($("#printticket").html(ticketDesign.htmlDesign).contents())($scope);

                $timeout(function () {
                    $window.print();
                    return;
                }, 500);
            });
        };

        var getPoliticaPrivacidadData = function () {
            APIVariable.byNameAndTenant(sessionService.getTenantId(), 'POLITICA_PRIVACIDAD').then(function (r) {
                if (r !== undefined) {
                    $scope.model.politicaPrivacidad = r.plain();
                }
            });
        };

        var initializeNavigation = function () {
            document.body.scrollTop = document.documentElement.scrollTop = 0;
            $scope.customerDataForm = false;
            $scope.ticketInfoForm = false;
            $scope.ticketSmsOrPrintedForm = false;
        };

        var init = function () {
            if ($scope.ticket) {
                $scope.form.ticket = $scope.ticket;
            } else {
                $scope.cancel();
            }

            if ($scope.customerDescriptionArray) {
                $scope.model.customerDescriptionArray = $scope.customerDescriptionArray;
            } else {
                $scope.model.customerDescriptionArray = [];
            }

            // obtenemos información si existe Politica de Privacidad configurada
            getPoliticaPrivacidadData();

            // obtenemos información de la tienda, el ID está en la URL
            if (typeof $stateParams.id != 'undefined' && $stateParams.id != null) {
                APIQueue.loadOne($stateParams.id)
                    .then(function (data) {
                        $scope.form.queue = data.plain();
                        if ($scope.form.queue != {}) {
                            debugger;

                            return $scope.getQueueServices();
                            console.log("SE HA EJECUTADO LA FUNCION");

                        }
                    })
                    .then(function() {
                        // Este bloque se ejecutará después de que getQueueServices haya terminado
                        if ($scope.customerDescriptionForm == true) {
                            $scope.navigateTo("customerDescriptionForm");
                        } else if ($scope.selectServiceForm == true) {
                            $scope.navigateTo("selectServiceForm");
                        } else {
                            $scope.navigateTo("customerDataForm");
                        }
                    })
                    .catch(function(error) {
                        console.error("Error en la inicialización:", error);
                        // Maneja el error apropiadamente
                    });
            } else {
                // Si no hay ID, simplemente navega
                if ($scope.customerDescriptionForm == true) {
                    $scope.navigateTo("customerDescriptionForm");
                } else if ($scope.selectServiceForm == true) {
                    $scope.navigateTo("selectServiceForm");
                } else {
                    $scope.navigateTo("customerDataForm");
                }
            }
        };

        function isServiceSelected() {
            return $scope.form.ticket.serviceId && $scope.form.ticket.serviceId !== '0';
        }

        function completeTicketClean(){
            $scope.visibility.orange = false;
            $scope.visibility.jazztel = false;
            $scope.visibility.noCliente = false;
            $scope.visibility.fastTrack = false;
            $scope.ticketSmsOrPrintedForm = false;
        }

        init();

    })
    .controller('CustomerDescriptionModalCtrl', function ($scope, $uibModalInstance) {

        $scope.isSelected = function (value) {

            if ($scope.model.customerDescriptionArray.indexOf(value) === -1) {
                return false;
            }
            return true;
        };

        $scope.saveCustomerDesc = function (value) {
            if ($scope.model.customerDescriptionArray.indexOf(value) == -1) {
                $scope.model.customerDescriptionArray.push(value);
            } else {
                var index = $scope.model.customerDescriptionArray.indexOf(value);
                $scope.model.customerDescriptionArray.splice(index, 1);
            }
        };

        $scope.accept = function () {
            $uibModalInstance.close($scope.model.customerDescriptionArray);
        };

    });
