'use strict';

angular.module('bewin_orange')
    .controller('StationHomeController', function ($log, $scope, $state, $stateParams, $window, APIQueue,
                                                   APIUser, APIOperationsProfile, NavigationServices, sessionService) {
        $log.log("StationHomeController started ..");

        $scope.model = {
            queues: sessionService.user.queues,
            queue: {},
            workingUsers: [],
            users: [],
            user: sessionService.user,
            workProfiles: []
        };

        $scope.form = {
            queue: null,
            workProfile: null,
            stations: [],
            currentQueue: {},
            currentStation: {},
            currentWorkProfile: {},
            filter: []
        };

        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.selectWorkProfile = function (wp) {

            $scope.form.workProfile = wp;
        };

        $scope.nameQueueSort = 'name';
        $scope.nameStationSort = 'name';


        $scope.selectQueue = function (queueId) {

            APIQueue.loadOne(queueId).then(function (q) {

                $scope.model.queue = q.plain();

                if (!sessionService.isServing) {
                    sessionService.setQueue($scope.model.queue);
                }

                $scope.form.stations = $scope.model.queue.stations;
                $scope.form.currentQueue = {};
                $scope.form.currentStation = {};
                $scope.form.currentWorkProfile = {};
                $scope.form.workProfile = null;

                APIUser.workingInQueue(queueId).then(function (users) {
                    users = users.plain() || [];

                    users.map(function (u) {

                        if (typeof u.currentStation != 'undefined') {

                            var k = _.find($scope.form.stations, {id: u.currentStation.id});
                            if (k) {
                                console.log("k >", k);
                                k.user = u;
                            }
                        }
                    });
                });
            });
        };

        $scope.loadWorkProfiles = function (s) {

            $log.log("loadWorkProfiles s>", s);

            $scope.form.currentQueue = $scope.model.queue;
            $scope.form.currentStation = s;
            $scope.form.workProfile = null;
            $scope.model.workProfiles = [];

            APIOperationsProfile.loadOne($scope.form.currentQueue.operationsProfile.id).then(function (op) {

                var workProfiles = op.workProfiles;

                APIUser.workprofiles(sessionService.getUserId()).then(function (wp) {
                    _.each(wp, function (w) {
                        if (_.find(workProfiles, {id: w.id})) {
                            $scope.model.workProfiles.push(w);
                        }
                    });

                    if ($scope.model.workProfiles.length == 1) {
                        $scope.selectWorkProfile($scope.model.workProfiles [0]);
                    }

                    if (typeof $scope.model.user.currentWorkProfile != 'undefined') {

                        var k = _.find($scope.model.workProfiles, {id: $scope.model.user.currentWorkProfile.id});
                        $scope.selectWorkProfile(k);
                    }
                });
            });
        };

        $scope.joinStation = function (wp) {
            var user = sessionService.getUser();

            $log.log("joinstation user>", user);
            $log.log("joinstation wp>", wp);

            user.currentStation = $scope.form.currentStation;
            user.currentWorkProfile = wp;

            sessionService.setUser(user);

            APIUser.joinStation(sessionService.getUserId(), sessionService.getQueueId(), sessionService.getWorkProfileId(), sessionService.getStationId()).then(function () {
                $state.go('station.serve', {
                    queueId: sessionService.getQueueId(),
                    stationId: sessionService.getStationId()
                });
            });

        };

        $scope.cleanFilter = function () {
            $scope.form.filter = [];
        };

        var init = function () {
            if (sessionService.getQueueId() !== null) {
                // el usuario tiene elegido la oficina
                $scope.selectQueue(sessionService.getQueueId());
            }
        };

        init();

    });
