'use strict';

angular.module('bewin_orange')
    .controller('ServeStationController', function ($filter, $interval, $log, $q, $scope, $state, $stateParams, $timeout, $uibModal, $window,
                                                    APIGeneric, APIQueue, APIUser, APITicket, APIVariable, APIEncuesta, APIOperationsProfile,
                                                    APITicketCloseInfo, APIServicePoint, APIWorkProfile, APITicketMarcaInfo, APICapacity,
                                                    modalService, NavigationServices, sessionService, toastr, ORANGE_QUEUES) {

        /* Variables */
        $scope.model = {
            queue: [],
            ticket: {},
            workProfiles: [],
            waitingTickets: [],
            userTickets: [],
            appointmentTickets: [],
            variablesUsers: [],
            ticketGenerics: APIGeneric.byType(11).$object,
            tipoCliente: APIGeneric.byType(12).$object,
            user: sessionService.user,
            editClientData: null,
            puesto: {},
            queueEncuesta: {
                esActiva: null,
                nombreTienda: null,
                codigoTienda: null
            },
            politicaPrivacidad: null,
            encuestaSuccess: null,
            encuestaNoPresentados: null,
            zone: null,
            capacity: 0
        };

        $scope.form = {
            workProfile: null,
            visitor: {
                nombre: "",
                phone: "",
                email: "",
                isCliente: 0,
                tipoCliente: 'Otro',
                info: "",
                dni: "",
                motivo: "",
                politica: true
            }
        };

        $scope.orderByField = '';
        $scope.reverseSort = false;

        $scope.buttonNextSpinner = false;
        $scope.buttonClienteSinTicketSpinner = false;
        $scope.buttonCloseTicketSpinner = false;
        $scope.buttonNoPresentadoSpinner = false;
        $scope.buttonRecallTicketSpinner = false;
        $scope.buttonReenviarTicket = false;
        $scope.isDataLoading = false;

        $scope.showTicketForm = false;

        $scope.queueHasCounter = false;

        var refreshInterval;

        /* END Variables */


        /**
         * MÉTODOS DE LOS BOTONES
         */
        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.callTicket = function (st) {

            $scope.isDataLoading = true;

            if ($scope.model.ticket && $scope.model.ticket.id !== undefined) {
                showWarningForClosingTickect();
                $scope.isDataLoading = false;
            } else {
                APITicket.loadOne(st.id).then(function (resp) {
                    st = resp.plain();
                    // comprobar si hay turnos con cita y si alguno de ellos tiene un issue_time inferior al del turno que se esta llamando
                    if ($scope.model.appointmentTickets.length > 0) {
                        var appointmentConfig = sessionService.user.tenant.tenantAppointmentConfig;
                        if (appointmentConfig !== undefined && appointmentConfig.allowOverwrite !== undefined && !appointmentConfig.allowOverwrite) {
                            $scope.model.appointmentTickets = _.sortBy($scope.model.appointmentTickets, 'scheduledServiceTime');
                            if ($scope.model.appointmentTickets[0].id !== st.id && moment($scope.model.appointmentTickets[0].scheduledServiceTime).isBefore(moment(st.issueTime))) {
                                var msg = 'Existen turnos con cita previa anteriores al turno que está intentando llamar. Por favor compruebe el listado de turnos preferentes.';
                                modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
                                $scope.isDataLoading = false;
                            } else {
                                proceedCall(st);
                            }
                        } else {
                            proceedCall(st);
                        }
                    } else {
                        proceedCall(st);
                    }
                });
            }
        };

        $scope.callNextTicket = function () {

            $scope.buttonNextSpinner = true;
            $scope.isDataLoading = true;

            // si estamos atendiendo un turno, mostramos el dialogo de cierre para todas las tiendas menos tdb
            if (sessionService.isServing && typeof $scope.model.ticket !== 'undefined' && !angular.equals({}, $scope.model.ticket)) {

                var closeAtentionModalWithManagementForm = $uibModal.open({
                    templateUrl: 'views/station/serve/closetAtentionWithManagementModalForm.html',
                    controller: 'CloseAtentionWithManagementModalCtrl',
                    backdrop: 'static',
                    keyboard: false,
                    scope: $scope,
                    resolve: {
                        ticket: function () {
                            return $scope.model.ticket;
                        }
                    }
                });

                closeAtentionModalWithManagementForm.result.then(function (ticketCloseInfo) {

                    saveTicketCloseInfo(ticketCloseInfo);

                    readTicketForm();

                    APITicket.markTicketAsServiced($scope.model.ticket).then(function (res) {
                        markAsSevicedCallback(res);
                        getNextTicket();
                    });
                });
            } else {
                sessionService.isServing = false;
                getNextTicket();
            }
        };

        $scope.clienteSinTicket = function (service, isClienteSinTicket) {

            $scope.isDataLoading = true;
            $scope.buttonClienteSinTicketSpinner = true;

            if (isClienteSinTicket) {
                // es un turno para cliente sin ticket - comprobamos si está serviendo y primero cerramos ese ticket antes de crear el nuevo
                if (sessionService.isServing && typeof $scope.model.ticket !== 'undefined' && !angular.equals({}, $scope.model.ticket)) {

                    readTicketForm();

                    // cerrar el ticket
                    APITicket.markTicketAsServiced($scope.model.ticket).then(function (res) {

                        markAsSevicedCallback(res);
                        $scope.issueTicket(service, isClienteSinTicket);
                        $scope.buttonClienteSinTicketSpinner = false;
                        $scope.isDataLoading = false;
                    });
                } else {
                    sessionService.isServing = false;
                    $scope.issueTicket(service, isClienteSinTicket);
                }
            }
        };

        $scope.closeTicket = function () {

            $scope.isDataLoading = true;
            $scope.buttonCloseTicketSpinner = true;

            var ticket = $scope.model.ticket;

            var closeAtentionModalWithManagementForm = $uibModal.open({
                templateUrl: 'views/station/serve/closetAtentionWithManagementModalForm.html',
                controller: 'CloseAtentionWithManagementModalCtrl',
                backdrop: 'static',
                keyboard: false,
                scope: $scope,
                resolve: {
                    ticket: function () {
                        return ticket;
                    }
                }
            });

            closeAtentionModalWithManagementForm.result.then(function (ticketCloseInfo) {
                saveTicketCloseInfo(ticketCloseInfo);
                return;
            });

            readTicketForm();

            APITicket.markTicketAsServiced($scope.model.ticket).then(function (res) {

                $scope.buttonCloseTicketSpinner = false;
                $scope.isDataLoading = false;
                markAsSevicedCallback(res);
            });
        };

        $scope.notPresented = function (ticket) {
            $scope.isDataLoading = true;
            $scope.buttonNoPresentadoSpinner = true;
            $scope.showTicketForm = false;

            if (ticket !== undefined && ticket.id !== undefined) {
                $scope.model.ticket = ticket;
            }

            readTicketForm();

            $scope.model.ticket.station = {
                id: sessionService.user.currentStation.id
            };

            APITicket.markTicketAsNotPresented($scope.model.ticket).then(function (res) {

                $scope.buttonNoPresentadoSpinner = false;
                $scope.buttonNextSpinner = false;
                $scope.isDataLoading = false;

                $scope.loadWaitLine();

                markAsSevicedCallback(res);
            });

        };

        $scope.recallTicket = function () {
            $scope.isDataLoading = true;
            $scope.buttonRecallTicketSpinner = true;

            APITicket.recallTicket($scope.model.ticket.id).then(function (resp) {
                $scope.buttonRecallTicketSpinner = false;
                $scope.isDataLoading = false;
                $scope.model.ticket = resp.plain();
                $scope.loadWaitLine();
            });

        };

        $scope.ventanaRedireccion = function (st, cb) {

            $scope.isDataLoading = true;

            // actualizar primero los datos del formulario por si hubo cambios
            if (cb) {
                readTicketForm();
            }

            if (typeof $scope.model.queue !== 'undefined' && $scope.model.queue.id != null) {

                deferredFn().then(function () {
                    var modalInstanceServeOrangeReenvia = $uibModal.open({
                        templateUrl: 'views/station/serve/modal_reenvio_servicio_puesto.html',
                        controller: 'ReenvioServiceUserCtrl',
                        backdrop: 'static',
                        keyboard: false,
                        resolve: {
                            ticket: function () {
                                return st;
                            },
                            ticketServiced: function () {
                                return $scope.model.ticket;
                            },
                            services: function () {
                                return $scope.model.redirectServices;
                            },
                            users: function () {
                                return $scope.model.redirectUsers;
                            }
                        }
                    });

                    modalInstanceServeOrangeReenvia.result.then(function (t) {
                            if (t !== null) {
                                if ($scope.model.ticket === undefined || $scope.model.ticket.id === t.id) {
                                    $scope.model.ticket = {};
                                }
                            }
                            $scope.isDataLoading = false;
                            $scope.loadWaitLine();
                        }
                    );
                });
            }
        };

        /**
         * ORDENACIÓN
         */
        $scope.sortBy = function (field, collectionName, collection) {
            $scope.orderByField = field;
            $scope.reverseSort = !$scope.reverseSort;

            $scope.model[collectionName] = $filter('orderBy')(collection, $scope.orderByField, $scope.reverseSort);
        };

        $scope.loadWaitLine = function () {

            APITicket.waitLineForUser(sessionService.getUserId()).then(function (f) {

                // mostramos solo los tickets que pueden ser servidos por el usuario
                $scope.model.waitingTickets = [];
                $scope.model.appointmentTickets = [];
                if (f.length > 0) {
                    $scope.model.waitingTickets = f;
                    $scope.model.appointmentTickets = [];

                    _.each($scope.model.waitingTickets, function (t) {

                        if (t.scheduledServiceTime === undefined) {
                            var dateIssueTime = new Date(t.issueTime);

                            if (moment(dateIssueTime).isBefore(moment(new Date()))) {
                                t.waitTimeSeconds = parseInt(($scope.clock - dateIssueTime) / 1000);
                            }
                        } else {
                            var m = _.find($scope.model.appointmentTickets, ['id', t.id]);
                            if (!m) {
                                $scope.model.appointmentTickets.push(t);
                            }
                        }
                    });

                    $scope.model.appointmentTickets = $filter('orderBy')($scope.model.appointmentTickets, 'scheduledServiceTime', false);
                }

                APITicket.waitRedirectLineForUser(sessionService.getUserId()).then(function (resp) {
                    $scope.model.userTickets = [];

                    if (resp.length > 0) {
                        $scope.model.userTickets = resp;
                    }
                });
            });

            if ($scope.queueHasCounter) {
                APICapacity.findCapacity($scope.model.queue.internalCode).then(function (capacity) {
                    if (capacity !== undefined) {
                        $scope.model.capacity = Math.round(capacity * 100 / $scope.model.zone.max_capacity);
                    } else {
                        $scope.model.capacity = '-';
                    }
                });
            }
        };

        $scope.serviceTime = function (t) {

            var d = new Date();
            d.setHours(0, 0, 0, 0);

            if (typeof t == 'undefined' || t == undefined) return d;

            if (typeof $scope.clock == 'undefined' || $scope.clock == undefined) return d;

            if (!t) return d;

            var dateCalledInTime = $scope.parseTimeZone(t);
            var serviceTime = $scope.clock - dateCalledInTime;
            if (serviceTime < 0) return d;

            return new Date(1970, 0, 1).setMilliseconds(serviceTime);

        };

        $scope.tiempoEsperaCalled = function (t) {
            if (!t) return 0;
            var dateIssueTime = new Date(t.issueTime);
            var dateCalledInTime = new Date($scope.parseTimeZone(t.calledInTime));
            return msTodhms(dateCalledInTime - dateIssueTime);
        };

        $scope.tiempoEspera = function (t) {
            if (!t) return 0;
            var dateIssueTime = $scope.parseTimeZone(t.issueTime);
            if (moment(dateIssueTime).isAfter(moment($scope.clock))) {
                return '00:00:00';
            } else {
                return msTodhms($scope.clock - dateIssueTime);
            }
        };

        $scope.inSLA = function (t) {
            if (!t.slaTime) return true;
            var d = new Date();
            var dateSlaTime = new Date(t.slaTime);
            return moment(d).isBefore(dateSlaTime);
        };

        $scope.selectWorkProfile = function (wp) {
            $scope.form.workProfile = wp;
            APIUser.joinStation(sessionService.getUserId(), $scope.model.queue.id, $scope.form.workProfile.id, $stateParams.stationId).then(function (resp) {
                var user = sessionService.getUser();
                user.currentQueue = resp.currentQueue;
                user.currentWorkProfile = resp.currentWorkProfile;
                user.currentStation = resp.currentStation;

                sessionService.setUser(user);

                $scope.loadWaitLine();
            });
        };

        $scope.encuestaSuccess = function (res) {
            $scope.model.encuestaSuccess = res;
            $timeout(function () {
                $scope.model.encuestaSuccess = null;
            }, 5000);
        };

        $scope.initInterval = function () {

            $scope.cancelInterval();

            refreshWindow();

            refreshInterval = $interval(refreshWindow, 5000);
        };

        $scope.cancelInterval = function () {
            $interval.cancel(refreshInterval);
        };

        $scope.$on('$destroy', function (e) {
            $scope.cancelInterval();
        });

        $scope.issueTicket = function (service, isClienteSinTicket) {

            var t = {
                queue: {
                    id: $scope.model.queue.id
                },

                servicePoint: {
                    id: service.id
                },

                issueChannel: {
                    id: 900
                },

                visitor: null
            };

            if (isClienteSinTicket) {
                t.assignedTo = {
                    id: sessionService.getUserId()
                };

                t.ticketGenerics = [];
                var extendedData = {
                    generic: _.find($scope.model.ticketGenerics, ['id', 11009]), // cliente sin ticket,
                    valor: 1
                };
                t.ticketGenerics.push(extendedData);
            }

            $log.log("Issuing ticket!");

            APITicket.issueTicket(t).then(function (nt) {
                //LLamamos al ticket tras ser emitido
                APITicket.callTicket(sessionService.getUserId(), nt.id).then(function (t) {

                    $scope.model.ticket = t;

                    //Pasamos la hora a la zona horaria correspndiente
                    $scope.model.ticket.issueTime =   $scope.parseTimeZone($scope.model.ticket.issueTime);

                    sessionService.isServing = true;

                    if (typeof $scope.model.ticket !== undefined && !angular.equals({}, $scope.model.ticket)) {
                        loadVisitorData($scope.model.ticket);
                    }

                    checkTicket(t);

                    $scope.buttonClienteSinTicketSpinner = false;
                    $scope.isDataLoading = false;

                    $scope.showTicketForm = true;

                    $scope.loadWaitLine();
                });


            });
        };

        $scope.mapTicketAdditionalData = function (ticket) {

            if (!ticket) {
                return null;
            }

            if (ticket.servicePoint.id == ORANGE_QUEUES.Cliente_AtencionCliente_sinCita
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_InfoComercial_Empresas_sinCita
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_InfoComercial_Empresas_conCita
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_AtencionCliente_conCita
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_Ayuda_Sobre_TuServicio) {

                ticket.vipScore = 1;

                if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') { // tipoCliente

                    var k = _.find(ticket.ticketGenerics, ['id', 11004]);

                    if (typeof k != 'undefined') {
                        k.valor = 'Empresa';
                    } else {
                        var dataTipo = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11004]),
                            valor: 'Empresa'
                        };
                        ticket.ticketGenerics.push(dataTipo);
                    }
                }
            } else if (ticket.servicePoint.id == ORANGE_QUEUES.Cliente_InfoComercial_Particulares
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_RecargasPrepago
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_TuExperto
                || ticket.servicePoint.id == ORANGE_QUEUES.Cliente_InfoComercial_Particulares_Cita) {

                ticket.vipScore = 1;

                if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') { // tipoCliente

                    var k = _.find(ticket.ticketGenerics, ['id', 11004]);

                    if (typeof k != 'undefined') {
                        k.valor = 'Particular';
                    } else {
                        var dataTipo = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11004]),
                            valor: 'Particular'
                        };
                        ticket.ticketGenerics.push(dataTipo);
                    }
                }

            } else if (ticket.servicePoint.id == ORANGE_QUEUES.FT_Duplicados_SIM
                || ticket.servicePoint.id == ORANGE_QUEUES.FT_Info_Accesorios
                || ticket.servicePoint.id == ORANGE_QUEUES.FT_RecogidaPedidos
                || ticket.servicePoint.id == ORANGE_QUEUES.Reparaciones
                || ticket.servicePoint.id == ORANGE_QUEUES.Entrega_Router
                || ticket.servicePoint.id == ORANGE_QUEUES.ClienteJazztel_Informacion_comercial
                || ticket.servicePoint.id == ORANGE_QUEUES.ClienteJazztel_DuplicadoSIM
                || ticket.servicePoint.id == ORANGE_QUEUES.ClienteJazztel_Consulta) {

                ticket.vipScore = 1;

                if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') { // tipoCliente

                    var k = _.find(ticket.ticketGenerics, ['id', 11004]);

                    if (typeof k != 'undefined') {
                        k.valor = 'Otro';
                    } else {
                        var dataTipo = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11004]),
                            valor: 'Otro'
                        };
                        ticket.ticketGenerics.push(dataTipo);
                    }
                }
            } else if (ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_InfoComercial_Empresas_conCita
                || ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_InfoComercial_Empresas_sinCita) {

                ticket.vipScore = 0;

                if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') { // tipoCliente

                    var k = _.find(ticket.ticketGenerics, ['id', 11004]);

                    if (typeof k != 'undefined') {
                        k.valor = 'Empresa';
                    } else {
                        var dataTipo = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11004]),
                            valor: 'Empresa'
                        };
                        ticket.ticketGenerics.push(dataTipo);
                    }
                }
            } else if (ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_FT_Info_Accesorios
                || ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_RecargasPrepago
                || ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_InfoComercial_Particulares
                || ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_InfoComercial_Particulares_Cita
                || ticket.servicePoint.id == ORANGE_QUEUES.NoCliente_TuExperto) {

                ticket.vipScore = 0;

                if (typeof _.find($scope.model.ticketGenerics, ['id', 11004]) != 'undefined') { // tipoCliente

                    var k = _.find(ticket.ticketGenerics, ['id', 11004]);

                    if (typeof k != 'undefined') {
                        k.valor = 'Particular';
                    } else {
                        var dataTipo = {
                            generic: _.find($scope.model.ticketGenerics, ['id', 11004]),
                            valor: 'Particular'
                        };
                        ticket.ticketGenerics.push(dataTipo);
                    }
                }
            }

            return ticket;
        };

        $scope.tiempoMedioEspera = function (waitTickets) {

            var te = 0;

            if (waitTickets.length == 0) {
                return "00:00:00";
            }

            var maxTicketsWaiting = 0;
            _.each(waitTickets, function (wt) {
                //var ts = wt.scheduledServiceTime || wt.issueTime;
                var dateIssueTime = $scope.parseTimeZone(wt.issueTime);
                if (moment(dateIssueTime).isBefore(moment($scope.clock))) {
                    te = te + ($scope.clock - dateIssueTime);
                    // hay que tener en cuenta solo aquellos tickets con tiempo de espera positivo (descartando tickets
                    // de cita que aun no han llegado)
                    maxTicketsWaiting = maxTicketsWaiting + 1;
                }
            });
            return msTodhms(te / maxTicketsWaiting);
        };

        $scope.openEureka = function (phone, dni) {
            if (phone == null || phone.length < 9 || phone.length > 9) {
                // abrimos un modal con advertencia
                var msg = 'El campo Teléfono no es correcto o está vacío';
                modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
            } else {
                // abrimos en una nueva ventana la URL de Eureka

                var eurekaUrl = 'https://eureka.orange.es/EUREKA3/DashboardResumenLineasAction.action?';

                if (dni != null) {
                    eurekaUrl += 'nif=' + dni + '&';
                } else {
                    eurekaUrl += 'nif=&';
                }

                eurekaUrl += 'msisdn=' + phone + '&app=QMATIC';

                var h = $window.innerHeight * 0.9;
                var w = $window.innerWidth * 0.8;
                var left = ($window.innerWidth / 2) - (w / 2);
                var top = ($window.innerHeight / 2) - (h / 2);
                $window.open(eurekaUrl, '_blank', 'width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);
            }
        };

        $scope.isTicketAssignable = function (t) {
            if ($scope.model.user.supportsRole('STATION_ROLE')) {
                if (t.assignedTo === undefined || t.assignedTo.id === $scope.model.user.id) {
                    if (t.scheduledServiceTime === undefined) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            } else {
                return false;
            }
        };

        /**
         * MÉTODOS PRIVADOS
         */

        var getNextTicket = function () {

            // comprobamos primero si hay tickets reenviados
            if ($scope.model.userTickets.length > 0) {
                var i = 0;
                while (i < $scope.model.userTickets.length) {
                    if ($scope.model.userTickets[i].assignedTo !== undefined && $scope.model.userTickets[i].assignedTo.id === sessionService.getUserId()) {

                        proceedCall($scope.model.userTickets[i]);

                        i = $scope.model.userTickets.length;
                    }
                    i += 1;
                }
            } else {
                APITicket.findNext(sessionService.getUserId()).then(function (t) {
                    proceedCall(t);
                });
            }
        };

        var callTicket = function (t) {

            $scope.model.ticket = t;
            sessionService.isServing = true;

            if (typeof $scope.model.ticket != 'undefined' && !angular.equals({}, $scope.model.ticket)) {
                loadVisitorData($scope.model.ticket);
            }

            checkTicket(t);

            $scope.loadWaitLine();
        };

        var proceedCall = function (st) {

            if (st === undefined || st === null) {
                checkTicket(st);
                $scope.buttonNextSpinner = false;
                $scope.isDataLoading = false;
            } else {
                var modalInstance = $uibModal.open({
                    templateUrl: 'views/station/serve/modal_call_ticket.html',
                    controller: 'CallTicketCtrl',
                    scope: $scope,
                    backdrop: 'static',
                    keyboard: false,
                    resolve: {
                        ticket: function () {
                            return st;
                        }
                    }
                });

                modalInstance.result.then(function (ok) {

                    APITicket.loadOne(st.id).then(function (t) {
                        // buscamos el ticket que hemos llamado para ver si esta en uso por otro usuario
                        if (t && t.id && t.servicedBy !== undefined && sessionService.getUserId() !== t.servicedBy.id) {
                            sessionService.isServing = false;
                            $scope.isDataLoading = false;
                            $scope.buttonNextSpinner = false;
                            toastr.warning("Turno servido por otro usuario", "No se puede ejecutar la operación");
                        } else {
                            // si no esta en uso podemos llamarlo o marcarlo como no presentado
                            if (t.assignedTo !== undefined && t.assignedTo.id !== undefined && t.assignedTo.id !== sessionService.getUserId()) {
                                toastr.error("Turno asignado a otro usuario", "No se puede ejecutar la operación");

                                sessionService.isServing = false;

                                $scope.isDataLoading = false;
                                $scope.buttonNextSpinner = false;

                                $scope.showTicketForm = false;

                                $scope.loadWaitLine();
                            } else {
                                if (ok) {
                                    APITicket.callTicket(sessionService.getUserId(), t.id).then(function (ticket) {

                                        var offset = moment().utcOffset();
                                        var serverOffset = moment().tz("Europe/Madrid").utcOffset();
                                        var difference = parseInt(offset) - parseInt(serverOffset);
                                        ticket.issueTime = moment(t.issueTime).add(difference, 'm').toDate();
                                        // para que no aparezca como flash el ticket si esta marcado como NO PRESENTADO
                                        $scope.model.ticket = ticket;
                                        sessionService.isServing = true;
                                        loadVisitorData($scope.model.ticket);

                                        checkTicket(t);

                                        $scope.isDataLoading = false;
                                        $scope.buttonNextSpinner = false;

                                        $scope.showTicketForm = true;

                                        $scope.loadWaitLine();
                                    });
                                } else {
                                    $scope.notPresented(t);
                                }
                            }
                        }
                    });
                });
            }
        };

        var checkTicket = function (t) {
            if (!t || !t.id) {
                sessionService.isServing = false;
                toastr.info("No hay turnos en espera que sea posible atender.", "Operación ejecutada");
            }
        };

        var markAsSevicedCallback = function (res) {
            // comprobamos si para la tienda hay configurados datos para enviar la encuesta a Madison
            if ($scope.model.queueEncuesta.esActiva && $scope.model.queueEncuesta.codigoTienda != null && $scope.model.queueEncuesta.nombreTienda != null) {
                // comprobamos si el ticket tiene asociado un numero de movil para enviar la encuesta
                if (typeof res.visitorMobile != 'undefined' && res.visitorMobile != null && res.visitorMobile != "") {
                    // comprobamos si el ticket esta marcado como No presentado
                    if (typeof res.resolution == 'undefined' || res.resolution != 1) {
                        enviarEncuestaMadison(res);
                    } else {
                        // si el ticket está marcado como No presentado comprobamos el valor de la variable ENCUESTA_NO_PRESENTADOS
                        // si la variable ENCUESTA_NO_PRESENTADOS no esta definida NO enviamos encuesta a Madison
                        // si la variable ENCUESTA_NO_PRESENTADOS = 1 SI enviamos encuesta a Madison
                        // si la variable ENCUESTA_NO_PRESENTADOS = 0 NO enviamos encuesta a Madison
                        if ($scope.model.encuestaNoPresentados != null && $scope.model.encuestaNoPresentados.value === "1") {
                            enviarEncuestaMadison(res);
                        }
                    }
                }
            }

            $scope.model.ticket = {};
            sessionService.isServing = false;
            toastr.info("Ticket cerrado con éxito");
        };

        var enviarEncuestaMadison = function (res) {
            APIEncuesta.sendEncuestaData(res, $scope.model.queueEncuesta).then(function (data) {
                if (data == null) {
                    $scope.encuestaSuccess('KO');
                } else {
                    if (data.status == 1) {
                        $scope.encuestaSuccess('OK');
                    }
                }
            });
        };

        var showWarningForClosingTickect = function () {
            var msg = 'Por favor, cierre el ticket en servicio antes de llamar al siguiente.';
            modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
        };

        var loadWorkProfiles = function (q) {

            $scope.model.workProfiles = [];

            if (q.operationsProfile.workProfiles) {

                APIUser.workprofiles($scope.model.user.id).then(function (wp) {

                    _.each(wp, function (w) {
                        if (_.find(q.operationsProfile.workProfiles, {id: w.id})) {
                            $scope.model.workProfiles.push(w);
                        }
                    });

                    // configuramos el Work Profile que ha seleccionado el usuario
                    $scope.form.workProfile = _.find($scope.model.workProfiles, ['id', $scope.model.user.currentWorkProfile.id]);

                    if ($scope.model.workProfiles.length == 1) {
                        $scope.selectWorkProfile($scope.model.workProfiles [0]);
                    }
                });
            }
        };

        var msTodhms = function (ms) {

            var cd = 24 * 60 * 60 * 1000,
                ch = 60 * 60 * 1000,
                cm = 60 * 1000,
                d = Math.floor(ms / cd),
                h = Math.floor((ms - d * cd) / ch),
                m = Math.floor((ms - d * cd - h * ch) / cm),
                s = Math.round((ms - d * cd - h * ch - m * cm) / 1000),
                pad = function (n) {
                    return n < 10 ? '0' + n : n;
                };

            if (s === 60) {
                m++;
                s = 0;
            }
            if (m === 60) {
                h++;
                m = 0;
            }
            if (h === 24) {
                d++;
                h = 0;
            }

            var time = (d > 0) ? d + " días, " : "";
            time += pad(h) + ":" + pad(m) + ":" + pad(s);

            return time;
        };

        var loadVisitorData = function (ticketData) {

            $scope.form.visitor = {
                nombre: "",
                phone: "",
                email: "",
                isCliente: 0,
                tipoCliente: 'Otro',
                info: "",
                dni: "",
                motivo: "",
                politica: true
            };

            if (typeof ticketData.visitorName != 'undefined') {
                $scope.form.visitor.nombre = ticketData.visitorName;
            }
            if (typeof ticketData.visitorFamilyName != 'undefined') {
                $scope.form.visitor.nombre += " " + ticketData.visitorFamilyName;
            }

            $scope.form.visitor.phone = ticketData.visitorMobile;
            $scope.form.visitor.email = ticketData.visitorEmail;

            $scope.form.visitor.isCliente = ticketData.vipScore;

            if (ticketData.ticketGenerics.length > 0) {

                var infoFiltered = ticketData.ticketGenerics.filter(function (data) {
                    if (data.generic.id == '11002') {
                        return data;
                    } else {
                        return "";
                    }
                });
                if (infoFiltered.length > 0) {
                    $scope.form.visitor.info = infoFiltered[0].valor;
                }

                var dniFiltered = ticketData.ticketGenerics.filter(function (data) {
                    if (data.generic.id == '11001') {
                        return data;
                    } else {
                        return "";
                    }
                });
                if (dniFiltered.length > 0) {
                    $scope.form.visitor.dni = dniFiltered[0].valor;
                }

                var motivoFiltered = ticketData.ticketGenerics.filter(function (data) {
                    if (data.generic.id == '11003') {
                        return data;
                    } else {
                        return "";
                    }
                });
                if (motivoFiltered.length > 0) {
                    $scope.form.visitor.motivo = motivoFiltered[0].valor;
                }

                var tipoFiltered = ticketData.ticketGenerics.filter(function (data) {
                    return (data.generic.id == '11004');
                });

                if (tipoFiltered.length > 0) {
                    $scope.form.visitor.tipoCliente = tipoFiltered[0].valor;
                } else {
                    $scope.form.visitor.tipoCliente = 'Otro';
                }
            }

        };

        var readTicketForm = function () {

            console.log("readTicketForm");
            console.log($scope.model.ticket);

            console.log($scope.form);

            if ($scope.form.visitor.nombre !== undefined && $scope.form.visitor.nombre != '') {
                $scope.model.ticket.visitorName =
                    ($scope.form.visitor.nombre.indexOf(' ') != -1) ? $scope.form.visitor.nombre.substr(0, $scope.form.visitor.nombre.indexOf(' ')) : $scope.form.visitor.nombre;
                $scope.model.ticket.visitorFamilyName =
                    ($scope.form.visitor.nombre.indexOf(' ') != -1) ? $scope.form.visitor.nombre.substr($scope.form.visitor.nombre.indexOf(' ') + 1) : "";
            } else {
                $scope.model.ticket.visitorName = null;
                $scope.model.ticket.visitorFamilyName = null;
            }

            if ($scope.form.visitor.phone !== undefined && $scope.form.visitor.phone != '') {
                $scope.model.ticket.visitorMobile = $scope.form.visitor.phone;
            } else {
                $scope.model.ticket.visitorMobile = null;
            }

            if ($scope.form.visitor.email !== undefined && $scope.form.visitor.email != '') {
                $scope.model.ticket.visitorEmail = $scope.form.visitor.email;
            } else {
                $scope.model.ticket.visitorEmail = null;
            }

            $scope.model.ticket.visitor = null;

            $scope.model.ticket.ticketGenerics = [];

            if ($scope.form.visitor.info != undefined && $scope.form.visitor.info != '') {
                if (typeof _.find($scope.model.ticketGenerics, ['id', 11002]) != 'undefined') {
                    var data1 = {
                        generic: _.find($scope.model.ticketGenerics, ['id', 11002]), // Informacion
                        valor: $scope.form.visitor.info
                    };
                    $scope.model.ticket.ticketGenerics.push(data1);
                }
            }

            if ($scope.form.visitor.dni != undefined && $scope.form.visitor.dni != '') {
                if (typeof _.find($scope.model.ticketGenerics, ['id', 11001]) != 'undefined') {
                    var data2 = {
                        generic: _.find($scope.model.ticketGenerics, ['id', 11001]), // documento de identificacion
                        valor: $scope.form.visitor.dni
                    };
                    $scope.model.ticket.ticketGenerics.push(data2);
                }
            }

            if ($scope.form.visitor.motivo !== undefined && $scope.form.visitor.motivo != '') {
                if (typeof _.find($scope.model.ticketGenerics, ['id', 11003]) != 'undefined') {
                    var data3 = {
                        generic: _.find($scope.model.ticketGenerics, ['id', 11003]), // motivo de la visita
                        valor: $scope.form.visitor.motivo
                    };
                    $scope.model.ticket.ticketGenerics.push(data3);
                }
            }

            $scope.model.ticket.vipScore = ($scope.model.ticket.vipScore !== undefined) ? $scope.model.ticket.vipScore : null;

            $scope.model.ticket = $scope.mapTicketAdditionalData($scope.model.ticket);
        };

        var getServingTicket = function () {
            // comprobamos si el usuario tiene ticket que no ha cerrado
            APIUser.servingTicket($scope.model.user.id).then(function (t) {
                if (t) {
                    if ($scope.model.ticket === undefined || $scope.model.ticket.id !== t.id) {
                        $scope.model.ticket = t;

                        $scope.model.ticket.issueTime = $scope.parseTimeZone(t.issueTime);

                        sessionService.isServing = true;
                        $scope.showTicketForm = true;
                        loadVisitorData($scope.model.ticket);
                    }
                } else {
                    $scope.model.ticket = {};
                    sessionService.isServing = false;
                    $scope.showTicketForm = false;
                }
            });
        };

        var getEditClientData = function () {
            APIVariable.byNameAndTenant(sessionService.getTenantId(), 'EDIT_VISITOR_DATA').then(function (r) {
                $scope.model.editClientData = r;
            });
        };

        var getPoliticaPrivacidadData = function () {
            APIVariable.byNameAndTenant(sessionService.getTenantId(), 'POLITICA_PRIVACIDAD').then(function (r) {
                $scope.model.politicaPrivacidad = r;
            });
        };

        var getEncuestaData = function () {
            APIVariable.byNameAndQueue($scope.model.queue.id, 'ENCUESTA_ACTIVA').then(function (r) {

                if (typeof r == 'undefined') {
                    $scope.model.queueEncuesta.esActiva = false;
                } else {
                    // la variable existe
                    if (r.value === 'true') {
                        $scope.model.queueEncuesta.esActiva = true;
                        $scope.model.queueEncuesta.nombreTienda = $scope.model.queue.name;
                        $scope.model.queueEncuesta.codigoTienda = $scope.model.queue.internalCode;

                    } else if (r.value === 'false') {
                        $scope.model.queueEncuesta.esActiva = false;
                    }
                }
            });
        };

        /**
         * Funcion para recuperar el valor de la variable global del Tenant ENCUESTA_NO_PRESENTADOS
         * que configura si la Encuesta se puede enviar a un turno marcado como "No presentado" si el
         * cliente ha dejado su numero de movil
         */
        var getEncuestaNoPresentadosData = function () {
            APIVariable.byNameAndTenant(sessionService.getTenantId(), 'ENCUESTA_NO_PRESENTADOS').then(function (r) {
                if (r !== undefined) {
                    $scope.model.encuestaNoPresentados = r.plain();
                    console.log($scope.model.encuestaNoPresentados);
                }
            });
        };

        var init = function () {

            getClock();

            APIQueue.loadOne($scope.model.user.currentQueue.id).then(function (q) {

                // obtenemos datos de la queue (oficina) que ha seleccionado el usuario
                $scope.model.queue = q;

                APIOperationsProfile.loadOne(q.operationsProfile.id).then(function (response) {

                    q.operationsProfile = response;

                    //Mostramos el nombre del puesto
                    _.each($scope.model.queue.stations, function (s) {
                        if (s.id == $scope.model.user.currentStation.id) {
                            $scope.model.puesto = s;
                        }
                    });

                    // ORANGE
                    // obtenemos informacion si la tienda tiene encuesta Madison habilitada
                    getEncuestaData();

                    // obtenemos informacion sobre la politica de privacidad
                    getPoliticaPrivacidadData();

                    // obtenemos informacion sobre si se debe enviar encuesta a los no presentados
                    getEncuestaNoPresentadosData();

                    // configuramos la lista de Work Profiles definidos para el usuario
                    loadWorkProfiles($scope.model.queue);

                    // si tenemos ticket en servicio lo mostramos
                    getServingTicket();

                    // obtenemos informacion de como debemos mostrar los datos del cliente - editables o no
                    getEditClientData();

                    if ($scope.model.zone === null) {
                        // obtenemos un valor para saber si la tienda tiene conteo o no
                        APICapacity.findZone($scope.model.queue.internalCode).then(function (resp) {
                            if (resp && resp.internal_code && resp.max_capacity) {
                                $scope.queueHasCounter = true;
                                $scope.model.zone = resp;
                            }
                        });
                    }

                    // cargamos tickets en espera
                    $scope.loadWaitLine();

                    $scope.initInterval();
                });
            });
        };

        //Calcula la hora a partir de la zona horaria
        $scope.parseTimeZone = function (hour){
            var offset = moment().utcOffset();
            var serverOffset = moment().tz("Europe/Madrid").utcOffset();
            var difference = parseInt(offset) - parseInt(serverOffset);
            hour = moment(hour).add(difference, 'm').toDate();
            return hour;
        };

        function getClock() {
            $scope.clock = Date.now(); // get the current time
            /**
             var offset = moment().utcOffset();
             var serverOffset = moment().tz("Europe/Madrid").utcOffset();
             var difference = parseInt(offset) - parseInt(serverOffset);
             $scope.clock = moment(date).subtract(difference, 'm').toDate();
             */
        }

        function refreshWindow() {
            getClock();
            getServingTicket();
            $scope.loadWaitLine();
        }

        /*
        function saveTicketCloseInfo(closeInfoArray) {
            APITicketCloseInfo.update(closeInfoArray);
        }
        */

        function saveTicketCloseInfo(closeInfoData) {

            if (closeInfoData.clientMarcaInfo !== null) {
                // guardamos los datos para el marca del cliente
                APITicketMarcaInfo.update(closeInfoData.clientMarcaInfo);
            }
        }

        function deferredFn() {
            var defer = $q.defer();
            var promises = [];

            $scope.model.redirectServices = [];

            // obtenemos los usuarios trabajando en la tienda
            APIUser.workingInQueue($scope.model.queue.id).then(function (users) {

                _.each(users, function (u) {

                    if (typeof u.currentStation !== 'undefined') {

                        promises.push(APIWorkProfile.loadOne(u.currentWorkProfile.id).then(function (workProfile) {
                            _.each(workProfile.servicePoints, function (sp) {
                                var found = _.find($scope.model.redirectServices, ['id', sp.id]);
                                if (!found) {
                                    $scope.model.redirectServices.push(sp);
                                }
                            });
                        }));
                    }
                });

                $scope.model.redirectUsers = [];

                promises.push(APIUser.workingInQueue($scope.model.queue.id).then(function (users) {

                    _.each(users, function (u) {
                        if (typeof u.currentStation !== 'undefined') {
                            $scope.model.redirectUsers.push(u);
                        }
                    });
                }));

                $q.all(promises).then(lastTask);
            });

            function lastTask() {
                APIServicePoint.byQueue($scope.model.queue.id).then(function (services) {
                    // obtenemos los servicios de la tienda que pueden ser atendidos por los usuarios logeados con sus perfiles actuales
                    $scope.model.servicios = services.filter(function (service) {
                        var serviceFound = _.find($scope.model.redirectServices, ['id', service.id]);
                        return serviceFound !== undefined;
                    });

                    defer.resolve();
                });
            }

            return defer.promise;
        }

        init();

    })
    .controller('CallTicketCtrl', function ($scope, $uibModalInstance, ticket) {

        $scope.ticket = ticket;

        $scope.ok = function () {
            $uibModalInstance.close(true);
        };
        $scope.notPresented = function () {
            $uibModalInstance.close(false);
        };

    })
    .controller('CloseAtentionModalCtrl', function ($scope, $uibModalInstance) {

        $scope.withoutManagement = function () {
            $uibModalInstance.close('WITHOUT_MANAGEMENT');
        };
        $scope.withManagement = function () {
            $uibModalInstance.close('WITH_MANAGEMENT');
        };

    })
    .controller('CloseAtentionWithManagementModalCtrl', function ($scope, $uibModalInstance, ticket, APIQueue) {

        $scope.attentionType = {};

        $scope.activaciones = {
            movilRs: 0,
            fijaRs: 0,
            movilPro: 0,
            fijaPro: 0,
            prepago: 0,
            terminales: 0,
            miFijo: 0,
            renoves: 0
        };

        $scope.procesos = {
            prospecto: 0,
            agendado: 0,
            multicanalidad: 0,
            entregaActivaciones: 0,
            reparaciones: 0,
            otrasPosventas: 0
        };

        $scope.comboVisible = {
            movilRs: true,
            fijaRs: true,
            movilPro: true,
            fijaPro: true,
            prepago: true,
            terminales: true,
            miFijo: true,
            renoves: true,
            prospecto: true,
            agendado: true,
            multicanalidad: true,
            entregaActivaciones: true,
            reparaciones: true,
            otrasPosventas: true
        };

        $scope.amounts = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, "+10"];

        $scope.closeInfoData = {
            closeInfoArray: [],
            clientMarcaInfo: null
        };

        $scope.closeInfoArray = [];

        $scope.tenantMarcas = [];

        $scope.clientInfo = {
            si: false,
            no: false,
            marcaSi: null,
            marcaNo: null
        };

        $scope.validateForm = function () {
            return isMarcaSelected();
            /*
            * Ocultacion de Actividade sal cierre del ticket
            * && ( isActivivacionsAmountGreaterThanZero() || isProcesosAmountGreaterThanZero() );
            */
        };

        $scope.selectAmount = function (amount, combo) {

            if (amount === "+10") {

                var defaul_initial_value = 11;

                switch (combo) {
                    case 'movilRs': {
                        $scope.comboVisible.movilRs = false;
                        $scope.activaciones.movilRs = defaul_initial_value;
                        break;
                    }
                    case 'fijaRs': {
                        $scope.comboVisible.fijaRs = false;
                        $scope.activaciones.fijaRs = defaul_initial_value;
                        break;
                    }
                    case 'movilPro': {
                        $scope.comboVisible.movilPro = false;
                        $scope.activaciones.movilPro = defaul_initial_value;
                        break;
                    }
                    case 'fijaPro': {
                        $scope.comboVisible.fijaPro = false;
                        $scope.activaciones.fijaPro = defaul_initial_value;
                        break;
                    }
                    case 'prepago': {
                        $scope.comboVisible.prepago = false;
                        $scope.activaciones.prepago = defaul_initial_value;
                        break;
                    }
                    case 'terminales': {
                        $scope.comboVisible.terminales = false;
                        $scope.activaciones.terminales = defaul_initial_value;
                        break;
                    }
                    case 'miFijo': {
                        $scope.comboVisible.miFijo = false;
                        $scope.activaciones.miFijo = defaul_initial_value;
                        break;
                    }
                    case 'renoves': {
                        $scope.comboVisible.renoves = false;
                        $scope.activaciones.renoves = defaul_initial_value;
                        break;
                    }
                    case 'prospecto': {
                        $scope.comboVisible.prospecto = false;
                        $scope.procesos.prospecto = defaul_initial_value;
                        break;
                    }
                    case 'agendado': {
                        $scope.comboVisible.agendado = false;
                        $scope.procesos.agendado = defaul_initial_value;
                        break;
                    }
                    case 'multicanalidad': {
                        $scope.comboVisible.multicanalidad = false;
                        $scope.procesos.multicanalidad = defaul_initial_value;
                        break;
                    }
                    case 'entregaActivaciones': {
                        $scope.comboVisible.entregaActivaciones = false;
                        $scope.procesos.entregaActivaciones = defaul_initial_value;
                        break;
                    }
                    case 'reparaciones': {
                        $scope.comboVisible.reparaciones = false;
                        $scope.procesos.reparaciones = defaul_initial_value;
                        break;
                    }
                    case 'otrasPosventas': {
                        $scope.comboVisible.otrasPosventas = false;
                        $scope.procesos.otrasPosventas = defaul_initial_value;
                        break;
                    }
                }
            }
        };

        $scope.save = function () {
            if ($scope.activaciones.movilRs > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Móvil RS', $scope.activaciones.movilRs, 'ACTIVATION'));
            }
            if ($scope.activaciones.fijaRs > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Fija RS', $scope.activaciones.fijaRs, 'ACTIVATION'));
            }
            if ($scope.activaciones.movilPro > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Móvil Pro', $scope.activaciones.movilPro, 'ACTIVATION'));
            }
            if ($scope.activaciones.fijaPro > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Fija Pro', $scope.activaciones.fijaPro, 'ACTIVATION'));
            }
            if ($scope.activaciones.prepago > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Prepago', $scope.activaciones.prepago, 'ACTIVATION'));
            }
            if ($scope.activaciones.terminales > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Terminales', $scope.activaciones.terminales, 'ACTIVATION'));
            }
            if ($scope.activaciones.miFijo > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Mi fijo', $scope.activaciones.miFijo, 'ACTIVATION'));
            }
            if ($scope.activaciones.renoves > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Renoves', $scope.activaciones.renoves, 'ACTIVATION'));
            }
            if ($scope.procesos.prospecto > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Prospecto Eureka (Info Com)', $scope.procesos.prospecto, 'PROCESS'));
            }
            if ($scope.procesos.agendado > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Agendado', $scope.procesos.agendado, 'PROCESS'));
            }
            if ($scope.procesos.multicanalidad > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Multicanalidad', $scope.procesos.multicanalidad, 'PROCESS'));
            }
            if ($scope.procesos.entregaActivaciones > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Entrega activaciones', $scope.procesos.entregaActivaciones, 'PROCESS'));
            }
            if ($scope.procesos.reparaciones > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Reparaciones', $scope.procesos.reparaciones, 'PROCESS'));
            }
            if ($scope.procesos.otrasPosventas > 0) {
                $scope.closeInfoArray.push(buildTicketCloseInfo(ticket, 'Otras Posventas (Atac)', $scope.procesos.otrasPosventas, 'PROCESS'));
            }

            $scope.closeInfoData.closeInfoArray = $scope.closeInfoArray;

            //$uibModalInstance.close($scope.closeInfoArray);
            $uibModalInstance.close($scope.closeInfoData);
        };

        $scope.selectClient = function (isClient) {
            if (isClient) {
                $scope.clientInfo.si = true;
                $scope.clientInfo.no = false;
                $scope.clientInfo.marcaNo = null;
            } else {
                $scope.clientInfo.no = true;
                $scope.clientInfo.si = false;
                $scope.clientInfo.marcaSi = null;
            }
        };

        $scope.selectClientMarca = function (marca) {

            //$scope.clientInfo.marca = marca;

            if ($scope.clientInfo.si === false && $scope.clientInfo.no === false && $scope.clientInfo.marcaSi === null && $scope.clientInfo.marcaNo === null) {
                $scope.closeInfoData.clientMarcaInfo = null;
            } else {
                var clientMarca = {
                    ticket: null,
                    isClient: false,
                    tenantMarca: null
                };

                clientMarca.ticket = ticket;

                if ($scope.clientInfo.si === true) {
                    clientMarca.isClient = true;
                    $scope.clientInfo.marcaSi = marca;
                    clientMarca.tenantMarca = $scope.clientInfo.marcaSi;
                }
                if ($scope.clientInfo.no === true) {
                    clientMarca.isClient = false;
                    $scope.clientInfo.marcaNo = marca;
                    clientMarca.tenantMarca = $scope.clientInfo.marcaNo;
                }

                //clientMarca.tenantMarca = $scope.clientInfo.marca;

                $scope.closeInfoData.clientMarcaInfo = clientMarca;
            }

        };

        var buildTicketCloseInfo = function (ticket, description, amount, type) {

            var ticketCloseInfo = {
                ticket: ticket,
                description: description,
                amount: amount,
                type: type
            };

            return ticketCloseInfo;
        };

        var isActivivacionsAmountGreaterThanZero = function () {
            return $scope.activaciones.movilRs > 0 || $scope.activaciones.fijaRs > 0 || $scope.activaciones.movilPro > 0 || $scope.activaciones.fijaPro > 0 || $scope.activaciones.prepago > 0 || $scope.activaciones.terminales > 0 || $scope.activaciones.miFijo > 0 || $scope.activaciones.renoves > 0;
        };

        var isProcesosAmountGreaterThanZero = function () {
            return $scope.procesos.prospecto > 0 || $scope.procesos.agendado > 0 || $scope.procesos.multicanalidad > 0 ||
                $scope.procesos.entregaActivaciones > 0 || $scope.procesos.reparaciones > 0 || $scope.procesos.otrasPosventas > 0;
        };

        var isMarcaSelected = function () {
            if ($scope.clientInfo.si === false && $scope.clientInfo.no === false) {
                return false;
            } else {
                if ($scope.clientInfo.si === true && $scope.clientInfo.marcaSi === null) {
                    return false;
                } else if ($scope.clientInfo.no === true && $scope.clientInfo.marcaNo === null) {
                    return false;
                } else {
                    return true;
                }
            }
        };

        var init = function () {

            APIQueue.loadOne(ticket.queue.id).then(function (response) {
                $scope.tenantMarcas = response.tenant.tenantMarca;
            });
        };

        init();

    })
    .controller('ReenvioServiceUserCtrl', function ($scope, $uibModalInstance, $stateParams, $log, $q, APIServicePoint, APIUser,
                                                    APITicket, APIVariable, APIWorkProfile, sessionService, ticket, ticketServiced,
                                                    services, users) {

        $scope.reenvioServicio = false;
        $scope.reenvioUsuario = false;

        $scope.model = {
            t: ticket,
            ticket: ticketServiced,
            servicios: services,
            usuarios: users
        };

        $scope.form = {
            servicio: {},
            usuario: {}
        };

        $scope.noSelection = function () {
            if (typeof $scope.form.servicio.id == 'undefined' && typeof $scope.form.usuario.id == 'undefined') {
                return true;
            } else {
                return false;
            }
        };

        $scope.verServicios = function () {

            $scope.reenvioServicio = true;
            $scope.reenvioUsuario = false;
            $scope.form.usuario = {};
        };

        $scope.verUsuarios = function () {
            $scope.reenvioUsuario = true;
            $scope.reenvioServicio = false;
            $scope.form.servicio = {};
        };

        $scope.cancel = function () {
            $uibModalInstance.close(null);
        };

        $scope.reenviaTicket = function (waitStatus) {
            // cuando se reenvia un ticket siempre se mantiene el tiempo de espera
            // en el caso de reenvio al servicio se llamara segun el tiempo de espera
            // en el caso de reenvio a un usuario, se mantiene el mismo servicio del ticket y sera atendido antes que los tickets que no esten asignado a el

            $scope.isDataLoading = true;
            $scope.buttonReenviarTicket = true;

            var t = $scope.model.t;
            t.servicePoint = (typeof $scope.form.servicio.id != 'undefined') ? $scope.form.servicio : $scope.model.t.servicePoint;
            t.assignedTo = (typeof $scope.form.usuario.id != 'undefined') ? $scope.form.usuario : null;
            t.status = waitStatus;

            APITicket.redirect(t).then(function (result) {
                $scope.isDataLoading = false;
                $scope.buttonReenviarTicket = false;
                if ($scope.model.ticket === undefined || $scope.model.ticket.id === t.id) {
                    sessionService.isServing = false;
                }
                $uibModalInstance.close(result);
            });
        };

    });
