'use strict';

angular.module('bewin_orange')
    .controller('SupervisionDashboardController', function ($filter, $log, $scope, $stateParams, $timeout, $interval, utilService,
                                                            APIQueue, APIServicePoint, APIUser, APITicket, APIStats, APICapacity,
                                                            NavigationServices) {

        $log.log("SupervisionDashboardController started ..");

        $scope.model = {
            queue: {},
            servicePoint: {},
            waitingTickets: [],
            waitingTicketsServicePoint: [],
            maxWaitByServicePoint: [],
            servicedBy: [],
            servicePoints: [],
            queueStats: {
                servedHoyCount: 0,          // numero de tickets servidos hoy
                servedHoy: [],              // array con los datos de los tickets servidor hoy
                enEspera: 0,                // numero de tickets en espera
                puestosAbiertos: 0,         // numero de puestos abiertos
                atendiendo: 0,              // numero de puestos que estan atendiendo un ticket
                maxTEspera: 0               // tiempo maximo de espera de todos los tickets en espera
            },
            servicesStats: [],
            zone: null,
            capacity: 0
        };

        $scope.form = {
            stations: []
        };

        $scope.tab = 1;

        $scope.servicePointLoadSort = 'name';

        var refreshInterval;

        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.selectServicePoint = function (sp) {
            $scope.tab = 3;
            $scope.model.servicePoint = sp;
            loadTicketByServicePoint($scope.model.queue.id, $scope.model.servicePoint.id);
        };


        $scope.toDate = function (clock, t) {
            var diff;

            if (typeof t != 'undefined' && typeof clock != 'undefined' && t != null && t != '') {
                var date = new Date(t);
                diff = clock.valueOf() - date.valueOf();
                diff = $scope.toUTCDate(new Date(diff));
                return diff;
            }

            return diff;
        };

        $scope.toDateM = function (t) {

            var diff;
            if (typeof t != 'undefined' && t != null && t != '') {
                diff = $scope.toUTCDate(new Date(t));
                return diff;
            }
            return diff;
        };


        $scope.showPuestosData = function () {

            $scope.tab = 1;

            $scope.model.servicePoint = {};
            $scope.model.waitingTicketsServicePoint = [];
        };

        $scope.showServicesData = function () {

            $scope.tab = 2;

            $scope.model.servicePoint = {};
            $scope.model.waitingTicketsServicePoint = [];
        };

        $scope.showTicketsData = function () {
            $scope.tab = 3;

            $scope.model.servicePoint = {};
            $scope.model.waitingTicketsServicePoint = [];
        };


        $scope.toUTCDate = function (date) {
            return new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(),
                date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds());
        };

        $scope.initInterval = function () {

            $scope.cancelInterval();

            refreshWindow();

            refreshInterval = $interval(refreshWindow, 5000);
        };

        $scope.cancelInterval = function () {
            $interval.cancel(refreshInterval);
        };

        $scope.$on('$destroy', function (e) {
            $scope.cancelInterval();
        });

        var loadAll = function (id) {

            APIServicePoint.byQueue(id).then(function (sps) {
                $scope.model.servicePoints = sps;

                if ($scope.model.servicesStats.length == 0) {
                    _.each($scope.model.servicePoints, function (sp) {
                        var service = {
                            id: 0,
                            name: "",
                            ticketsEnEspera: [],
                            numTicketsEnEspera: 0,
                            tMaxEspera: '00:00:00'
                        };
                        service.id = sp.id;
                        service.name = sp.name;

                        $scope.model.servicesStats.push(service);
                    });
                }

                APITicket.waitingInQueue(id).then(function (ts) {

                    $scope.model.waitingTickets = ts;

                    $scope.model.queueStats.enEspera = ts.length;

                    if ($scope.model.servicesStats.length > 0 && $scope.model.queueStats.enEspera > 0) {

                        _.each($scope.model.servicesStats, function (sp) {
                            var ticketsPorSp = $filter('filter')(ts, {servicePoint: {id: sp.id}}, true);
                            sp.ticketsEnEspera = ticketsPorSp;
                            sp.numTicketsEnEspera = ticketsPorSp.length;
                            sp.tMaxEspera = getServiceMaxTEspera(ticketsPorSp);
                        });

                        $scope.model.queueStats.maxTEspera = Math.max.apply(Math, $scope.model.servicesStats.map(function (o) {
                            return o.tMaxEspera;
                        }));
                    } else {
                        // este codigo es la fecha del 1 de enero de 1970 a las 00:00:00 porque como en el frontal para
                        // poner tiempos de espera hay que usar fechas, si pones la fecha 0 te devuelve la 1 de la mañana,
                        // y por tanto cuando no hay ttickets en espera, el max tiempo de espera no es 0 si no 1, asi que
                        // dejo esto para que la fecha sea el 01-01-1970 00:00:00
                        $scope.model.queueStats.maxTEspera = new Date(0 - 3600000);
                    }

                });

            });


            APIUser.workingInQueue(id).then(function (users) {
                $scope.model.users = users;

                var inOpen = 0;
                for (var i = 0; i < $scope.model.users.length; i++) {
                    //console.log(users[i].currentQueue.name + " - " + users[i].name + ": " + users[i].currentStation);
                    if (typeof $scope.model.users[i].currentStation != 'undefined') {
                        inOpen++;
                    }
                }
                $scope.model.queueStats.puestosAbiertos = inOpen;

                // obtenemos informacion para las estaciones y los usuario logeados
                _.each($scope.form.stations, function (s) {

                    // comprobamos si todavia esta activo
                    var k = _.find($scope.model.users, {currentStation: {id: s.id}});

                    if (k) {
                        s.user = k;

                        APITicket.inServiceByUser(s.user.id).then(function (ts) {
                            s.ticket = ts;
                        });
                    } else {
                        s.user = {};
                        s.ticket = {};
                    }

                });

                if ($scope.queueHasCounter) {
                    APICapacity.findCapacity($scope.model.queue.internalCode).then(function (capacity) {
                        if (capacity !== undefined) {
                            $scope.model.capacity = Math.round(capacity * 100 / $scope.model.zone.max_capacity);
                        } else {
                            $scope.model.capacity = '-';
                        }
                    });
                }

            });

        };


        var loadStats = function (id) {

            APIStats.byQueue(id).then(function (res) {
                $scope.model.queueStats.servedHoyCount = res.served;
                $scope.model.queueStats.atendiendo = res.inService;
            });

            APIStats.queueByServicePoint(id).then(function (data) {
                $scope.model.maxWaitByServicePoint = data;
            });

            APIStats.servicedBy(id).then(function (data) {
                $scope.model.queueStats.servedHoy = data;
            });

        };


        var getServiceMaxTEspera = function (ticketsPorSp) {

            var result = 0;

            if (ticketsPorSp.length > 0) {
                var sortedTickets = $filter('orderBy')(ticketsPorSp, 'issueTime');

                var time = new Date(sortedTickets[0].issueTime);
                result = $scope.clock.valueOf() - time.valueOf();
            }

            return $scope.toUTCDate(new Date(result));

        };


        var loadTicketByServicePoint = function (idq, idsp) {

            APITicket.waitingInQueueByServicePoint(idq, idsp).then(function (ts) {
                $scope.model.waitingTicketsServicePoint = ts;
            });

        };

        var init = function () {

            getClock();

            if (typeof $stateParams != 'undefined' && $stateParams.id != null && $stateParams.id != '') {

                // obtenemos los datos para la Tienda seleccionada
                APIQueue.loadOne($stateParams.id).then(function (data) {

                    $scope.model.queue = data;

                    $scope.form.stations = $scope.model.queue.stations;

                    if ($scope.model.zone === null) {
                        // obtenemos un valor para saber si la tienda tiene conteo o no
                        APICapacity.findZone($scope.model.queue.internalCode).then(function (resp) {
                            if (resp && resp.internal_code && resp.max_capacity) {
                                $scope.queueHasCounter = true;
                                $scope.model.zone = resp;
                            }
                        });
                    }

                    loadAll($scope.model.queue.id);

                    loadStats($scope.model.queue.id);

                    $scope.initInterval();
                });

            }

            $scope.tab = 1;
        };

        function getClock() {
            var date = Date.now(); // get the current time
            console.log(date);
            var offset = moment().utcOffset();
            var serverOffset = moment().tz("Europe/Madrid").utcOffset();
            var difference = parseInt(offset) - parseInt(serverOffset);
            $scope.clock = moment(date).subtract(difference, 'm').toDate();
        }

        function refreshWindow() {
            getClock();

            loadAll($scope.model.queue.id);
            loadStats($scope.model.queue.id);

            if (typeof $scope.model.servicePoint.id != 'undefined') {
                loadTicketByServicePoint($scope.model.queue.id, $scope.model.servicePoint.id);
            }
        }

        init();

    }); 
