'use strict';

angular.module('bewin_orange')
    .controller('SupervisionSalesController', function ($log, $scope, $stateParams, APIQueue, APITicketCloseInfo,
                                                        NavigationServices) {
        $log.log("SupervisionSalesController started ...");

        $scope.itemsByPage = 5;

        $scope.model = {
            queue: {}
        };

        $scope.activacion = {
            ticketId: {},
            commercial: {},
            movilRs: 0,
            fijaRs: 0,
            movilPro: 0,
            fijaPro: 0,
            prepago: 0,
            terminales: 0,
            miFijo: 0,
            renoves: 0
        };

        $scope.proceso = {
            ticketId: {},
            commercial: {},
            renoves: 0,
            prospecto: 0,
            agendado: 0,
            multicanalidad: 0,
            entregaActivaciones: 0,
            reparaciones: 0,
            otrasPosventas: 0
        }

        $scope.activaciones = [];
        $scope.procesos = [];

        $scope.tableAct = [];
        $scope.tablePro = [];

        $scope.totalAct = 0;
        $scope.totalPro = 0;

        $scope.tab = 1;

        $scope.volverAtras = function (link) {
            NavigationServices.unselectCurrentQueue(link);
        };

        $scope.showToday = function () {
            resetSalesTables();
            APITicketCloseInfo.getTodayHistory($scope.model.queue.id).then(function (sales) {
                populateSalesToTables(sales);
            });
            $scope.tab = 1;
        };
        $scope.showYesterday = function () {
            resetSalesTables();
            APITicketCloseInfo.getYesterdayHistory($scope.model.queue.id).then(function (sales) {
                populateSalesToTables(sales);
            });
            $scope.tab = 2;
        };
        $scope.showLastWeek = function () {
            resetSalesTables();
            APITicketCloseInfo.getLastWeekHistory($scope.model.queue.id).then(function (sales) {
                populateSalesToTables(sales);
            });
            $scope.tab = 3;
        };
        $scope.showLastMonth = function () {
            resetSalesTables();
            APITicketCloseInfo.getLastMonthHistory($scope.model.queue.id).then(function (sales) {
                populateSalesToTables(sales);
            });
            $scope.tab = 4;
        };
        $scope.showLastYear = function () {
            resetSalesTables();
            APITicketCloseInfo.getLastYearHistory($scope.model.queue.id).then(function (sales) {
                populateSalesToTables(sales);
            });
            $scope.tab = 5;
        };

        var populateSalesToTables = function (sales) {

            $scope.tableAct = [];
            $scope.tablePro = [];

            $scope.movilRsSum = 0;
            $scope.fijaRsSum = 0;
            $scope.movilProSum = 0;
            $scope.fijaProSum = 0;
            $scope.prepagoSum = 0;
            $scope.terminalesSum = 0;
            $scope.miFijoSum = 0;
            $scope.renovesSum = 0;

            $scope.prospectoSum = 0;
            $scope.agendadoSum = 0;
            $scope.multicanalidadSum = 0;
            $scope.entregaActivacionesSum = 0;
            $scope.reparacionesSum = 0;
            $scope.otrasPosventasSum = 0;

            if (sales.length <= 0) {
                return;
            }

            angular.forEach(sales, function (sale, key) {
                readSale(sale);
            });

            $scope.groupedActivations = _.groupBy($scope.activaciones, 'commercial');
            $scope.groupedProcess = _.groupBy($scope.procesos, 'commercial');

            angular.forEach($scope.groupedActivations, function (activations, key) {

                var item = {
                    comercial: {},
                    movilRs: 0,
                    fijaRs: 0,
                    movilPro: 0,
                    fijaPro: 0,
                    prepago: 0,
                    terminales: 0,
                    miFijo: 0,
                    renoves: 0
                };


                item.comercial = key;
                angular.forEach(activations, function (activation, key) {
                    item.movilRs += activation.movilRs ? activation.movilRs : 0;
                    item.fijaRs += activation.fijaRs ? activation.fijaRs : 0;
                    item.movilPro += activation.movilPro ? activation.movilPro : 0;
                    item.fijaPro += activation.fijaPro ? activation.fijaPro : 0;
                    item.prepago += activation.prepago ? activation.prepago : 0;
                    item.terminales += activation.terminales ? activation.terminales : 0;
                    item.miFijo += activation.miFijo ? activation.miFijo : 0;
                    item.renoves += activation.renoves ? activation.renoves : 0;

                    if ($scope.tableAct.indexOf(item) === -1) {

                        $scope.tableAct.push(item);
                    }
                });

                $scope.totalAct += item.movilRs + item.fijaRs + item.movilPro + item.fijaPro + item.prepago + item.terminales + item.miFijo + item.renoves;
                $scope.movilRsSum += item.movilRs;
                $scope.fijaRsSum += item.fijaRs;
                $scope.movilProSum += item.movilPro;
                $scope.fijaProSum += item.fijaPro;
                $scope.prepagoSum += item.prepago;
                $scope.terminalesSum += item.terminales;
                $scope.miFijoSum += item.miFijo;
                $scope.renovesSum += item.renoves;

            });

            angular.forEach($scope.groupedProcess, function (processes, key) {

                var item = {
                    commercial: {},
                    prospecto: 0,
                    agendado: 0,
                    multicanalidad: 0,
                    entregaActivaciones: 0,
                    reparaciones: 0,
                    otrasPosventas: 0
                };


                item.comercial = key;
                angular.forEach(processes, function (process, key) {
                    item.prospecto += process.prospecto ? process.prospecto : 0;
                    item.agendado += process.agendado ? process.agendado : 0;
                    item.multicanalidad += process.multicanalidad ? process.multicanalidad : 0;
                    item.entregaActivaciones += process.entregaActivaciones ? process.entregaActivaciones : 0;
                    item.reparaciones += process.reparaciones ? process.reparaciones : 0;
                    item.otrasPosventas += process.otrasPosventas ? process.otrasPosventas : 0;


                    if ($scope.tablePro.indexOf(item) === -1) {
                        $scope.tablePro.push(item);
                    }
                });

                $scope.totalPro += item.prospecto + item.agendado + item.multicanalidad + item.entregaActivaciones + item.reparaciones + item.otrasPosventas;

                $scope.prospectoSum += item.prospecto;
                $scope.agendadoSum += item.agendado;
                $scope.multicanalidadSum += item.multicanalidad;
                $scope.entregaActivacionesSum += item.entregaActivaciones;
                $scope.reparacionesSum += item.reparaciones;
                $scope.otrasPosventasSum += item.otrasPosventas;

            });

        }


        var resetSalesTables = function () {
            $scope.activaciones = [];
            $scope.procesos = [];
            $scope.totalPro = 0;
            $scope.totalAct = 0;
        }

        var readSale = function (sale) {

            $scope.activacion = {};
            $scope.proceso = {};

            if (sale === null) {
                return;
            }

            if (sale.description === "Móvil RS") {
                $scope.activacion.movilRs = sale.amount;

            }
            else if (sale.description === "Fija RS") {
                $scope.activacion.fijaRs = sale.amount;


            }
            else if (sale.description === "Móvil Pro") {
                $scope.activacion.movilPro = sale.amount;

            }
            else if (sale.description === "Fija Pro") {
                $scope.activacion.fijaPro = sale.amount;

            }
            else if (sale.description === "Prepago") {
                $scope.activacion.prepago = sale.amount;

            }
            else if (sale.description === "Terminales") {
                $scope.activacion.terminales = sale.amount;

            }
            else if (sale.description === "Mi fijo") {
                $scope.activacion.miFijo = sale.amount;

            }

            else if (sale.description === "Renoves") {
                $scope.activacion.renoves = sale.amount;

            }

            else if (sale.description === "Prospecto Eureka (Info Com)") {
                $scope.proceso.prospecto = sale.amount;

            }

            else if (sale.description === "Agendado") {
                $scope.proceso.agendado = sale.amount;

            }

            else if (sale.description === "Multicanalidad") {
                $scope.proceso.multicanalidad = sale.amount;

            }

            else if (sale.description === "Entrega activaciones") {
                $scope.proceso.entregaActivaciones = sale.amount;

            }

            else if (sale.description === "Reparaciones") {
                $scope.proceso.reparaciones = sale.amount;

            }

            else if (sale.description === "Otras Posventas (Atac)") {
                $scope.proceso.otrasPosventas = sale.amount;

            }

            if (sale.type === 'ACTIVATION') {
                $scope.activacion.ticketId = sale.ticketId;
                $scope.activacion.commercial = sale.commercial;
                $scope.activaciones.push($scope.activacion);
            } else {
                $scope.proceso.ticketId = sale.ticketId;
                $scope.proceso.commercial = sale.commercial;
                $scope.procesos.push($scope.proceso);
            }

        };

        var init = function () {

            if (typeof $stateParams !== 'undefined' && $stateParams.id !== null && $stateParams.id !== '') {
                // obtenemos los datos para la Tienda seleccionada
                APIQueue.loadOne($stateParams.id).then(function (queue) {
                    $scope.model.queue = queue;
                    if ($scope.model.queue !== null) {
                        APITicketCloseInfo.getTodayHistory($scope.model.queue.id).then(function (sales) {
                            populateSalesToTables(sales);
                        });
                    }
                });
            }
        };

        init();

    });