'use strict';

angular.module('bewin_orange')
    .controller('OperationsProfileController', function ($scope, $log, $stateParams, utilService, sessionService, modalService,
                                                         APIOperationsProfile, APIServicePoint, APIGeneric, toastr) {

        $scope.form = {
            filter: '',
            profile: null,
            workProfile: {},
            servicePriorityExpressions: [],
            servicePriorityExpression: {},
            tab: 1
        };

        // tenants have operational profiles. These profiles have priorizations inside.
        // priorizations are made of services and servicePriorityExpressions.

        $scope.model = {
            services: APIServicePoint.byTenant(sessionService.getTenantId()).$object,
            profiles: APIOperationsProfile.byTenant(sessionService.getTenantId()).$object,
            workProfiles: [],
            workProfile: {},
            priorityPolicies: APIGeneric.priorityPolicies().$object
        };

        $scope.selectProfile = function (u) {
            $scope.form.profile = JSON.parse(JSON.stringify(u));

            if (u.id) {
                APIOperationsProfile.workProfile(u.id).then(function (response) {
                    $scope.model.workProfiles = response;
                });
            }

            $scope.cancelWorkProfile();
            $scope.form.tab = 1;
        };

        $scope.selectWorkProfile = function (p) {
            $scope.model.workProfile = p;
            $scope.form.workProfile = JSON.parse(JSON.stringify(p));
            $scope.form.servicePriorityExpressions = p.servicePriorityExpressions;
            $scope.form.tab = 3;
        };

        $scope.removeWorkProfile = function (p) {
            $log.log("deleting>", p)
            APIOperationsProfile.removeWorkProfile($scope.form.profile.id, p.id).then(function () {
                toastr.info("Perfil de trabajo eliminado correctamente", "Operación realizada");
                $scope.model.workProfiles = APIOperationsProfile.workProfile($scope.form.profile.id).$object;
                $scope.model.workProfile = {};
                $scope.form.tab = 2;
            });
        };

        $scope.cancelWorkProfile = function () {

            $scope.form.servicePriorityExpression = {};
            $scope.selectedService = [];

            $scope.selectWorkProfile({
                serviceGroup: [],
                servicePriorityExpressions: []
            });
        };

        $scope.saveWorkProfile = function () {
            var workProfile = {
                id: $scope.form.workProfile.id,
                name: $scope.form.workProfile.name,
                hideWorkProfile: $scope.form.workProfile.hideWorkProfile,
                operationsProfile: {
                    id: $scope.form.profile.id
                },
                servicePriorityExpressions: $scope.form.servicePriorityExpressions
            }

            APIOperationsProfile.addWorkProfile($scope.form.profile.id, workProfile).then(function (res) {
                $scope.model.workProfiles = APIOperationsProfile.workProfile($scope.form.profile.id).$object;
                $scope.selectWorkProfile(res);
                toastr.info("Perfil de trabajo actualizado correctamente", "Operación realizada");
                $scope.selectedService = [];
            });
        }

        var loadAll = function () {
            $scope.model.profiles = APIOperationsProfile.byTenant(sessionService.getTenantId()).$object;
            $scope.cancel();
        }

        $scope.cancel = function () {
            $scope.selectProfile({});
        }

        $scope.remove = function () {

            var msg = 'Se va a deshabilitar el perfil de operaciones. ' +
                'Si realiza esta acción, deberá cambiar el perfil en TODAS las tiendas asociadas al mismo. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                APIOperationsProfile.deleteOne($scope.form.profile.id).then(function (r) {
                    toastr.info("Perfil de operaciones eliminado correctamente", "Operación realizada");
                    $scope.cancel();
                    loadAll();
                });
                $scope.cancel();
            });
        }

        $scope.save = function () {

            if (!$scope.form.profile.tenant) {
                $scope.form.profile.tenant = {
                    id: sessionService.getTenantId()
                }
            }

            APIOperationsProfile.update($scope.form.profile).then(function (res) {
                $scope.selectProfile(res);
                toastr.info("Perfil de trabajo actualizado correctamente", "Operación realizada");
                loadAll();
            })
        }

        $scope.isIn = function (g, col) {
            return _.find(col, g);
        }

        $scope.removeFromGroup = function (g, col) {
            return _.remove(col, g);
        }

        $scope.addPriorityExpression = function (g, col) {
            if (!g.priorityPolicy && !g.serviceGroup) return;
            if (!g.priorityPolicy && !g.serviceGroup.length) return;

            $log.log("adding>", g, $scope.form.servicePriorityExpressions);

            var e = _.find($scope.form.servicePriorityExpressions, ['id', g.id]);
            if (!e) {
                e = JSON.parse(JSON.stringify(g));
                $scope.form.servicePriorityExpressions.push(e);
            }
            e.priorityPolicy = $scope.form.servicePriorityExpression.priorityPolicy;
            e.serviceGroup = $scope.form.servicePriorityExpression.serviceGroup;

            $scope.form.servicePriorityExpression = {};
            $scope.selectedService = [];
        };

        $scope.cancelEditPriorityExpression = function () {
            $scope.editPriorityExpression({id: null, serviceGroup: [], priorityPolicy: {id: null}});
        };

        $scope.editPriorityExpression = function (e) {

            $scope.form.servicePriorityExpression.priorityPolicy = _.find($scope.model.priorityPolicies, ['id', e.priorityPolicy.id]);
            $scope.form.servicePriorityExpression.serviceGroup = [];
            $scope.form.servicePriorityExpression.id = e.id;

            e.serviceGroup.map(function (f) {
                var service = _.find($scope.model.services, ['id', f.id]);
                if (service !== undefined) {
                    $scope.form.servicePriorityExpression.serviceGroup.push(service);
                }
            });

            $scope.selectedService = [];
            $scope.selectedService = $scope.form.servicePriorityExpression.serviceGroup;

        };

        $scope.isSelected = function (g) {
            if (typeof $scope.selectedService != 'undefined') {
                var k = _.find($scope.selectedService, ['id', g.id]);
                if (k) return true;
            }
            return false;
        }

        $scope.serviceGroupSelected = function (g) {

            console.log("serviceGroupSelected");

            if (typeof $scope.form.servicePriorityExpression.serviceGroup == 'undefined') {
                $scope.form.servicePriorityExpression.serviceGroup = [];
            }

            // si ya existe lo borramos, si no lo añadimos
            var k = _.find($scope.form.servicePriorityExpression.serviceGroup, ['id', g.id]);

            console.log(k);

            if (!k) {
                $scope.form.servicePriorityExpression.serviceGroup.push(g);
            } else {
                var idx = $scope.form.servicePriorityExpression.serviceGroup.indexOf(g);
                console.log(idx);

                $scope.form.servicePriorityExpression.serviceGroup.splice(idx, 1);
            }

        };

    });
 