'use strict';

angular.module('bewin_orange')
    .controller('PromotionsController', function ($q, $scope, $compile, configurationService, modalService, sessionService, toastr,
                                                  PromotionAPI, PromotionGroupingAPI, APIQueue, APITicketDesign, Upload) {

        $scope.model = {
            promotions: [],
            availableQueues: [],
            selectedQueues: [],
            initialPromotions: [],
            search: {
                promotionName: "",
                promotionType: "",
                interval: "",
                active: false
            },
            ticket: {},
            ticketDesign: ""
        };

        $scope.form = {
            promotion: null,
            promotionFile: null,
            promotionFileLoad: null,
            filter: [],
            secondFilter: []
        };

        $scope.select = function (u) {
            var deferred = $q.defer();

            if (u && u.id) {
                PromotionGroupingAPI.loadOne(u.id).then(function (promotion) {
                    $scope.form.promotion = promotion;
                    reloadSelectedQueueList();

                    if (typeof $scope.form.promotion.promotion != 'undefined' && typeof $scope.form.promotion.promotion.id != 'undefined') {
                        var promo = _.find($scope.model.promotions, ['id', $scope.form.promotion.promotion.id]);
                        if (promo != undefined) {
                            $scope.form.promotion.promotion = promo;
                        }
                    } else if (typeof $scope.form.promotion.html != 'undefined'){
                        $compile($("#promotion").html($scope.form.promotion.html).contents())($scope);
                    }

                    deferred.resolve(true);
                });
            } else {
                $scope.form.promotion = {};
                $scope.model.promotion = {};
                reloadSelectedQueueList();

                deferred.resolve(false);
            }
            $scope.tab = 1;

            return deferred.promise;
        };

        $scope.search = function () {
            var params = {
                name: ($scope.model.search.promotionName && $scope.model.search.promotionName !== '' ? $scope.model.search.promotionName : null),
                interval: ($scope.model.search.interval && $scope.model.search.interval !== '' ? $scope.model.search.interval : null),
                promotionType: ($scope.model.search.promotionType && $scope.model.search.promotionType !== '' ? $scope.model.search.promotionType : null),
                active: ($scope.model.search.active === true ? false : null)
            };
            PromotionGroupingAPI.search(params).then(function (response) {
                $scope.model.initialPromotions = response;
            });
        };

        $scope.cancel = function () {
            $scope.select(null);
        };

        $scope.reactivate = function (promotion) {
            $scope.select(promotion).then(function (loadOk) {
                if (loadOk === true) {
                    $scope.form.promotion.active = true;
                    $scope.save();
                }
            });
        };

        $scope.selectedPromo = function () {
            var video = document.getElementById("selectedVideo");
            if (video) {
                video.load();
            }
        };

        $scope.save = function () {
            $scope.form.promotion.promotionQueues = [];
            if (typeof $scope.model.selectedQueues != 'undefined' && $scope.model.selectedQueues.length !== 0) {
                var queueId = null;
                for (var i = 0; i < $scope.model.selectedQueues.length; i++) {
                    queueId = $scope.model.selectedQueues[i].id;
                    $scope.form.promotion.promotionQueues.push({"queueId": queueId});
                }
            }
            if ($scope.form.promotion.promotionType === 'DISPLAY' || $scope.form.promotion.promotionType === 'KIOSK') {
                $scope.form.promotion.html = null;
            }
            if ($scope.form.promotion.promotionType === 'TICKET') {
                $scope.form.promotion.promotion = null;
            }

            PromotionGroupingAPI.save($scope.form.promotion).then(function (r) {
                toastr.info("Promoción guardada correctamente", "Operación realizada");
                $scope.select(r);
                $scope.search();
            });

        };

        $scope.remove = function () {
            var msg = 'Se va a deshabilitar una promoción . ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                PromotionGroupingAPI.deleteOne($scope.form.promotion.id).then(function (r) {
                    toastr.info("Promoción eliminada correctamente", "Operación realizada");
                    $scope.cancel();
                    $scope.search();
                });
            });
        };

        $scope.nameQ = function (q) {
            return q.idCode + '-' + q.name;
        };

        $scope.moveItem = function (item, from, to) {
            var idx = from.indexOf(item);
            if (idx !== -1) {
                from.splice(idx, 1);
                to.push(item);
            }
        };

        $scope.moveItems = function (items, from, to) {
            if (items && items.length) {
                for (var i = 0; i < items.length; i++) {
                    var item = items [i];
                    $scope.moveItem(item, from, to);
                }
            }
        };

        $scope.moveAll = function (from, to) {
            angular.forEach(from, function (item) {
                to.push(item);
            });
            from.length = 0;
        };

        /* ########## PROMOTIONS ########## */
        $scope.listPromotion = function () {
            PromotionAPI.search().then(function (list) {
                $scope.model.promotions = list;
            });
        };

        $scope.selectPromotion = function (promo) {
            var deferred = $q.defer();

            $scope.form.promotionFile = null;
            $scope.form.promotionFileLoad = null;

            if (promo && promo.id) {
                PromotionAPI.loadOne(promo.id).then(function (promotion) {

                    $scope.form.promotionFile = promotion;
                    deferred.resolve(true);
                });
            } else {
                deferred.resolve(false);
            }

            return deferred.promise;
        };

        $scope.cancelPromotion = function () {
            $scope.selectPromotion(null);
        };

        $scope.savePromotion = function () {
            var deferred = $q.defer();
            var uploadUrl = configurationService.apiPromotionURL + "/promotion";

            Upload.upload({
                url: uploadUrl,
                headers: {
                    'Content-Type': 'multipart/form-data'
                },
                data: {"promotion": JSON.stringify({})},
                file: $scope.form.promotionFileLoad
            }).then(function (response) {
                if (response) {
                    if (response.data) {
                        $scope.selectPromotion(response.data);
                    } else {
                        $scope.selectPromotion(response);
                    }

                    $scope.listPromotion();
                    toastr.info("Fichero guardado correctamente", "¡Perfecto!");
                    deferred.resolve(true);
                }
            }, function (error) {
                var mensaje = "";
                if (typeof error.data.details.errorMessage != 'undefined') {
                    mensaje = error.data.details.errorMessage;
                } else {
                    mensaje = "Error: " + error.status + " (" + error.statusText + ")";
                }
                toastr.error(mensaje);
            });

            return deferred.promise;
        };

        $scope.removePromotion = function (promotionFile) {
            var msg = 'Se va a deshabilitar el fichero. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                PromotionAPI.deleteOne(promotionFile.id).then(function (r) {
                    toastr.info("Fichero eliminado correctamente", "Operación realizada");
                    $scope.listPromotion();
                    $scope.cancelPromotion();
                });
            });
        };

        $scope.completePromotion = function () {
            $compile($("#promotion").html($scope.form.promotion.html).contents())($scope);
        };

        $scope.cleanFilter = function () {
            $scope.form.filter = [];
        };

        $scope.cleanSecondFilter = function () {
            $scope.form.secondFilter = [];
        };

        var reloadQueueList = function () {
            APIQueue.byTenant(sessionService.getTenantId()).then(function (list) {
                $scope.model.queues = list;
                reloadSelectedQueueList();
            });
        };

        var reloadSelectedQueueList = function () {
            if (typeof $scope.form.promotion.promotionQueues != 'undefined' && $scope.form.promotion.promotionQueues.length !== 0) {
                var queue = null;
                var selectedQueue = null;
                $scope.model.selectedQueues = [];
                $scope.model.availableQueues = [];

                for (var i = 0; i < $scope.model.queues.length; i++) {
                    queue = $scope.model.queues[i];
                    if (queue) {
                        selectedQueue = _.find($scope.form.promotion.promotionQueues, ['queueId', queue.id]);
                        if (selectedQueue !== undefined) {
                            $scope.model.selectedQueues.push(queue);
                        } else {
                            $scope.model.availableQueues.push(queue);
                        }
                    }
                }
            } else {
                $scope.model.availableQueues = $scope.model.queues;
                $scope.model.selectedQueues = [];
            }
        };

        var init = function () {
            $scope.cancel();
            reloadQueueList();

            $scope.listPromotion();
            $scope.search();

            // generamos un ticket por defecto para la previsualizacion
            $scope.model.ticket.queue = new Object();
            $scope.model.ticket.queue.name = "TIENDA ORANGE GENERICA PARA DISEÑO";
            $scope.model.ticket.visitorName = "Cliente Prueba";
            $scope.model.ticket.name = "A001";
            $scope.model.ticket.servicePoint = new Object();
            $scope.model.ticket.servicePoint.name = "ATENCIÓN AL CLIENTE";
            $scope.model.ticket.issueTime = new Date();
            // buscamos el diseño del ticket
            APITicketDesign.byTenant(sessionService.getTenant().id).then(function (response) {
                $compile($("#previewTicket").html(response[0].htmlDesign).contents())($scope);
            });
        };


        init();
    });
