'use strict';

angular.module('bewin_orange')
    .controller('QueueAdminController', function ($scope, $log, $stateParams, $filter, $uibModal, utilService, toastr,
                                                  sessionService, paginationService, modalService, APIQueue, APIGeneric,
                                                  APIOperationsProfile, APITicketDesign, APIVariable, APIDevice,
                                                  APIApplication, APIWorkProfile, APIQueueLicenseType, TIMETABLE_TYPE,
                                                  TimetableGroupingAPI, FestiveGroupingAPI, TimetableServices,
                                                  FestiveServices) {

        $scope.model = {
            queues: [],
            queue: {},
            countries: APIGeneric.countries().$object,
            provinces: [],
            cities: [],
            operationsProfiles: APIOperationsProfile.byTenant(sessionService.getTenantId()).$object,
            queueLicenseType: APIQueueLicenseType.loadAll().$object,
            queueConfiguration: APIGeneric.queueConfParams().$object,
            ticketDesigns: APITicketDesign.byTenant(sessionService.getTenantId()).$object,
            genericTimetableGrouping: TimetableGroupingAPI.findGenerics(TIMETABLE_TYPE.OPEN_CLOSE).$object,
            genericTimetableAppoiment: TimetableGroupingAPI.findGenerics(TIMETABLE_TYPE.APPOINTMENT).$object,
            genericFestiveGrouping: FestiveGroupingAPI.findGenerics().$object,
            stations: [],
            variables: [],
            devices: [],
            device: {},
            festive: {},
            timetableGrouping: {},
            timetableGroupingAppoiment: {},
            showSaveTimetable: false,
            showSaveAppointments: false
        };

        $scope.initialQueues = [];

        $scope.stations = [];

        $scope.stationDuplicated = false;
        $scope.flgCreateStation = true;
        $scope.isQueueSelected = false;

        $scope.form = {
            filter: '',
            queue: {},
            variable: {},
            device: {},
            licenseType: $scope.model.queueLicenseType[0],
            name: '',
            code: '',
            active: false,

            timetableGrouping: {},
            timeTable: {},
            time: {},
            timetableGroupingAppoiment: {},
            timeTableAppoiment: {},
            timeAppoiment: {},
            festiveGrouping: {},
            listfestive: [],
            festive: {},
            image_base64: ''
        };

        $scope.pager = {};
        $scope.setPage = setPage;

        $scope.onCountrySelect = function () {
            if (!$scope.form.queue.country || !$scope.form.queue.country.id) return;

            APIGeneric.provinces($scope.form.queue.country.id).then(function (provinces) {
                $scope.model.provinces = provinces;
                if ($scope.form.queue.province)
                    $scope.form.queue.province = utilService.byID(provinces, $scope.form.queue.province.id);
            });
        };


        $scope.onProvinceSelect = function () {
            if (!$scope.form.queue.province || !$scope.form.queue.province.id) return;
            APIGeneric.cities($scope.form.queue.province.id).then(function (cities) {
                $scope.model.cities = cities;
                if ($scope.form.queue.city)
                    $scope.form.queue.city = utilService.byID(cities, $scope.form.queue.city.id);
            });
        };


        if (typeof google !== "undefined") {
            $scope.mapOptions = {
                center: new google.maps.LatLng(36.964645, -122.01523),
                zoom: 18,
                // Style for Google Maps
                MapTypeId: google.maps.MapTypeId.HYBRID,
                styles: [

                    {
                        featureType: "all",
                        elementType: "labels",
                        stylers: [{
                            visibility: "on"
                        }]
                    }
                ],
                mapTypeId: google.maps.MapTypeId.HYBRID
            };
        }

        $scope.myMarker = null;
        $scope.myMap = null;
        $scope.addMarker = function ($event, $params) {

            if ($scope.myMarker)
                $scope.myMarker.setMap(null);

            $scope.myMarker = new google.maps.Marker({
                map: $scope.myMap,
                position: $params[0].latLng
            });

            if ($event) {
                $scope.form.queue.lat = $params[0].latLng.lat();
                $scope.form.queue.lon = $params[0].latLng.lng();
                APIQueue.update($scope.form.queue).then(function (l) {
                    var cached = utilService.byID($scope.model.queues, l.id);
                    cached.lat = $params[0].latLng.lat();
                    cached.lon = $params[0].latLng.lng();
                })
            }
        };

        $scope.remove = function () {

            var msg = 'Se va a deshabilitar la tienda y se eliminará de las relaciones con usuarios, servicios, etc. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                APIQueue.deleteOne($scope.form.queue.id).then(function (r) {
                    toastr.info("Tienda eliminada correctamente", "Operación realizada");
                    loadAll();
                });
                $scope.cancel();
            });
        };

        $scope.selectVar = function (v) {
            $scope.form.variable = angular.copy(v);
        };

        $scope.cancelVar = function () {
            $scope.form.variable = {};
        };

        $scope.deleteVar = function () {
            if (!$scope.form.variable || !$scope.form.variable.id) return;
            APIVariable.deleteOne($scope.form.variable.id).then(function () {
                $scope.model.variables = APIVariable.byQueue($scope.model.queue.id).$object;
                toastr.info("Variable eliminada correctamente", "Operacion realizada");
                $scope.cancelVar();
            });

            loadAll($scope.model.queue.id);
        };

        $scope.saveVar = function () {
            delete $scope.form.variable.tenant;
            $scope.form.variable.queue = {
                id: $scope.form.queue.id
            };

            APIVariable.update($scope.form.variable).then(function (v) {
                $scope.model.variables = APIVariable.byQueue($scope.model.queue.id).$object;
                toastr.info("Variable actualizada correctamente", "Operacion realizada");
                $scope.cancelVar();
            });

            loadAll($scope.form.queue.id);
        };

        $scope.select = function (q) {
            $scope.model.showSaveTimetable = false;
            $scope.model.showSaveAppointments = false;

            if (q && q.id) {
                APIQueue.loadOne(q.id).then(function (resp) {

                    $scope.model.queue = resp;
                    $scope.form.queue = JSON.parse(JSON.stringify(resp));
                    $scope.stationDuplicated = false;
                    $scope.stations = $scope.form.queue.stations;

                    if (resp.operationsProfile !== undefined) {
                        $scope.form.queue.operationsProfile = _.find($scope.model.operationsProfiles, ['id', resp.operationsProfile.id]);
                    }
                    if (resp.queueLicenseType !== undefined) {
                        $scope.form.queue.queueLicenseType = _.find($scope.model.queueLicenseType, ['id', resp.queueLicenseType.id]);
                    }

                    if (q && resp.configuration) {
                        $scope.form.queue.configuration = [];
                        resp.configuration.map(function (f) {
                            var k = _.find($scope.model.queueConfiguration, ['id', f.id]);
                            if (k !== undefined) {
                                $scope.form.queue.configuration.push(k);
                            }
                        });

                        if ($scope.form.queue.configuration.length > 0) {
                            if (_.find($scope.form.queue.configuration, {id: 10001}) !== undefined) {
                                $scope.isQueueSMSEnabled = true;
                            } else {
                                $scope.isQueueSMSEnabled = false;
                            }
                        }
                    }

                    if ($scope.form.queue.country) {
                        $scope.form.queue.country = utilService.byID($scope.model.countries, $scope.form.queue.country.id);
                    }

                    $scope.onCountrySelect();
                    $scope.onProvinceSelect();

                    if (resp.id !== undefined) {
                        $scope.model.variables = APIVariable.byQueue(resp.id).$object;
                        $scope.model.devices = APIDevice.byQueue(resp.id).$object;
                        APIDevice.byQueueDeleted(resp.id).then(function (resp) {
                            $scope.model.deletedDevices = resp;
                        });
                        $scope.isQueueSelected = true;
                    } else {
                        $scope.model.variables = null;
                        $scope.model.devices = null;
                        $scope.model.deletedDevices = null;
                        $scope.isQueueSelected = false;
                    }

                    if (typeof resp.ticketDesign != 'undefined') {
                        $scope.form.queue.ticketDesign = _.find($scope.model.ticketDesigns, ['id', resp.ticketDesign.id]);
                    }

                    if (q) {
                        if ($scope.myMap && q.lat && q.lon) {
                            var ll = new google.maps.LatLng(q.lat, q.lon);
                            $scope.myMap.setCenter(ll);
                            $scope.addMarker(null, [{
                                latLng: ll
                            }]);
                        }
                    }

                    TimetableServices.selectTimetableGroupingAppoiment(q, $scope.form, $scope.model);
                    TimetableServices.selectTimetableGrouping(q, $scope.form, $scope.model);
                    FestiveServices.selectFestiveGrouping(q, $scope.form, $scope.model.genericFestiveGrouping);

                    $scope.form.festive = {};
                    $scope.form.variable = {};
                    $scope.form.device = {};

                    $scope.model.festive = {};

                    APIQueue.loadCode(q.id).then(function (resp) {
                        $scope.form.image_base64 = 'data:image/png;base64,' + resp;
                        $("#qr-code-image").attr('src', $scope.form.image_base64);
                    });
                });
            } else {
                $scope.isQueueSelected = false;
                $scope.stations = [];

                $scope.model.queue = {};
                $scope.model.variables = [];
                $scope.model.devices = [];
                $scope.model.deletedDevices = [];
                $scope.model.festive = {};
                $scope.model.timetableGrouping = {};
                $scope.model.timetableGroupingAppoiment = {};
                $scope.model.showSaveTimetable = false;
                $scope.model.showSaveAppointments = false;

                $scope.form.filter = '';
                $scope.form.queue = {};
                $scope.form.variable = {};
                $scope.form.device = {};
                $scope.form.timetableGrouping = {};
                $scope.form.timeTable = {};
                $scope.form.time = {};
                $scope.form.timetableGroupingAppoiment = {};
                $scope.form.timeTableAppoiment = {};
                $scope.form.timeAppoiment = {};
                $scope.form.festiveGrouping = {};
                $scope.form.festive = {};
                $scope.form.listfestive = [];
            }

            $("#timetableDetails").removeAttr('open');
            $("#appointmentDetails").removeAttr('open');
        };

        $scope.saveTimetable = function () {
            if ($scope.form.timeTable.name === '') {
                alert("El nombre para el horario es requerido");
            } else {
                TimetableServices.saveTimetable($scope.form, $scope.model);
            }
        };

        $scope.saveTimetableAppoiment = function () {
            if ($scope.form.timeTableAppoiment.intervalo === 0) {
                alert("El intervalo entre citas tiene que ser mayor de 0 minutos");
            } else if ($scope.form.timeTableAppoiment.name === '') {
                alert("El nombre para el horario es requerido");
            } else {
                TimetableServices.saveTimetableAppoiment($scope.form, $scope.model);
            }
        };

        $scope.selectTimeTable = function (select) {
            TimetableServices.selectTimeTable(select, $scope.form);
            $("#timetableDetails").attr('open', 'open');
        };

        $scope.selectTimeTableAppoiment = function (select) {
            TimetableServices.selectTimeTableAppoiment(select, $scope.form);
            $("#appointmentDetails").attr('open', 'open');
        };

        $scope.removeTimetable = function (timetableId) {
            TimetableServices.removeTimetable(timetableId, $scope.form, $scope.model);
        };

        $scope.removeTimetableAppoiment = function (timetableId) {
            TimetableServices.removeTimetableAppoiment(timetableId, $scope.form, $scope.model);
        };

        $scope.cancelTimetable = function () {
            $scope.form.timeTable = {};
            $scope.form.time = {};
            $("#timetableDetails").removeAttr('open');
        };

        $scope.cancelTimetableAppoiment = function () {
            $scope.form.timeTableAppoiment = {};
            $scope.form.timeAppoiment = {};
            $("#appointmentDetails").removeAttr('open');
        };

        $scope.selectGenericTimeTable = function () {
            TimetableServices.selectGenericTimeTable($scope.form, $scope.model);
        };

        $scope.selectGenericTimeTableAppoiment = function () {
            TimetableServices.selectGenericTimeTableAppoiment($scope.form, $scope.model);
        };

        $scope.selectEspecificTimeTable = function () {
            TimetableServices.selectEspecificTimeTable($scope.form, $scope.model);
        };

        $scope.selectEspecificTimeTableAppoiment = function () {
            TimetableServices.selectEspecificTimeTableAppoiment($scope.form, $scope.model);
        };

        $scope.cancel = function () {
            $scope.select({});
        };

        $scope.cancelFestive = function (q) {
            $scope.form.festive = {};
            $scope.model.festive = {};
        };

        $scope.removeFestive = function (festiveId) {
            FestiveServices.removeFestive(festiveId, $scope.form.queue, $scope.form, $scope.model.genericFestiveGrouping);
        };

        $scope.saveFestiveGrouping = function () {
            FestiveServices.saveFestiveGrouping($scope.form, $scope.model.genericFestiveGrouping);
        };

        $scope.saveFestive = function () {
            FestiveServices.saveFestive($scope.form, $scope.model, $scope.model.genericFestiveGrouping);
        };

        $scope.saveTimeTableGrouping = function () {
            TimetableServices.saveTimeTableGrouping($scope.form, $scope.model);
        };

        $scope.saveTimeTableGroupingAppoiment = function () {
            TimetableServices.saveTimeTableGroupingAppoiment($scope.form, $scope.model);
        };

        $scope.addStation = function (newStation) {
            if ($scope.model.queue.id != null) {
                $scope.flgCreateStation = true;

                $scope.newStation = newStation;


                var stationFound = $filter('filter')($scope.stations, {'name': $scope.newStation}, true)[0];
                if (!stationFound) {
                    $scope.stations.push({'name': $scope.newStation, 'queueId': $scope.model.queue.id, 'done': false});
                    $scope.stationDuplicated = false;
                } else {
                    $scope.stationDuplicated = true;
                    $scope.stationDuplicatedName = newStation;
                }
            }
            $scope.newStation = "";
            $('#newStattionId').val("");

        };

        $scope.removeStation = function (index) {
            $scope.stationDuplicated = false;
            $scope.stations.splice(index, 1);
        };

        $scope.editStation = function (station) {
            if ($scope.flgCreateStation === false && $scope.stationDuplicated === true) {
                return;
            }
            station.editing = true;
            $scope.stationDuplicated = false;
            $scope.flgCreateStation = false;
        };

        $scope.doneEditingStation = function (station, $index) {

            station.editing = false;
            $scope.stationDuplicated = false;

            for (var i = 0; i < $scope.stations.length; i++) {
                if (i !== $index && $scope.stations[i].name === station.name) {
                    $scope.stationDuplicated = true;
                    $scope.stationDuplicatedName = station.name;
                    station.editing = true;
                    break;
                }
            }

            if ($scope.stationDuplicated === false) {
                $scope.removeStation($index);
                $scope.stations.push({'name': station.name, 'queueId': $scope.model.queue.id, 'done': false});
            }

        };

        $scope.save = function () {
            $scope.form.queue.tenant = {
                id: sessionService.getTenantId()
            };

            $scope.form.queue.stations = $scope.stations;

            saveQueue();
        };

        $scope.search = function () {
            $scope.cancel();
            $scope.pager = {};
            var params = {
                name: ($scope.form.name === '' ? null : $scope.form.name),
                code: ($scope.form.code === '' ? null : $scope.form.code),
                tenant_id: sessionService.getTenantId(),
                active: ($scope.form.active === true ? false : null),
                queryLicenseType_id: (($scope.form.licenseType === undefined || $scope.form.licenseType === null) ? null : $scope.form.licenseType.id)
            };
            APIQueue.search(params).then(function (response) {
                $scope.initialQueues = response;
                $scope.setPage(1);
            });
        };

        var getQueueConfiguration = function () {
            // mapeamos las opciones disponibles para configuración teniendo en cuenta las opciones del Tenant
            // por ejemplo, es disponible la opción "Enviar ticket por SMS" solo si el tenant tiene configurado el envio por SMS

            APIGeneric.queueConfParams().then(function (resp) {

                var tQueueConfParams = resp.plain();

                if (!sessionService.selectedTenant.smsEnabled) {
                    $scope.model.queueConfiguration = _.filter(tQueueConfParams, function (el) {
                        return el.id != 10001;
                    });
                    $scope.isQueueSMSEnabled = false;
                } else {
                    $scope.model.queueConfiguration = tQueueConfParams;
                }
            });
        };

        $scope.reactivate = function (queue) {
            if (queue && queue.id) {
                APIQueue.loadOne(queue.id).then(function (queueBBDD) {
                    if (queueBBDD !== undefined) {
                        queueBBDD.active = true;
                        APIQueue.update(queueBBDD).then(function (res) {
                            $scope.select(res);
                            toastr.info("Tienda actualizada correctamente", "Operación realizada");
                            $scope.search();
                        });
                    }
                });
            }
        };

        var saveQueue = function () {
            if ($scope.form.queue.promotionURL === '') {
                $scope.form.queue.promotionURL = 'https://promocionesentutienda.orange.es/?utm_source=turno&utm_medium=sms&utm_campaign=pideturno/';
            }
            APIQueue.update($scope.form.queue).then(function (res) {
                toastr.info("Tienda actualizada correctamente", "Operación realizada");
                $scope.search();
                $scope.select(res);
            });
        };

        var loadAll = function (id) {
            var params = {
                tenant_id: sessionService.getTenantId()
            };
            APIQueue.search(params).then(function (response) {
                $scope.initialQueues = response;
                $scope.setPage(1);
                if ($scope.model.genericTimetableGrouping.length > 0) {
                    $scope.form.timetableGrouping.extend = _.find($scope.model.genericTimetableGrouping, ['predetermined', true]);
                }
                if ($scope.model.genericTimetableAppoiment.length > 0) {
                    $scope.form.timetableGroupingAppoiment.extend = _.find($scope.model.genericTimetableAppoiment, ['predetermined', true]);
                }
            });
        };

        function setPage(page) {
            if (page < 1 || page > $scope.pager.totalPages) {
                return;
            }

            // get pager object from service
            $scope.pager = paginationService.getPager($scope.initialQueues.length, page, 15);

            // get current page of items
            $scope.model.queues = $scope.initialQueues.slice($scope.pager.startIndex, $scope.pager.endIndex + 1);
        }

        loadAll($stateParams.id);

        getQueueConfiguration();

        /*********************************************************************
         *          DISPOSITIVOS
         **********************************************************************/

        var reloadDevices = function (q) {
            $scope.model.devices = APIDevice.byQueue(q.id).$object;
            APIDevice.byQueueDeleted(q.id).then(function (resp) {
                $scope.model.deletedDevices = resp;
            });
        };

        $scope.deactivateDevice = function (id) {
            APIDevice.deactivate(id).then(function () {
                reloadDevices($scope.model.queue);
            });
        };

        $scope.activateDevice = function (id) {
            APIDevice.activate(id).then(function () {
                reloadDevices($scope.model.queue);
            });
        };

        $scope.openDeviceModal = function (id) {

            $scope.deviceId = id;
            var deviceWorkProfiles;
            var applications;

            APIApplication.byTenant(sessionService.getTenantId()).then(function (resp) {
                applications = resp;
                APIWorkProfile.byQueue($scope.form.queue.id).then(function (resp) {
                    deviceWorkProfiles = resp;
                    var deviceModal = $uibModal.open({
                        scope: $scope,
                        animation: true,
                        controller: 'DeviceModalController',
                        controllerAs: '$deviceModalCtrl',
                        templateUrl: 'views/tenant/queueadmin/device_modal.html',
                        backdrop: 'static',
                        keyboard: false,
                        size: 'lg',
                        resolve: {
                            device: function () {
                                var id = $scope.deviceId;
                                if (id) {
                                    return APIDevice.loadOne(id);
                                }
                            },
                            applications: function () {
                                return applications;
                            },
                            deviceWorkProfiles: function () {
                                return deviceWorkProfiles;
                            }
                        }
                    });

                    deviceModal.result.then(function (device) {
                        console.log("Device", device);
                        reloadDevices($scope.model.queue);
                        toastr.info("Dispositivo guardado correctamente", "Operación realizada");
                    });
                });
            });
        };

        $scope.deleteDevice = function (id) {

            var msg = 'Si elimina el dispositivo, el kiosko o display donde se mostraba el contenido dejará de funcionar. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                APIDevice.deleteOne(id).then(function () {
                    reloadDevices($scope.model.queue);
                    toastr.info("Dispositivo eliminado correctamente", "Operación realizada");
                });

            });
        };

        /*********************************************************************
         *          QUEUE SMS Text
         **********************************************************************/

        $scope.isQueueSMSEnabled = false;
        $scope.isQueueSMSTxtConf = false;

        $scope.$watch("form.queue.configuration", function (newValue, oldValue) {

            // en el caso de que se hay borrado de la configuracion de la tienda la opcion de configurar los SMS,
            // pero hay textos introducidos para los mensajes
            // se van a borrar los valores
            if (newValue != oldValue) {

                var oldValueSms = _.find(oldValue, {id: 10001});
                var newValueSms = _.find(newValue, {id: 10001});

                if (newValueSms !== undefined) {
                    $scope.isQueueSMSEnabled = true;
                }

                if (oldValueSms !== undefined && newValueSms === undefined) {
                    if ($scope.form.queue.ticketSmsMsg !== undefined && $scope.form.queue.ticketSmsMsg != "") {
                        $scope.form.queue.ticketSmsMsg = null;
                    }
                    if ($scope.form.queue.noticeSmsMsg !== undefined && $scope.form.queue.noticeSmsMsg != "") {
                        $scope.form.queue.noticeSmsMsg = null;
                    }
                    $scope.isQueueSMSEnabled = false;
                    $scope.isQueueSMSTxtConf = false;
                }
            }
        });

        $scope.$watch("[form.queue.ticketSmsMsg, form.queue.noticeSmsMsg]", function (newValue) {
            if ($scope.isQueueSMSEnabled) {
                if ((newValue[0] === undefined || newValue[0] === "") && (newValue[1] === undefined || newValue[1] === "")) {
                    $scope.isQueueSMSTxtConf = false;
                } else {
                    $scope.isQueueSMSTxtConf = true;
                }
            } else {
                $scope.isQueueSMSTxtConf = false;
            }
        });
    });


/**
 * Controlador de la ventana modal para gestionar dispositivos
 */
angular.module('bewin_orange')
    .controller('DeviceModalController', function ($scope, $uibModalInstance, device, applications, deviceWorkProfiles, APIDevice) {

        var PRECONDITION_FAILED = 412;

        $scope.form = {
            device: device
        };

        $scope.form.errorMessage = null;
        $scope.applications = applications;
        $scope.selectedItems = [];
        $scope.availableItems = deviceWorkProfiles;

        if (device != null) {
            $scope.form.device.application = device.application;

            if (device.workProfiles != null && device.workProfiles.length > 0) {
                $scope.selectedItems = device.workProfiles;

                var diff = $scope.availableItems.filter(function (item) {
                    return !_.find($scope.selectedItems, ['id', item.id]);
                });
                $scope.availableItems = diff;
            }
        }

        $scope.save = function () {
            $scope.form.device.workProfiles = $scope.selectedItems;
            $scope.form.device.queue = $scope.model.queue;
            APIDevice.update($scope.form.device).then(
                function (device) {
                    $uibModalInstance.close(device);
                },
                function (res) {
                    if (res.status === PRECONDITION_FAILED) {
                        $scope.form.errorMessage = "Ya existe un dispositivo con ese nombre.";
                    }
                }
            );
        };

        $scope.cancel = function () {
            $uibModalInstance.dismiss('cancel');
        };

        $scope.moveItem = function (item, from, to) {
            var idx = from.indexOf(item);
            if (idx !== -1) {
                from.splice(idx, 1);
                to.push(item);
            }
        };

        $scope.moveItems = function (items, from, to) {
            if (items && items.length) {
                for (var i = 0; i < items.length; i++) {
                    var item = items [i];
                    $scope.moveItem(item, from, to);
                }
            }
        };

        $scope.moveAll = function (from, to) {
            angular.forEach(from, function (item) {
                to.push(item);
            });
            from.length = 0;
        };

    });

angular.module('bewin_orange')
    .controller('GoogleMaps', function ($scope) {

        $scope.mapOptions = {
            zoom: 11,
            center: new google.maps.LatLng(40.6700, -73.9400),
            // Style for Google Maps
            styles: [{
                "featureType": "water",
                "stylers": [{
                    "saturation": 43
                }, {
                    "lightness": -11
                }, {
                    "hue": "#0088ff"
                }]
            }, {
                "featureType": "road",
                "elementType": "geometry.fill",
                "stylers": [{
                    "hue": "#ff0000"
                }, {
                    "saturation": -100
                }, {
                    "lightness": 99
                }]
            }, {
                "featureType": "road",
                "elementType": "geometry.stroke",
                "stylers": [{
                    "color": "#808080"
                }, {
                    "lightness": 54
                }]
            }, {
                "featureType": "landscape.man_made",
                "elementType": "geometry.fill",
                "stylers": [{
                    "color": "#ece2d9"
                }]
            }, {
                "featureType": "poi.park",
                "elementType": "geometry.fill",
                "stylers": [{
                    "color": "#ccdca1"
                }]
            }, {
                "featureType": "road",
                "elementType": "labels.text.fill",
                "stylers": [{
                    "color": "#767676"
                }]
            }, {
                "featureType": "road",
                "elementType": "labels.text.stroke",
                "stylers": [{
                    "color": "#ffffff"
                }]
            }, {
                "featureType": "poi",
                "stylers": [{
                    "visibility": "off"
                }]
            }, {
                "featureType": "landscape.natural",
                "elementType": "geometry.fill",
                "stylers": [{
                    "visibility": "on"
                }, {
                    "color": "#b8cb93"
                }]
            }, {
                "featureType": "poi.park",
                "stylers": [{
                    "visibility": "on"
                }]
            }, {
                "featureType": "poi.sports_complex",
                "stylers": [{
                    "visibility": "on"
                }]
            }, {
                "featureType": "poi.medical",
                "stylers": [{
                    "visibility": "on"
                }]
            }, {
                "featureType": "poi.business",
                "stylers": [{
                    "visibility": "simplified"
                }]
            }],
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        $scope.mapOptions2 = {
            zoom: 11,
            center: new google.maps.LatLng(40.6700, -73.9400),
            // Style for Google Maps
            styles: [{
                "featureType": "all",
                "elementType": "all",
                "stylers": [{
                    "invert_lightness": true
                }, {
                    "saturation": 10
                }, {
                    "lightness": 30
                }, {
                    "gamma": 0.5
                }, {
                    "hue": "#435158"
                }]
            }],
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        $scope.mapOptions3 = {
            center: new google.maps.LatLng(36.964645, -122.01523),
            zoom: 18,
            // Style for Google Maps
            MapTypeId: google.maps.MapTypeId.SATELLITE,
            styles: [{
                "featureType": "road",
                "elementType": "geometry",
                "stylers": [{
                    "visibility": "off"
                }]
            }, {
                "featureType": "poi",
                "elementType": "geometry",
                "stylers": [{
                    "visibility": "off"
                }]
            }, {
                "featureType": "landscape",
                "elementType": "geometry",
                "stylers": [{
                    "color": "#fffffa"
                }]
            }, {
                "featureType": "water",
                "stylers": [{
                    "lightness": 50
                }]
            }, {
                "featureType": "road",
                "elementType": "labels",
                "stylers": [{
                    "visibility": "off"
                }]
            }, {
                "featureType": "transit",
                "stylers": [{
                    "visibility": "off"
                }]
            }, {
                "featureType": "administrative",
                "elementType": "geometry",
                "stylers": [{
                    "lightness": 40
                }]
            }],
            mapTypeId: google.maps.MapTypeId.SATELLITE
        };
        $scope.mapOptions4 = {
            zoom: 11,
            center: new google.maps.LatLng(40.6700, -73.9400),
            // Style for Google Maps
            styles: [{
                "stylers": [{
                    "hue": "#d35826"
                }, {
                    "visibility": "on"
                }, {
                    "invert_lightness": true
                }, {
                    "saturation": 40
                }, {
                    "lightness": 10
                }]
            }],
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };

    });



