'use strict';

angular.module('bewin_orange')
    .controller('ServiceAdminController', function ($scope, $log, $stateParams, modalService, utilService, sessionService,
                                                    APIServicePoint, APITicketDesign, toastr, $http, APIGeneric) {
        $log.log("ServiceAdminController started ..");

        $scope.form = {
            filter: '',
            service: {}
        };

        $scope.model = {
            services: APIServicePoint.byTenant(sessionService.getTenantId()).$object,
            service: {},
            serviceConfiguration: [],
            ticketDesigns: APITicketDesign.byTenant(sessionService.getTenantId()).$object
        };

        $scope.seqResult = false;

        $scope.select = function (u) {

            if (u && u.id) {

                APIServicePoint.loadOne(u.id).then(function (response) {

                    $scope.model.service = response;
                    $scope.form.service = JSON.parse(JSON.stringify(response));
                    $scope.form.service.configuration = [];

                    if (response && response.configuration) {
                        response.configuration.map(function (f) {
                            var config = _.find($scope.model.serviceConfiguration, ['id', f.id]);
                            if (config !== undefined) {
                                $scope.form.service.configuration.push(config);
                            }
                        });

                        if (_.find($scope.form.service.configuration, {id: 10001}) !== undefined) {
                            $scope.isServiceSMSEnabled = true;
                        } else {
                            $scope.isServiceSMSEnabled = false;
                        }
                    }

                    if (typeof response.ticketDesign != 'undefined') {
                        $scope.form.service.ticketDesign = _.find($scope.model.ticketDesigns, ['id', response.ticketDesign.id]);
                    }

                    if (response.randomNum) {
                        $scope.form.service.ticketInitials = null;
                        $scope.form.service.minNumber = null;
                        $scope.form.service.maxNumber = null;
                    }

                    $log.log("$scope.form.service>", $scope.form.service);
                });
            } else {
                $scope.model.service = {};
                $scope.form.service = {};
            }
        };

        var loadAll = function () {
            APIServicePoint.byTenant(sessionService.getTenantId()).then(function (response) {
                $scope.model.services = response;
                $scope.cancel();
            });
        };

        $scope.cancel = function () {
            $scope.select({});
        };

        $scope.remove = function () {
            var msg = 'Se va a deshabilitar el servicio y desaparecerá de todos los perfiles de trabajo asociados.' +
                ' Además ya no se podrán solicitar turnos con el servicio. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;
            modalInstance.result.then(function () {
                APIServicePoint.deleteOne($scope.form.service.id).then(function (r) {
                    toastr.info("Servicio eliminado correctamente", "Operación realizada");
                    loadAll();
                    $scope.cancel();
                });
            });
        }

        $scope.save = function () {

            // si el boton de "random" para el numero de turno esta seleccionado no mandamos el resto de los datos
            if ($scope.form.service.randomNum) {
                $scope.form.service.ticketInitials = "";
                $scope.form.service.minNumber = 0;
                $scope.form.service.maxNumber = 0;
            }

            $log.log("$scope.form.service>", $scope.form.service);
            if (!$scope.form.service.tenant) {
                $scope.form.service.tenant = {
                    id: sessionService.getTenantId()
                }
            }

            delete $scope.form.service.authorities;

            APIServicePoint.update($scope.form.service).then(function (res) {
                $log.log("res>", res);
                toastr.info("Servicio actualizado correctamente", "Operación realizada");
                loadAll();
                $scope.select(res);
            });
        }

        $scope.seqSelected = false;

        $scope.isSequentialSelected = function () {
            $scope.seqSelected = false;
            if ($scope.form.service.randomNum === undefined) {
                $scope.seqSelected = false;
            } else {
                if ($scope.form.service.randomNum) {
                    $scope.seqSelected = false;
                } else {
                    $scope.seqSelected = true;
                }
            }
        };

        $scope.$watch('form.service.randomNum', function () {
            $scope.isSequentialSelected();
        });


        $scope.delSequentialVals = function () {
            /*
            $scope.form.service.ticketInitials = "";
            $scope.form.service.minNumber = "";
            $scope.form.service.maxNumber = "";
            */
            $scope.form.service.ticketInitials = null;
            $scope.form.service.minNumber = null;
            $scope.form.service.maxNumber = null;
            $scope.seqResult = false;
        };

        $scope.checkSequenceAvailability = function () {
            if (typeof $scope.form.service.ticketInitials != "undefined" && $scope.form.service.ticketInitials != null
                && typeof $scope.form.service.minNumber != "undefined" && $scope.form.service.minNumber != null
                && typeof $scope.form.service.maxNumber != "undefined" && $scope.form.service.maxNumber != null) {

                // GET /api/servicepoint/checkSequence?tenantId=1&letter=A&minNum=1&maxNum=100
                APIServicePoint.checkSequence(sessionService.getTenantId(), $scope.form.service.ticketInitials, $scope.form.service.minNumber, $scope.form.service.maxNumber).then(function (response) {
                    if (response.resultado == "KO") {
                        $scope.seqResult = true;
                    }
                    if (response.resultado == "OK") {
                        $scope.seqResult = false;
                    }
                });
            }
        };


        /*********************************************************************
         *
         *          QUEUE SMS Text
         *
         **********************************************************************/

        $scope.isServiceSMSEnabled = false;

        $scope.$watch('form.service.configuration', function (newValue, oldValue) {

            if (newValue != oldValue) {

                var oldValueSms = _.find(oldValue, {id: 10001});
                var newValueSms = _.find(newValue, {id: 10001});

                if (newValueSms !== undefined) {
                    $scope.isServiceSMSEnabled = true;
                }

                if (oldValueSms !== undefined && newValueSms === undefined) {
                    if ($scope.form.service.smsMessages !== undefined) {
                        if ($scope.form.service.smsMessages.ticketSmsMsg !== undefined && $scope.form.service.smsMessages.ticketSmsMsg != "") {
                            $scope.form.service.smsMessages.ticketSmsMsg = null;
                        }
                        if ($scope.form.service.smsMessages.noticeSmsMsg !== undefined && $scope.form.service.smsMessages.noticeSmsMsg != "") {
                            $scope.form.service.smsMessages.noticeSmsMsg = null;
                        }
                    }

                    $scope.isServiceSMSEnabled = false;
                }
            }
        });


        $scope.isTicketDesignSelected = false;
        $scope.$watch('form.service.ticketDesign', function (newValue) {
            if (newValue != null) {
                $scope.isTicketDesignSelected = true;
            } else {
                $scope.isTicketDesignSelected = false;
            }
        });

        var getServiceConfParams = function () {

            // mapeamos las opciones disponibles para configuración teniendo en cuenta las opciones del Tenant
            // por ejemplo, es disponible la opción "Enviar ticket por SMS" solo si el tenant tiene configurado el envio por SMS

            APIGeneric.serviceConfParams().then(function (resp) {

                var tServiceConfParams = resp.plain();

                if (!sessionService.selectedTenant.smsEnabled) {
                    $scope.model.serviceConfiguration = _.filter(tServiceConfParams, function (el) {
                        return el.id != 10001;
                    });
                } else {
                    $scope.model.serviceConfiguration = tServiceConfParams;
                }
            });
        };

        // Funcion a ejecutar al inicio del controlador
        var init = function () {
            getServiceConfParams();
        };

        init();

    });
