'use strict';

angular.module('bewin_orange')
    .controller('UserAdminController', function ($q, $scope, $log, $stateParams, utilService, sessionService, paginationService,
                                                 configurationService, modalService, APIUser, APIQueue, APIGeneric, toastr, Upload) {

        $scope.model = {
            queues: [],
            users: [],
            user: {},
            roles: APIGeneric.roles().$object,
            availableQueues: [],
            selectedQueues: []
        };

        $scope.form = {
            user: {},
            userImage: null,
            assignedRoles: [],
            assignedWorkProfiles: [],
            userName: "",
            queue: {},
            name: "",
            active: false,
            rol: $scope.model.roles[0],
            validateSearchParams: true,
            filter: [],
            secondFilter: []
        };

        $scope.initialUsers = [];

        $scope.pager = {};
        $scope.setPage = setPage;

        $scope.nameQ = function (q) {
            return q.name;
        };

        $scope.select = function (u) {
            var deferred = $q.defer();

            $scope.form.assignedWorkProfiles = [];
            $scope.form.assignedRoles = [];
            $scope.form.user.userImageUrl = null;
            $scope.form.userImage = null;

            if (u && u.id) {
                APIUser.loadOne(u.id).then(function (user) {

                    $scope.form.user = user;

                    var userForm = $scope.form.user;

                    if (userForm.roles) {
                        userForm.roles.map(function (f) {
                            if (!f || !f.id) {
                                return;
                            }
                            var role = _.find($scope.model.roles, ['id', f.id]);
                            if (role !== undefined) {
                                $scope.form.assignedRoles.push(role);
                            }
                        });
                    } else {
                        $scope.form.assignedRoles = [];
                    }

                    if (userForm.queues) {
                        $scope.model.selectedQueues = userForm.queues;
                        var diff = $scope.model.queues.filter(function (item) {
                            return !_.find($scope.model.selectedQueues, ['id', item.id]);
                        });
                        $scope.model.availableQueues = diff;
                    } else {
                        $scope.model.model.selectedQueues = [];
                    }

                    if (userForm.workProfiles) {
                        userForm.workProfiles.map(function (f) {
                            if (!f || !f.id) {
                                return;
                            } else {
                                var workProfile = _.find($scope.model.workProfiles, ['id', f.id]);
                                if (workProfile !== undefined) {
                                    $scope.form.assignedWorkProfiles.push(workProfile);
                                }
                            }
                        });
                    } else {
                        $scope.form.assignedWorkProfiles = [];
                    }

                    deferred.resolve(true);
                });
            } else {
                $scope.form.user = {};
                $scope.model.user = {};
                $scope.model.selectedQueues = [];
                APIQueue.byTenant(sessionService.getTenantId()).then(function (qs) {

                    $scope.model.queues = qs;
                    $scope.model.availableQueues = qs;
                    $scope.model.workProfiles = [];

                    _.each(qs, function (q) {
                        _.each(q.operationsProfile.workProfiles, function (w) {
                            $scope.model.workProfiles.push(w);
                        });
                    });

                    $scope.model.workProfiles = _.uniqBy($scope.model.workProfiles, "id");
                });

                deferred.resolve(false);
            }

            return deferred.promise;
        };

        $scope.search = function () {
            $scope.cancel();
            $scope.pager = {};
            if (!$scope.form.userName && (!$scope.form.queue || !$scope.form.queue.id)
                && (!$scope.form.rol || !$scope.form.rol.id) && !$scope.form.name && $scope.form.active === false) {

                $scope.form.validateSearchParams = false;
                $scope.model.users = [];
            } else {
                $scope.form.validateSearchParams = true;
                var params = {
                    queues_id: ($scope.form.queue && $scope.form.queue.id ? [$scope.form.queue.id] : null),
                    firstName: ($scope.form.userName && $scope.form.userName !== '' ? $scope.form.userName : null),
                    roles_id: ($scope.form.rol && $scope.form.rol.id ? $scope.form.rol.id : null),
                    name: ($scope.form.name && $scope.form.name !== '' ? $scope.form.name : null),
                    active: ($scope.form.active === true ? false : null)
                };
                APIUser.search(params).then(function (response) {
                    $scope.initialUsers = response;
                    $scope.setPage(1);
                });
            }
        };

        $scope.cancel = function () {
            $scope.select(null);
        };

        $scope.reactivate = function (user) {
            $scope.select(user).then(function (loadOk) {
                if (loadOk === true) {
                    $scope.form.user.active = true;
                    saveNewUser().then(function (loadOk) {

                    });
                }
            });
        };

        $scope.save = function () {

            //antes de guardar hacemos peticion de los datos del usuario por si han cambiado
            if (typeof $scope.form.user != 'undefined' && typeof $scope.form.user.id != 'undefined') {

                APIUser.byId($scope.form.user.id).then(function (resp) {

                    var user = resp.plain();
                    if (user.currentQueue !== undefined || user.currentStation !== undefined || user.currentWorkProfile !== undefined) {
                        $scope.showAvisoModal();
                    } else {
                        // si el usuario que está serviendo ticket no se puede actualizar
                        APIUser.servingTicket($scope.form.user.id).then(function (resp) {

                            if (typeof resp !== "undefined") {
                                $scope.showAvisoModal();
                            } else {
                                saveNewUser();
                            }
                        });
                    }
                });
            } else {
                saveNewUser();
            }
        };

        $scope.remove = function () {
            var msg = 'Se va a deshabilitar el usuario y ya no podrá trabajar en ninguna tienda. ¿Desea continuar?';
            modalService.open('views/common/modal_delete.html', 'DeleteWarningCtrl', msg, null);
            var modalInstance = modalService.modalInstance;

            modalInstance.result.then(function () {
                APIUser.deleteOne($scope.form.user.id).then(function (r) {
                    toastr.info("Usuario eliminado correctamente", "Operación realizada");
                    searchUsers();
                });
                $scope.cancel();
            });
        };

        $scope.showAvisoModal = function () {
            var msg = 'El usuario está logeado en el sistema, no se puede actualizar.';
            modalService.open('views/common/modal_warning.html', 'WarningCtrl', msg, null);
        };

        $scope.moveItem = function (item, from, to) {
            var idx = from.indexOf(item);
            if (idx !== -1) {
                from.splice(idx, 1);
                to.push(item);
            }
        };

        $scope.moveItems = function (items, from, to) {
            if (items && items.length) {
                for (var i = 0; i < items.length; i++) {
                    var item = items [i];
                    $scope.moveItem(item, from, to);
                }
            }
        };

        $scope.moveAll = function (from, to) {
            angular.forEach(from, function (item) {
                to.push(item);
            });
            from.length = 0;
        };

        $scope.cleanFilter = function () {
            $scope.form.filter = [];
        };

        $scope.cleanSecondFilter = function () {
            $scope.form.secondFilter = [];
        };

        var buildUserJson = function () {
            var userJson = $scope.form.user;

            delete userJson.authorities;
            delete userJson.username;
            delete userJson.route;
            delete userJson.reqParams;
            delete userJson.restangularized;
            delete userJson.fromServer;
            delete userJson.parentResource;
            delete userJson.restangularCollection;
            delete userJson.currentWorkProfile;
            delete userJson.currentQueue;
            delete userJson.currentStation;
            return userJson;
        };

        var saveNewUser = function () {
            var deferred = $q.defer();
            if (!$scope.form.user.tenant) {
                $scope.form.user.tenant = {
                    id: sessionService.getTenantId()
                };
            }

            $scope.form.user.queues = _.map($scope.model.selectedQueues, function (q) {
                if (q !== undefined)
                    return {id: q.id};
            });

            $scope.form.user.roles = _.map($scope.form.assignedRoles, function (r) {
                if (r !== undefined)
                    return {id: r.id};
            });

            $scope.form.user.workProfiles = _.map($scope.form.assignedWorkProfiles, function (r) {
                if (r !== undefined)
                    return {id: r.id};
            });

            var uploadUrl = configurationService.apiURL + "/user";
            if ($scope.form.user.id) {
                uploadUrl += "/" + $scope.form.user.id;
            }

            Upload.upload({
                url: uploadUrl,
                headers: {
                    'X-Auth-Token': sessionService.securityToken,
                    'Content-Type': 'multipart/form-data'
                },
                data: {"user": JSON.stringify(buildUserJson())},
                file: $scope.form.userImage
            }).then(function (response) {
                if (response) {
                    if (response.data) {
                        $scope.select(response.data);
                    } else {
                        $scope.select(response);
                    }
                    toastr.info("Usuario guardado correctamente", "¡Perfecto!");
                    deferred.resolve(true);
                }
            }, function (error) {
                var mensaje = "";
                if (typeof error.data.details.errorMessage != 'undefined') {
                    mensaje = error.data.details.errorMessage;
                } else {
                    mensaje = error.status + " (" + error.statusText + ")";
                }
                toastr.error("Error: " + mensaje);
            });

            return deferred.promise;
        };

        var init = function () {
            APIQueue.byTenant(sessionService.getTenantId()).then(function (qs) {

                $scope.model.queues = qs;
                $scope.model.availableQueues = qs;
                $scope.model.workProfiles = [];

                _.each(qs, function (q) {
                    _.each(q.operationsProfile.workProfiles, function (w) {
                        $scope.model.workProfiles.push(w);
                    });
                });

                $scope.model.workProfiles = _.uniqBy($scope.model.workProfiles, "id");
            });
        };

        function setPage(page) {
            if (page < 1 || page > $scope.pager.totalPages) {
                return;
            }

            // get pager object from service
            $scope.pager = paginationService.getPager($scope.initialUsers.length, page, 15);

            // get current page of items
            $scope.model.users = $scope.initialUsers.slice($scope.pager.startIndex, $scope.pager.endIndex + 1);
        }

        init();
    });
